#pragma once
#pragma GCC diagnostic ignored "-Wunused-function"

#include "bt/BehaviorTree.hpp"
#include "bt/Composite.hpp"
#include "bt/Node.hpp"

// composites
#include "bt/composites/Selector.hpp"
#include "bt/composites/Sequence.hpp"
