id ICON "./mrboom.ico"
1 VERSIONINFO
FILEVERSION     5,4,0,0
PRODUCTVERSION  5,4,0,0
BEGIN
  BLOCK "StringFileInfo"
  BEGIN
    BLOCK "080904E4"
    BEGIN
      VALUE "CompanyName", "Remdy Software"
      VALUE "FileDescription", "MrBoom"
      VALUE "FileVersion", "5.4"
      VALUE "InternalName", "mrboom"
      VALUE "LegalCopyright", "Remdy Software"
      VALUE "OriginalFilename", "MrBoom.exe"
      VALUE "ProductName", "MrBoom"
      VALUE "ProductVersion", "5.4"
    END
  END
  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x809, 1252
  END
END
