#ifndef __NX_GLSYM_H__
#define __NX_GLSYM_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef void *GLeglImageOES;
typedef void (APIENTRYP RGLSYMGLCLEARINDEXPROC) ( GLfloat c );
typedef void (APIENTRYP RGLSYMGLCLEARCOLORPROC) ( GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha );
typedef void (APIENTRYP RGLSYMGLCLEARPROC) ( GLbitfield mask );
typedef void (APIENTRYP RGLSYMGLINDEXMASKPROC) ( GLuint mask );
typedef void (APIENTRYP RGLSYMGLCOLORMASKPROC) ( GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha );
typedef void (APIENTRYP RGLSYMGLALPHAFUNCPROC) ( GLenum func, GLclampf ref );
typedef void (APIENTRYP RGLSYMGLBLENDFUNCPROC) ( GLenum sfactor, GLenum dfactor );
typedef void (APIENTRYP RGLSYMGLLOGICOPPROC) ( GLenum opcode );
typedef void (APIENTRYP RGLSYMGLCULLFACEPROC) ( GLenum mode );
typedef void (APIENTRYP RGLSYMGLFRONTFACEPROC) ( GLenum mode );
typedef void (APIENTRYP RGLSYMGLPOINTSIZEPROC) ( GLfloat size );
typedef void (APIENTRYP RGLSYMGLLINEWIDTHPROC) ( GLfloat width );
typedef void (APIENTRYP RGLSYMGLLINESTIPPLEPROC) ( GLint factor, GLushort pattern );
typedef void (APIENTRYP RGLSYMGLPOLYGONMODEPROC) ( GLenum face, GLenum mode );
typedef void (APIENTRYP RGLSYMGLPOLYGONOFFSETPROC) ( GLfloat factor, GLfloat units );
typedef void (APIENTRYP RGLSYMGLPOLYGONSTIPPLEPROC) ( const GLubyte *mask );
typedef void (APIENTRYP RGLSYMGLGETPOLYGONSTIPPLEPROC) ( GLubyte *mask );
typedef void (APIENTRYP RGLSYMGLEDGEFLAGPROC) ( GLboolean flag );
typedef void (APIENTRYP RGLSYMGLEDGEFLAGVPROC) ( const GLboolean *flag );
typedef void (APIENTRYP RGLSYMGLSCISSORPROC) ( GLint x, GLint y, GLsizei width, GLsizei height);
typedef void (APIENTRYP RGLSYMGLCLIPPLANEPROC) ( GLenum plane, const GLdouble *equation );
typedef void (APIENTRYP RGLSYMGLGETCLIPPLANEPROC) ( GLenum plane, GLdouble *equation );
typedef void (APIENTRYP RGLSYMGLDRAWBUFFERPROC) ( GLenum mode );
typedef void (APIENTRYP RGLSYMGLREADBUFFERPROC) ( GLenum mode );
typedef void (APIENTRYP RGLSYMGLENABLEPROC) ( GLenum cap );
typedef void (APIENTRYP RGLSYMGLDISABLEPROC) ( GLenum cap );
typedef GLboolean (APIENTRYP RGLSYMGLISENABLEDPROC) ( GLenum cap );
typedef void (APIENTRYP RGLSYMGLENABLECLIENTSTATEPROC) ( GLenum cap );
typedef void (APIENTRYP RGLSYMGLDISABLECLIENTSTATEPROC) ( GLenum cap );
typedef void (APIENTRYP RGLSYMGLGETBOOLEANVPROC) ( GLenum pname, GLboolean *params );
typedef void (APIENTRYP RGLSYMGLGETDOUBLEVPROC) ( GLenum pname, GLdouble *params );
typedef void (APIENTRYP RGLSYMGLGETFLOATVPROC) ( GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETINTEGERVPROC) ( GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLPUSHATTRIBPROC) ( GLbitfield mask );
typedef void (APIENTRYP RGLSYMGLPOPATTRIBPROC) ( void );
typedef void (APIENTRYP RGLSYMGLPUSHCLIENTATTRIBPROC) ( GLbitfield mask );
typedef void (APIENTRYP RGLSYMGLPOPCLIENTATTRIBPROC) ( void );
typedef GLint (APIENTRYP RGLSYMGLRENDERMODEPROC) ( GLenum mode );
typedef GLenum (APIENTRYP RGLSYMGLGETERRORPROC) ( void );
typedef const GLubyte * (APIENTRYP RGLSYMGLGETSTRINGPROC) ( GLenum name );
typedef void (APIENTRYP RGLSYMGLFINISHPROC) ( void );
typedef void (APIENTRYP RGLSYMGLFLUSHPROC) ( void );
typedef void (APIENTRYP RGLSYMGLHINTPROC) ( GLenum target, GLenum mode );
typedef void (APIENTRYP RGLSYMGLCLEARDEPTHPROC) ( GLclampd depth );
typedef void (APIENTRYP RGLSYMGLDEPTHFUNCPROC) ( GLenum func );
typedef void (APIENTRYP RGLSYMGLDEPTHMASKPROC) ( GLboolean flag );
typedef void (APIENTRYP RGLSYMGLDEPTHRANGEPROC) ( GLclampd near_val, GLclampd far_val );
typedef void (APIENTRYP RGLSYMGLCLEARACCUMPROC) ( GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha );
typedef void (APIENTRYP RGLSYMGLACCUMPROC) ( GLenum op, GLfloat value );
typedef void (APIENTRYP RGLSYMGLMATRIXMODEPROC) ( GLenum mode );
typedef void (APIENTRYP RGLSYMGLORTHOPROC) ( GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble near_val, GLdouble far_val );
typedef void (APIENTRYP RGLSYMGLFRUSTUMPROC) ( GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble near_val, GLdouble far_val );
typedef void (APIENTRYP RGLSYMGLVIEWPORTPROC) ( GLint x, GLint y, GLsizei width, GLsizei height );
typedef void (APIENTRYP RGLSYMGLPUSHMATRIXPROC) ( void );
typedef void (APIENTRYP RGLSYMGLPOPMATRIXPROC) ( void );
typedef void (APIENTRYP RGLSYMGLLOADIDENTITYPROC) ( void );
typedef void (APIENTRYP RGLSYMGLLOADMATRIXDPROC) ( const GLdouble *m );
typedef void (APIENTRYP RGLSYMGLLOADMATRIXFPROC) ( const GLfloat *m );
typedef void (APIENTRYP RGLSYMGLMULTMATRIXDPROC) ( const GLdouble *m );
typedef void (APIENTRYP RGLSYMGLMULTMATRIXFPROC) ( const GLfloat *m );
typedef void (APIENTRYP RGLSYMGLROTATEDPROC) ( GLdouble angle, GLdouble x, GLdouble y, GLdouble z );
typedef void (APIENTRYP RGLSYMGLROTATEFPROC) ( GLfloat angle, GLfloat x, GLfloat y, GLfloat z );
typedef void (APIENTRYP RGLSYMGLSCALEDPROC) ( GLdouble x, GLdouble y, GLdouble z );
typedef void (APIENTRYP RGLSYMGLSCALEFPROC) ( GLfloat x, GLfloat y, GLfloat z );
typedef void (APIENTRYP RGLSYMGLTRANSLATEDPROC) ( GLdouble x, GLdouble y, GLdouble z );
typedef void (APIENTRYP RGLSYMGLTRANSLATEFPROC) ( GLfloat x, GLfloat y, GLfloat z );
typedef GLboolean (APIENTRYP RGLSYMGLISLISTPROC) ( GLuint list );
typedef void (APIENTRYP RGLSYMGLDELETELISTSPROC) ( GLuint list, GLsizei range );
typedef GLuint (APIENTRYP RGLSYMGLGENLISTSPROC) ( GLsizei range );
typedef void (APIENTRYP RGLSYMGLNEWLISTPROC) ( GLuint list, GLenum mode );
typedef void (APIENTRYP RGLSYMGLENDLISTPROC) ( void );
typedef void (APIENTRYP RGLSYMGLCALLLISTPROC) ( GLuint list );
typedef void (APIENTRYP RGLSYMGLCALLLISTSPROC) ( GLsizei n, GLenum type, const GLvoid *lists );
typedef void (APIENTRYP RGLSYMGLLISTBASEPROC) ( GLuint base );
typedef void (APIENTRYP RGLSYMGLBEGINPROC) ( GLenum mode );
typedef void (APIENTRYP RGLSYMGLENDPROC) ( void );
typedef void (APIENTRYP RGLSYMGLVERTEX2DPROC) ( GLdouble x, GLdouble y );
typedef void (APIENTRYP RGLSYMGLVERTEX2FPROC) ( GLfloat x, GLfloat y );
typedef void (APIENTRYP RGLSYMGLVERTEX2IPROC) ( GLint x, GLint y );
typedef void (APIENTRYP RGLSYMGLVERTEX2SPROC) ( GLshort x, GLshort y );
typedef void (APIENTRYP RGLSYMGLVERTEX3DPROC) ( GLdouble x, GLdouble y, GLdouble z );
typedef void (APIENTRYP RGLSYMGLVERTEX3FPROC) ( GLfloat x, GLfloat y, GLfloat z );
typedef void (APIENTRYP RGLSYMGLVERTEX3IPROC) ( GLint x, GLint y, GLint z );
typedef void (APIENTRYP RGLSYMGLVERTEX3SPROC) ( GLshort x, GLshort y, GLshort z );
typedef void (APIENTRYP RGLSYMGLVERTEX4DPROC) ( GLdouble x, GLdouble y, GLdouble z, GLdouble w );
typedef void (APIENTRYP RGLSYMGLVERTEX4FPROC) ( GLfloat x, GLfloat y, GLfloat z, GLfloat w );
typedef void (APIENTRYP RGLSYMGLVERTEX4IPROC) ( GLint x, GLint y, GLint z, GLint w );
typedef void (APIENTRYP RGLSYMGLVERTEX4SPROC) ( GLshort x, GLshort y, GLshort z, GLshort w );
typedef void (APIENTRYP RGLSYMGLVERTEX2DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLVERTEX2FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLVERTEX2IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLVERTEX2SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLVERTEX3DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLVERTEX3FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLVERTEX3IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLVERTEX3SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLVERTEX4DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLVERTEX4FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLVERTEX4IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLVERTEX4SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLNORMAL3BPROC) ( GLbyte nx, GLbyte ny, GLbyte nz );
typedef void (APIENTRYP RGLSYMGLNORMAL3DPROC) ( GLdouble nx, GLdouble ny, GLdouble nz );
typedef void (APIENTRYP RGLSYMGLNORMAL3FPROC) ( GLfloat nx, GLfloat ny, GLfloat nz );
typedef void (APIENTRYP RGLSYMGLNORMAL3IPROC) ( GLint nx, GLint ny, GLint nz );
typedef void (APIENTRYP RGLSYMGLNORMAL3SPROC) ( GLshort nx, GLshort ny, GLshort nz );
typedef void (APIENTRYP RGLSYMGLNORMAL3BVPROC) ( const GLbyte *v );
typedef void (APIENTRYP RGLSYMGLNORMAL3DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLNORMAL3FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLNORMAL3IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLNORMAL3SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLINDEXDPROC) ( GLdouble c );
typedef void (APIENTRYP RGLSYMGLINDEXFPROC) ( GLfloat c );
typedef void (APIENTRYP RGLSYMGLINDEXIPROC) ( GLint c );
typedef void (APIENTRYP RGLSYMGLINDEXSPROC) ( GLshort c );
typedef void (APIENTRYP RGLSYMGLINDEXUBPROC) ( GLubyte c );
typedef void (APIENTRYP RGLSYMGLINDEXDVPROC) ( const GLdouble *c );
typedef void (APIENTRYP RGLSYMGLINDEXFVPROC) ( const GLfloat *c );
typedef void (APIENTRYP RGLSYMGLINDEXIVPROC) ( const GLint *c );
typedef void (APIENTRYP RGLSYMGLINDEXSVPROC) ( const GLshort *c );
typedef void (APIENTRYP RGLSYMGLINDEXUBVPROC) ( const GLubyte *c );
typedef void (APIENTRYP RGLSYMGLCOLOR3BPROC) ( GLbyte red, GLbyte green, GLbyte blue );
typedef void (APIENTRYP RGLSYMGLCOLOR3DPROC) ( GLdouble red, GLdouble green, GLdouble blue );
typedef void (APIENTRYP RGLSYMGLCOLOR3FPROC) ( GLfloat red, GLfloat green, GLfloat blue );
typedef void (APIENTRYP RGLSYMGLCOLOR3IPROC) ( GLint red, GLint green, GLint blue );
typedef void (APIENTRYP RGLSYMGLCOLOR3SPROC) ( GLshort red, GLshort green, GLshort blue );
typedef void (APIENTRYP RGLSYMGLCOLOR3UBPROC) ( GLubyte red, GLubyte green, GLubyte blue );
typedef void (APIENTRYP RGLSYMGLCOLOR3UIPROC) ( GLuint red, GLuint green, GLuint blue );
typedef void (APIENTRYP RGLSYMGLCOLOR3USPROC) ( GLushort red, GLushort green, GLushort blue );
typedef void (APIENTRYP RGLSYMGLCOLOR4BPROC) ( GLbyte red, GLbyte green, GLbyte blue, GLbyte alpha );
typedef void (APIENTRYP RGLSYMGLCOLOR4DPROC) ( GLdouble red, GLdouble green, GLdouble blue, GLdouble alpha );
typedef void (APIENTRYP RGLSYMGLCOLOR4FPROC) ( GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha );
typedef void (APIENTRYP RGLSYMGLCOLOR4IPROC) ( GLint red, GLint green, GLint blue, GLint alpha );
typedef void (APIENTRYP RGLSYMGLCOLOR4SPROC) ( GLshort red, GLshort green, GLshort blue, GLshort alpha );
typedef void (APIENTRYP RGLSYMGLCOLOR4UBPROC) ( GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha );
typedef void (APIENTRYP RGLSYMGLCOLOR4UIPROC) ( GLuint red, GLuint green, GLuint blue, GLuint alpha );
typedef void (APIENTRYP RGLSYMGLCOLOR4USPROC) ( GLushort red, GLushort green, GLushort blue, GLushort alpha );
typedef void (APIENTRYP RGLSYMGLCOLOR3BVPROC) ( const GLbyte *v );
typedef void (APIENTRYP RGLSYMGLCOLOR3DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLCOLOR3FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLCOLOR3IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLCOLOR3SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLCOLOR3UBVPROC) ( const GLubyte *v );
typedef void (APIENTRYP RGLSYMGLCOLOR3UIVPROC) ( const GLuint *v );
typedef void (APIENTRYP RGLSYMGLCOLOR3USVPROC) ( const GLushort *v );
typedef void (APIENTRYP RGLSYMGLCOLOR4BVPROC) ( const GLbyte *v );
typedef void (APIENTRYP RGLSYMGLCOLOR4DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLCOLOR4FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLCOLOR4IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLCOLOR4SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLCOLOR4UBVPROC) ( const GLubyte *v );
typedef void (APIENTRYP RGLSYMGLCOLOR4UIVPROC) ( const GLuint *v );
typedef void (APIENTRYP RGLSYMGLCOLOR4USVPROC) ( const GLushort *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD1DPROC) ( GLdouble s );
typedef void (APIENTRYP RGLSYMGLTEXCOORD1FPROC) ( GLfloat s );
typedef void (APIENTRYP RGLSYMGLTEXCOORD1IPROC) ( GLint s );
typedef void (APIENTRYP RGLSYMGLTEXCOORD1SPROC) ( GLshort s );
typedef void (APIENTRYP RGLSYMGLTEXCOORD2DPROC) ( GLdouble s, GLdouble t );
typedef void (APIENTRYP RGLSYMGLTEXCOORD2FPROC) ( GLfloat s, GLfloat t );
typedef void (APIENTRYP RGLSYMGLTEXCOORD2IPROC) ( GLint s, GLint t );
typedef void (APIENTRYP RGLSYMGLTEXCOORD2SPROC) ( GLshort s, GLshort t );
typedef void (APIENTRYP RGLSYMGLTEXCOORD3DPROC) ( GLdouble s, GLdouble t, GLdouble r );
typedef void (APIENTRYP RGLSYMGLTEXCOORD3FPROC) ( GLfloat s, GLfloat t, GLfloat r );
typedef void (APIENTRYP RGLSYMGLTEXCOORD3IPROC) ( GLint s, GLint t, GLint r );
typedef void (APIENTRYP RGLSYMGLTEXCOORD3SPROC) ( GLshort s, GLshort t, GLshort r );
typedef void (APIENTRYP RGLSYMGLTEXCOORD4DPROC) ( GLdouble s, GLdouble t, GLdouble r, GLdouble q );
typedef void (APIENTRYP RGLSYMGLTEXCOORD4FPROC) ( GLfloat s, GLfloat t, GLfloat r, GLfloat q );
typedef void (APIENTRYP RGLSYMGLTEXCOORD4IPROC) ( GLint s, GLint t, GLint r, GLint q );
typedef void (APIENTRYP RGLSYMGLTEXCOORD4SPROC) ( GLshort s, GLshort t, GLshort r, GLshort q );
typedef void (APIENTRYP RGLSYMGLTEXCOORD1DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD1FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD1IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD1SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD2DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD2FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD2IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD2SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD3DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD3FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD3IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD3SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD4DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD4FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD4IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLTEXCOORD4SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS2DPROC) ( GLdouble x, GLdouble y );
typedef void (APIENTRYP RGLSYMGLRASTERPOS2FPROC) ( GLfloat x, GLfloat y );
typedef void (APIENTRYP RGLSYMGLRASTERPOS2IPROC) ( GLint x, GLint y );
typedef void (APIENTRYP RGLSYMGLRASTERPOS2SPROC) ( GLshort x, GLshort y );
typedef void (APIENTRYP RGLSYMGLRASTERPOS3DPROC) ( GLdouble x, GLdouble y, GLdouble z );
typedef void (APIENTRYP RGLSYMGLRASTERPOS3FPROC) ( GLfloat x, GLfloat y, GLfloat z );
typedef void (APIENTRYP RGLSYMGLRASTERPOS3IPROC) ( GLint x, GLint y, GLint z );
typedef void (APIENTRYP RGLSYMGLRASTERPOS3SPROC) ( GLshort x, GLshort y, GLshort z );
typedef void (APIENTRYP RGLSYMGLRASTERPOS4DPROC) ( GLdouble x, GLdouble y, GLdouble z, GLdouble w );
typedef void (APIENTRYP RGLSYMGLRASTERPOS4FPROC) ( GLfloat x, GLfloat y, GLfloat z, GLfloat w );
typedef void (APIENTRYP RGLSYMGLRASTERPOS4IPROC) ( GLint x, GLint y, GLint z, GLint w );
typedef void (APIENTRYP RGLSYMGLRASTERPOS4SPROC) ( GLshort x, GLshort y, GLshort z, GLshort w );
typedef void (APIENTRYP RGLSYMGLRASTERPOS2DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS2FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS2IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS2SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS3DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS3FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS3IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS3SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS4DVPROC) ( const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS4FVPROC) ( const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS4IVPROC) ( const GLint *v );
typedef void (APIENTRYP RGLSYMGLRASTERPOS4SVPROC) ( const GLshort *v );
typedef void (APIENTRYP RGLSYMGLRECTDPROC) ( GLdouble x1, GLdouble y1, GLdouble x2, GLdouble y2 );
typedef void (APIENTRYP RGLSYMGLRECTFPROC) ( GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2 );
typedef void (APIENTRYP RGLSYMGLRECTIPROC) ( GLint x1, GLint y1, GLint x2, GLint y2 );
typedef void (APIENTRYP RGLSYMGLRECTSPROC) ( GLshort x1, GLshort y1, GLshort x2, GLshort y2 );
typedef void (APIENTRYP RGLSYMGLRECTDVPROC) ( const GLdouble *v1, const GLdouble *v2 );
typedef void (APIENTRYP RGLSYMGLRECTFVPROC) ( const GLfloat *v1, const GLfloat *v2 );
typedef void (APIENTRYP RGLSYMGLRECTIVPROC) ( const GLint *v1, const GLint *v2 );
typedef void (APIENTRYP RGLSYMGLRECTSVPROC) ( const GLshort *v1, const GLshort *v2 );
typedef void (APIENTRYP RGLSYMGLVERTEXPOINTERPROC) ( GLint size, GLenum type, GLsizei stride, const GLvoid *ptr );
typedef void (APIENTRYP RGLSYMGLNORMALPOINTERPROC) ( GLenum type, GLsizei stride, const GLvoid *ptr );
typedef void (APIENTRYP RGLSYMGLCOLORPOINTERPROC) ( GLint size, GLenum type, GLsizei stride, const GLvoid *ptr );
typedef void (APIENTRYP RGLSYMGLINDEXPOINTERPROC) ( GLenum type, GLsizei stride, const GLvoid *ptr );
typedef void (APIENTRYP RGLSYMGLTEXCOORDPOINTERPROC) ( GLint size, GLenum type, GLsizei stride, const GLvoid *ptr );
typedef void (APIENTRYP RGLSYMGLEDGEFLAGPOINTERPROC) ( GLsizei stride, const GLvoid *ptr );
typedef void (APIENTRYP RGLSYMGLGETPOINTERVPROC) ( GLenum pname, GLvoid **params );
typedef void (APIENTRYP RGLSYMGLARRAYELEMENTPROC) ( GLint i );
typedef void (APIENTRYP RGLSYMGLDRAWARRAYSPROC) ( GLenum mode, GLint first, GLsizei count );
typedef void (APIENTRYP RGLSYMGLDRAWELEMENTSPROC) ( GLenum mode, GLsizei count, GLenum type, const GLvoid *indices );
typedef void (APIENTRYP RGLSYMGLINTERLEAVEDARRAYSPROC) ( GLenum format, GLsizei stride, const GLvoid *pointer );
typedef void (APIENTRYP RGLSYMGLSHADEMODELPROC) ( GLenum mode );
typedef void (APIENTRYP RGLSYMGLLIGHTFPROC) ( GLenum light, GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLLIGHTIPROC) ( GLenum light, GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLLIGHTFVPROC) ( GLenum light, GLenum pname, const GLfloat *params );
typedef void (APIENTRYP RGLSYMGLLIGHTIVPROC) ( GLenum light, GLenum pname, const GLint *params );
typedef void (APIENTRYP RGLSYMGLGETLIGHTFVPROC) ( GLenum light, GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETLIGHTIVPROC) ( GLenum light, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLLIGHTMODELFPROC) ( GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLLIGHTMODELIPROC) ( GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLLIGHTMODELFVPROC) ( GLenum pname, const GLfloat *params );
typedef void (APIENTRYP RGLSYMGLLIGHTMODELIVPROC) ( GLenum pname, const GLint *params );
typedef void (APIENTRYP RGLSYMGLMATERIALFPROC) ( GLenum face, GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLMATERIALIPROC) ( GLenum face, GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLMATERIALFVPROC) ( GLenum face, GLenum pname, const GLfloat *params );
typedef void (APIENTRYP RGLSYMGLMATERIALIVPROC) ( GLenum face, GLenum pname, const GLint *params );
typedef void (APIENTRYP RGLSYMGLGETMATERIALFVPROC) ( GLenum face, GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETMATERIALIVPROC) ( GLenum face, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLCOLORMATERIALPROC) ( GLenum face, GLenum mode );
typedef void (APIENTRYP RGLSYMGLPIXELZOOMPROC) ( GLfloat xfactor, GLfloat yfactor );
typedef void (APIENTRYP RGLSYMGLPIXELSTOREFPROC) ( GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLPIXELSTOREIPROC) ( GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLPIXELTRANSFERFPROC) ( GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLPIXELTRANSFERIPROC) ( GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLPIXELMAPFVPROC) ( GLenum map, GLsizei mapsize, const GLfloat *values );
typedef void (APIENTRYP RGLSYMGLPIXELMAPUIVPROC) ( GLenum map, GLsizei mapsize, const GLuint *values );
typedef void (APIENTRYP RGLSYMGLPIXELMAPUSVPROC) ( GLenum map, GLsizei mapsize, const GLushort *values );
typedef void (APIENTRYP RGLSYMGLGETPIXELMAPFVPROC) ( GLenum map, GLfloat *values );
typedef void (APIENTRYP RGLSYMGLGETPIXELMAPUIVPROC) ( GLenum map, GLuint *values );
typedef void (APIENTRYP RGLSYMGLGETPIXELMAPUSVPROC) ( GLenum map, GLushort *values );
typedef void (APIENTRYP RGLSYMGLBITMAPPROC) ( GLsizei width, GLsizei height, GLfloat xorig, GLfloat yorig, GLfloat xmove, GLfloat ymove, const GLubyte *bitmap );
typedef void (APIENTRYP RGLSYMGLREADPIXELSPROC) ( GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid *pixels );
typedef void (APIENTRYP RGLSYMGLDRAWPIXELSPROC) ( GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *pixels );
typedef void (APIENTRYP RGLSYMGLCOPYPIXELSPROC) ( GLint x, GLint y, GLsizei width, GLsizei height, GLenum type );
typedef void (APIENTRYP RGLSYMGLSTENCILFUNCPROC) ( GLenum func, GLint ref, GLuint mask );
typedef void (APIENTRYP RGLSYMGLSTENCILMASKPROC) ( GLuint mask );
typedef void (APIENTRYP RGLSYMGLSTENCILOPPROC) ( GLenum fail, GLenum zfail, GLenum zpass );
typedef void (APIENTRYP RGLSYMGLCLEARSTENCILPROC) ( GLint s );
typedef void (APIENTRYP RGLSYMGLTEXGENDPROC) ( GLenum coord, GLenum pname, GLdouble param );
typedef void (APIENTRYP RGLSYMGLTEXGENFPROC) ( GLenum coord, GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLTEXGENIPROC) ( GLenum coord, GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLTEXGENDVPROC) ( GLenum coord, GLenum pname, const GLdouble *params );
typedef void (APIENTRYP RGLSYMGLTEXGENFVPROC) ( GLenum coord, GLenum pname, const GLfloat *params );
typedef void (APIENTRYP RGLSYMGLTEXGENIVPROC) ( GLenum coord, GLenum pname, const GLint *params );
typedef void (APIENTRYP RGLSYMGLGETTEXGENDVPROC) ( GLenum coord, GLenum pname, GLdouble *params );
typedef void (APIENTRYP RGLSYMGLGETTEXGENFVPROC) ( GLenum coord, GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETTEXGENIVPROC) ( GLenum coord, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLTEXENVFPROC) ( GLenum target, GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLTEXENVIPROC) ( GLenum target, GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLTEXENVFVPROC) ( GLenum target, GLenum pname, const GLfloat *params );
typedef void (APIENTRYP RGLSYMGLTEXENVIVPROC) ( GLenum target, GLenum pname, const GLint *params );
typedef void (APIENTRYP RGLSYMGLGETTEXENVFVPROC) ( GLenum target, GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETTEXENVIVPROC) ( GLenum target, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLTEXPARAMETERFPROC) ( GLenum target, GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLTEXPARAMETERIPROC) ( GLenum target, GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLTEXPARAMETERFVPROC) ( GLenum target, GLenum pname, const GLfloat *params );
typedef void (APIENTRYP RGLSYMGLTEXPARAMETERIVPROC) ( GLenum target, GLenum pname, const GLint *params );
typedef void (APIENTRYP RGLSYMGLGETTEXPARAMETERFVPROC) ( GLenum target, GLenum pname, GLfloat *params);
typedef void (APIENTRYP RGLSYMGLGETTEXPARAMETERIVPROC) ( GLenum target, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLGETTEXLEVELPARAMETERFVPROC) ( GLenum target, GLint level, GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETTEXLEVELPARAMETERIVPROC) ( GLenum target, GLint level, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLTEXIMAGE1DPROC) ( GLenum target, GLint level, GLint internalFormat, GLsizei width, GLint border, GLenum format, GLenum type, const GLvoid *pixels );
typedef void (APIENTRYP RGLSYMGLTEXIMAGE2DPROC) ( GLenum target, GLint level, GLint internalFormat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const GLvoid *pixels );
typedef void (APIENTRYP RGLSYMGLGETTEXIMAGEPROC) ( GLenum target, GLint level, GLenum format, GLenum type, GLvoid *pixels );
typedef void (APIENTRYP RGLSYMGLGENTEXTURESPROC) ( GLsizei n, GLuint *textures );
typedef void (APIENTRYP RGLSYMGLDELETETEXTURESPROC) ( GLsizei n, const GLuint *textures);
typedef void (APIENTRYP RGLSYMGLBINDTEXTUREPROC) ( GLenum target, GLuint texture );
typedef void (APIENTRYP RGLSYMGLPRIORITIZETEXTURESPROC) ( GLsizei n, const GLuint *textures, const GLclampf *priorities );
typedef GLboolean (APIENTRYP RGLSYMGLARETEXTURESRESIDENTPROC) ( GLsizei n, const GLuint *textures, GLboolean *residences );
typedef GLboolean (APIENTRYP RGLSYMGLISTEXTUREPROC) ( GLuint texture );
typedef void (APIENTRYP RGLSYMGLTEXSUBIMAGE1DPROC) ( GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, const GLvoid *pixels );
typedef void (APIENTRYP RGLSYMGLTEXSUBIMAGE2DPROC) ( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *pixels );
typedef void (APIENTRYP RGLSYMGLCOPYTEXIMAGE1DPROC) ( GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLint border );
typedef void (APIENTRYP RGLSYMGLCOPYTEXIMAGE2DPROC) ( GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border );
typedef void (APIENTRYP RGLSYMGLCOPYTEXSUBIMAGE1DPROC) ( GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width );
typedef void (APIENTRYP RGLSYMGLCOPYTEXSUBIMAGE2DPROC) ( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height );
typedef void (APIENTRYP RGLSYMGLMAP1DPROC) ( GLenum target, GLdouble u1, GLdouble u2, GLint stride, GLint order, const GLdouble *points );
typedef void (APIENTRYP RGLSYMGLMAP1FPROC) ( GLenum target, GLfloat u1, GLfloat u2, GLint stride, GLint order, const GLfloat *points );
typedef void (APIENTRYP RGLSYMGLMAP2DPROC) ( GLenum target, GLdouble u1, GLdouble u2, GLint ustride, GLint uorder, GLdouble v1, GLdouble v2, GLint vstride, GLint vorder, const GLdouble *points );
typedef void (APIENTRYP RGLSYMGLMAP2FPROC) ( GLenum target, GLfloat u1, GLfloat u2, GLint ustride, GLint uorder, GLfloat v1, GLfloat v2, GLint vstride, GLint vorder, const GLfloat *points );
typedef void (APIENTRYP RGLSYMGLGETMAPDVPROC) ( GLenum target, GLenum query, GLdouble *v );
typedef void (APIENTRYP RGLSYMGLGETMAPFVPROC) ( GLenum target, GLenum query, GLfloat *v );
typedef void (APIENTRYP RGLSYMGLGETMAPIVPROC) ( GLenum target, GLenum query, GLint *v );
typedef void (APIENTRYP RGLSYMGLEVALCOORD1DPROC) ( GLdouble u );
typedef void (APIENTRYP RGLSYMGLEVALCOORD1FPROC) ( GLfloat u );
typedef void (APIENTRYP RGLSYMGLEVALCOORD1DVPROC) ( const GLdouble *u );
typedef void (APIENTRYP RGLSYMGLEVALCOORD1FVPROC) ( const GLfloat *u );
typedef void (APIENTRYP RGLSYMGLEVALCOORD2DPROC) ( GLdouble u, GLdouble v );
typedef void (APIENTRYP RGLSYMGLEVALCOORD2FPROC) ( GLfloat u, GLfloat v );
typedef void (APIENTRYP RGLSYMGLEVALCOORD2DVPROC) ( const GLdouble *u );
typedef void (APIENTRYP RGLSYMGLEVALCOORD2FVPROC) ( const GLfloat *u );
typedef void (APIENTRYP RGLSYMGLMAPGRID1DPROC) ( GLint un, GLdouble u1, GLdouble u2 );
typedef void (APIENTRYP RGLSYMGLMAPGRID1FPROC) ( GLint un, GLfloat u1, GLfloat u2 );
typedef void (APIENTRYP RGLSYMGLMAPGRID2DPROC) ( GLint un, GLdouble u1, GLdouble u2, GLint vn, GLdouble v1, GLdouble v2 );
typedef void (APIENTRYP RGLSYMGLMAPGRID2FPROC) ( GLint un, GLfloat u1, GLfloat u2, GLint vn, GLfloat v1, GLfloat v2 );
typedef void (APIENTRYP RGLSYMGLEVALPOINT1PROC) ( GLint i );
typedef void (APIENTRYP RGLSYMGLEVALPOINT2PROC) ( GLint i, GLint j );
typedef void (APIENTRYP RGLSYMGLEVALMESH1PROC) ( GLenum mode, GLint i1, GLint i2 );
typedef void (APIENTRYP RGLSYMGLEVALMESH2PROC) ( GLenum mode, GLint i1, GLint i2, GLint j1, GLint j2 );
typedef void (APIENTRYP RGLSYMGLFOGFPROC) ( GLenum pname, GLfloat param );
typedef void (APIENTRYP RGLSYMGLFOGIPROC) ( GLenum pname, GLint param );
typedef void (APIENTRYP RGLSYMGLFOGFVPROC) ( GLenum pname, const GLfloat *params );
typedef void (APIENTRYP RGLSYMGLFOGIVPROC) ( GLenum pname, const GLint *params );
typedef void (APIENTRYP RGLSYMGLFEEDBACKBUFFERPROC) ( GLsizei size, GLenum type, GLfloat *buffer );
typedef void (APIENTRYP RGLSYMGLPASSTHROUGHPROC) ( GLfloat token );
typedef void (APIENTRYP RGLSYMGLSELECTBUFFERPROC) ( GLsizei size, GLuint *buffer );
typedef void (APIENTRYP RGLSYMGLINITNAMESPROC) ( void );
typedef void (APIENTRYP RGLSYMGLLOADNAMEPROC) ( GLuint name );
typedef void (APIENTRYP RGLSYMGLPUSHNAMEPROC) ( GLuint name );
typedef void (APIENTRYP RGLSYMGLPOPNAMEPROC) ( void );
typedef void (APIENTRYP RGLSYMGLDRAWRANGEELEMENTSPROC) ( GLenum mode, GLuint start,GLuint end, GLsizei count, GLenum type, const GLvoid *indices );
typedef void (APIENTRYP RGLSYMGLTEXIMAGE3DPROC) ( GLenum target, GLint level, GLint internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, const GLvoid *pixels );
typedef void (APIENTRYP RGLSYMGLTEXSUBIMAGE3DPROC) ( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, const GLvoid *pixels);
typedef void (APIENTRYP RGLSYMGLCOPYTEXSUBIMAGE3DPROC) ( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height );
typedef void (APIENTRYP RGLSYMGLCOLORTABLEPROC) ( GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid *table );
typedef void (APIENTRYP RGLSYMGLCOLORSUBTABLEPROC) ( GLenum target, GLsizei start, GLsizei count, GLenum format, GLenum type, const GLvoid *data );
typedef void (APIENTRYP RGLSYMGLCOLORTABLEPARAMETERIVPROC) (GLenum target, GLenum pname, const GLint *params);
typedef void (APIENTRYP RGLSYMGLCOLORTABLEPARAMETERFVPROC) (GLenum target, GLenum pname, const GLfloat *params);
typedef void (APIENTRYP RGLSYMGLCOPYCOLORSUBTABLEPROC) ( GLenum target, GLsizei start, GLint x, GLint y, GLsizei width );
typedef void (APIENTRYP RGLSYMGLCOPYCOLORTABLEPROC) ( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width );
typedef void (APIENTRYP RGLSYMGLGETCOLORTABLEPROC) ( GLenum target, GLenum format, GLenum type, GLvoid *table );
typedef void (APIENTRYP RGLSYMGLGETCOLORTABLEPARAMETERFVPROC) ( GLenum target, GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETCOLORTABLEPARAMETERIVPROC) ( GLenum target, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLBLENDEQUATIONPROC) ( GLenum mode );
typedef void (APIENTRYP RGLSYMGLBLENDCOLORPROC) ( GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha );
typedef void (APIENTRYP RGLSYMGLHISTOGRAMPROC) ( GLenum target, GLsizei width, GLenum internalformat, GLboolean sink );
typedef void (APIENTRYP RGLSYMGLRESETHISTOGRAMPROC) ( GLenum target );
typedef void (APIENTRYP RGLSYMGLGETHISTOGRAMPROC) ( GLenum target, GLboolean reset, GLenum format, GLenum type, GLvoid *values );
typedef void (APIENTRYP RGLSYMGLGETHISTOGRAMPARAMETERFVPROC) ( GLenum target, GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETHISTOGRAMPARAMETERIVPROC) ( GLenum target, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLMINMAXPROC) ( GLenum target, GLenum internalformat,GLboolean sink );
typedef void (APIENTRYP RGLSYMGLRESETMINMAXPROC) ( GLenum target );
typedef void (APIENTRYP RGLSYMGLGETMINMAXPROC) ( GLenum target, GLboolean reset, GLenum format, GLenum types, GLvoid *values );
typedef void (APIENTRYP RGLSYMGLGETMINMAXPARAMETERFVPROC) ( GLenum target, GLenum pname, GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETMINMAXPARAMETERIVPROC) ( GLenum target, GLenum pname, GLint *params );
typedef void (APIENTRYP RGLSYMGLCONVOLUTIONFILTER1DPROC) ( GLenum target,GLenum internalformat, GLsizei width, GLenum format, GLenum type,const GLvoid *image );
typedef void (APIENTRYP RGLSYMGLCONVOLUTIONFILTER2DPROC) ( GLenum target,GLenum internalformat, GLsizei width, GLsizei height, GLenum format,GLenum type, const GLvoid *image );
typedef void (APIENTRYP RGLSYMGLCONVOLUTIONPARAMETERFPROC) ( GLenum target, GLenum pname,GLfloat params );
typedef void (APIENTRYP RGLSYMGLCONVOLUTIONPARAMETERFVPROC) ( GLenum target, GLenum pname,const GLfloat *params );
typedef void (APIENTRYP RGLSYMGLCONVOLUTIONPARAMETERIPROC) ( GLenum target, GLenum pname,GLint params );
typedef void (APIENTRYP RGLSYMGLCONVOLUTIONPARAMETERIVPROC) ( GLenum target, GLenum pname,const GLint *params );
typedef void (APIENTRYP RGLSYMGLCOPYCONVOLUTIONFILTER1DPROC) ( GLenum target,GLenum internalformat, GLint x, GLint y, GLsizei width );
typedef void (APIENTRYP RGLSYMGLCOPYCONVOLUTIONFILTER2DPROC) ( GLenum target,GLenum internalformat, GLint x, GLint y, GLsizei width,GLsizei height);
typedef void (APIENTRYP RGLSYMGLGETCONVOLUTIONFILTERPROC) ( GLenum target, GLenum format,GLenum type, GLvoid *image );
typedef void (APIENTRYP RGLSYMGLGETCONVOLUTIONPARAMETERFVPROC) ( GLenum target, GLenum pname,GLfloat *params );
typedef void (APIENTRYP RGLSYMGLGETCONVOLUTIONPARAMETERIVPROC) ( GLenum target, GLenum pname,GLint *params );
typedef void (APIENTRYP RGLSYMGLSEPARABLEFILTER2DPROC) ( GLenum target,GLenum internalformat, GLsizei width, GLsizei height, GLenum format,GLenum type, const GLvoid *row, const GLvoid *column );
typedef void (APIENTRYP RGLSYMGLGETSEPARABLEFILTERPROC) ( GLenum target, GLenum format,GLenum type, GLvoid *row, GLvoid *column, GLvoid *span );
typedef void (APIENTRYP RGLSYMGLACTIVETEXTUREPROC) ( GLenum texture );
typedef void (APIENTRYP RGLSYMGLCLIENTACTIVETEXTUREPROC) ( GLenum texture );
typedef void (APIENTRYP RGLSYMGLCOMPRESSEDTEXIMAGE1DPROC) ( GLenum target, GLint level, GLenum internalformat, GLsizei width, GLint border, GLsizei imageSize, const GLvoid *data );
typedef void (APIENTRYP RGLSYMGLCOMPRESSEDTEXIMAGE2DPROC) ( GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, const GLvoid *data );
typedef void (APIENTRYP RGLSYMGLCOMPRESSEDTEXIMAGE3DPROC) ( GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, const GLvoid *data );
typedef void (APIENTRYP RGLSYMGLCOMPRESSEDTEXSUBIMAGE1DPROC) ( GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imageSize, const GLvoid *data );
typedef void (APIENTRYP RGLSYMGLCOMPRESSEDTEXSUBIMAGE2DPROC) ( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, const GLvoid *data );
typedef void (APIENTRYP RGLSYMGLCOMPRESSEDTEXSUBIMAGE3DPROC) ( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, const GLvoid *data );
typedef void (APIENTRYP RGLSYMGLGETCOMPRESSEDTEXIMAGEPROC) ( GLenum target, GLint lod, GLvoid *img );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1DPROC) ( GLenum target, GLdouble s );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1DVPROC) ( GLenum target, const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1FPROC) ( GLenum target, GLfloat s );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1FVPROC) ( GLenum target, const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1IPROC) ( GLenum target, GLint s );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1IVPROC) ( GLenum target, const GLint *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1SPROC) ( GLenum target, GLshort s );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1SVPROC) ( GLenum target, const GLshort *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2DPROC) ( GLenum target, GLdouble s, GLdouble t );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2DVPROC) ( GLenum target, const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2FPROC) ( GLenum target, GLfloat s, GLfloat t );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2FVPROC) ( GLenum target, const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2IPROC) ( GLenum target, GLint s, GLint t );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2IVPROC) ( GLenum target, const GLint *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2SPROC) ( GLenum target, GLshort s, GLshort t );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2SVPROC) ( GLenum target, const GLshort *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3DPROC) ( GLenum target, GLdouble s, GLdouble t, GLdouble r );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3DVPROC) ( GLenum target, const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3FPROC) ( GLenum target, GLfloat s, GLfloat t, GLfloat r );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3FVPROC) ( GLenum target, const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3IPROC) ( GLenum target, GLint s, GLint t, GLint r );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3IVPROC) ( GLenum target, const GLint *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3SPROC) ( GLenum target, GLshort s, GLshort t, GLshort r );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3SVPROC) ( GLenum target, const GLshort *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4DPROC) ( GLenum target, GLdouble s, GLdouble t, GLdouble r, GLdouble q );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4DVPROC) ( GLenum target, const GLdouble *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4FPROC) ( GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4FVPROC) ( GLenum target, const GLfloat *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4IPROC) ( GLenum target, GLint s, GLint t, GLint r, GLint q );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4IVPROC) ( GLenum target, const GLint *v );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4SPROC) ( GLenum target, GLshort s, GLshort t, GLshort r, GLshort q );
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4SVPROC) ( GLenum target, const GLshort *v );
typedef void (APIENTRYP RGLSYMGLLOADTRANSPOSEMATRIXDPROC) ( const GLdouble m[16] );
typedef void (APIENTRYP RGLSYMGLLOADTRANSPOSEMATRIXFPROC) ( const GLfloat m[16] );
typedef void (APIENTRYP RGLSYMGLMULTTRANSPOSEMATRIXDPROC) ( const GLdouble m[16] );
typedef void (APIENTRYP RGLSYMGLMULTTRANSPOSEMATRIXFPROC) ( const GLfloat m[16] );
typedef void (APIENTRYP RGLSYMGLSAMPLECOVERAGEPROC) ( GLclampf value, GLboolean invert );
typedef void (APIENTRYP RGLSYMGLACTIVETEXTUREARBPROC) (GLenum texture);
typedef void (APIENTRYP RGLSYMGLCLIENTACTIVETEXTUREARBPROC) (GLenum texture);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1DARBPROC) (GLenum target, GLdouble s);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1DVARBPROC) (GLenum target, const GLdouble *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1FARBPROC) (GLenum target, GLfloat s);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1FVARBPROC) (GLenum target, const GLfloat *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1IARBPROC) (GLenum target, GLint s);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1IVARBPROC) (GLenum target, const GLint *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1SARBPROC) (GLenum target, GLshort s);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD1SVARBPROC) (GLenum target, const GLshort *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2DARBPROC) (GLenum target, GLdouble s, GLdouble t);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2DVARBPROC) (GLenum target, const GLdouble *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2FARBPROC) (GLenum target, GLfloat s, GLfloat t);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2FVARBPROC) (GLenum target, const GLfloat *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2IARBPROC) (GLenum target, GLint s, GLint t);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2IVARBPROC) (GLenum target, const GLint *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2SARBPROC) (GLenum target, GLshort s, GLshort t);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD2SVARBPROC) (GLenum target, const GLshort *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3DARBPROC) (GLenum target, GLdouble s, GLdouble t, GLdouble r);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3DVARBPROC) (GLenum target, const GLdouble *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3FARBPROC) (GLenum target, GLfloat s, GLfloat t, GLfloat r);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3FVARBPROC) (GLenum target, const GLfloat *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3IARBPROC) (GLenum target, GLint s, GLint t, GLint r);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3IVARBPROC) (GLenum target, const GLint *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3SARBPROC) (GLenum target, GLshort s, GLshort t, GLshort r);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD3SVARBPROC) (GLenum target, const GLshort *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4DARBPROC) (GLenum target, GLdouble s, GLdouble t, GLdouble r, GLdouble q);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4DVARBPROC) (GLenum target, const GLdouble *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4FARBPROC) (GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4FVARBPROC) (GLenum target, const GLfloat *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4IARBPROC) (GLenum target, GLint s, GLint t, GLint r, GLint q);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4IVARBPROC) (GLenum target, const GLint *v);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4SARBPROC) (GLenum target, GLshort s, GLshort t, GLshort r, GLshort q);
typedef void (APIENTRYP RGLSYMGLMULTITEXCOORD4SVARBPROC) (GLenum target, const GLshort *v);
typedef void (APIENTRYP RGLSYMGLEGLIMAGETARGETTEXTURE2DOESPROC) (GLenum target, GLeglImageOES image);
typedef void (APIENTRYP RGLSYMGLEGLIMAGETARGETRENDERBUFFERSTORAGEOESPROC) (GLenum target, GLeglImageOES image);
typedef void (APIENTRYP RGLSYMGLBINDTEXTURESPROC) (GLuint first, GLsizei count, const GLuint *textures);

RGLSYMGLCLEARINDEXPROC glClearIndex;
RGLSYMGLCLEARCOLORPROC glClearColor;
RGLSYMGLCLEARPROC glClear;
RGLSYMGLINDEXMASKPROC glIndexMask;
RGLSYMGLCOLORMASKPROC glColorMask;
RGLSYMGLALPHAFUNCPROC glAlphaFunc;
RGLSYMGLBLENDFUNCPROC glBlendFunc;
RGLSYMGLLOGICOPPROC glLogicOp;
RGLSYMGLCULLFACEPROC glCullFace;
RGLSYMGLFRONTFACEPROC glFrontFace;
RGLSYMGLPOINTSIZEPROC glPointSize;
RGLSYMGLLINEWIDTHPROC glLineWidth;
RGLSYMGLLINESTIPPLEPROC glLineStipple;
RGLSYMGLPOLYGONMODEPROC glPolygonMode;
RGLSYMGLPOLYGONOFFSETPROC glPolygonOffset;
RGLSYMGLPOLYGONSTIPPLEPROC glPolygonStipple;
RGLSYMGLGETPOLYGONSTIPPLEPROC glGetPolygonStipple;
RGLSYMGLEDGEFLAGPROC glEdgeFlag;
RGLSYMGLEDGEFLAGVPROC glEdgeFlagv;
RGLSYMGLSCISSORPROC glScissor;
RGLSYMGLCLIPPLANEPROC glClipPlane;
RGLSYMGLGETCLIPPLANEPROC glGetClipPlane;
RGLSYMGLDRAWBUFFERPROC glDrawBuffer;
RGLSYMGLREADBUFFERPROC glReadBuffer;
RGLSYMGLENABLEPROC glEnable;
RGLSYMGLDISABLEPROC glDisable;
RGLSYMGLISENABLEDPROC glIsEnabled;
RGLSYMGLENABLECLIENTSTATEPROC glEnableClientState;
RGLSYMGLDISABLECLIENTSTATEPROC glDisableClientState;
RGLSYMGLGETBOOLEANVPROC glGetBooleanv;
RGLSYMGLGETDOUBLEVPROC glGetDoublev;
RGLSYMGLGETFLOATVPROC glGetFloatv;
RGLSYMGLGETINTEGERVPROC glGetIntegerv;
RGLSYMGLPUSHATTRIBPROC glPushAttrib;
RGLSYMGLPOPATTRIBPROC glPopAttrib;
RGLSYMGLPUSHCLIENTATTRIBPROC glPushClientAttrib;
RGLSYMGLPOPCLIENTATTRIBPROC glPopClientAttrib;
RGLSYMGLRENDERMODEPROC glRenderMode;
RGLSYMGLGETERRORPROC glGetError;
RGLSYMGLGETSTRINGPROC glGetString;
RGLSYMGLFINISHPROC glFinish;
RGLSYMGLFLUSHPROC glFlush;
RGLSYMGLHINTPROC glHint;
RGLSYMGLCLEARDEPTHPROC glClearDepth;
RGLSYMGLDEPTHFUNCPROC glDepthFunc;
RGLSYMGLDEPTHMASKPROC glDepthMask;
RGLSYMGLDEPTHRANGEPROC glDepthRange;
RGLSYMGLCLEARACCUMPROC glClearAccum;
RGLSYMGLACCUMPROC glAccum;
RGLSYMGLMATRIXMODEPROC glMatrixMode;
RGLSYMGLORTHOPROC glOrtho;
RGLSYMGLFRUSTUMPROC glFrustum;
RGLSYMGLVIEWPORTPROC glViewport;
RGLSYMGLPUSHMATRIXPROC glPushMatrix;
RGLSYMGLPOPMATRIXPROC glPopMatrix;
RGLSYMGLLOADIDENTITYPROC glLoadIdentity;
RGLSYMGLLOADMATRIXDPROC glLoadMatrixd;
RGLSYMGLLOADMATRIXFPROC glLoadMatrixf;
RGLSYMGLMULTMATRIXDPROC glMultMatrixd;
RGLSYMGLMULTMATRIXFPROC glMultMatrixf;
RGLSYMGLROTATEDPROC glRotated;
RGLSYMGLROTATEFPROC glRotatef;
RGLSYMGLSCALEDPROC glScaled;
RGLSYMGLSCALEFPROC glScalef;
RGLSYMGLTRANSLATEDPROC glTranslated;
RGLSYMGLTRANSLATEFPROC glTranslatef;
RGLSYMGLISLISTPROC glIsList;
RGLSYMGLDELETELISTSPROC glDeleteLists;
RGLSYMGLGENLISTSPROC glGenLists;
RGLSYMGLNEWLISTPROC glNewList;
RGLSYMGLENDLISTPROC glEndList;
RGLSYMGLCALLLISTPROC glCallList;
RGLSYMGLCALLLISTSPROC glCallLists;
RGLSYMGLLISTBASEPROC glListBase;
RGLSYMGLBEGINPROC glBegin;
RGLSYMGLENDPROC glEnd;
RGLSYMGLVERTEX2DPROC glVertex2d;
RGLSYMGLVERTEX2FPROC glVertex2f;
RGLSYMGLVERTEX2IPROC glVertex2i;
RGLSYMGLVERTEX2SPROC glVertex2s;
RGLSYMGLVERTEX3DPROC glVertex3d;
RGLSYMGLVERTEX3FPROC glVertex3f;
RGLSYMGLVERTEX3IPROC glVertex3i;
RGLSYMGLVERTEX3SPROC glVertex3s;
RGLSYMGLVERTEX4DPROC glVertex4d;
RGLSYMGLVERTEX4FPROC glVertex4f;
RGLSYMGLVERTEX4IPROC glVertex4i;
RGLSYMGLVERTEX4SPROC glVertex4s;
RGLSYMGLVERTEX2DVPROC glVertex2dv;
RGLSYMGLVERTEX2FVPROC glVertex2fv;
RGLSYMGLVERTEX2IVPROC glVertex2iv;
RGLSYMGLVERTEX2SVPROC glVertex2sv;
RGLSYMGLVERTEX3DVPROC glVertex3dv;
RGLSYMGLVERTEX3FVPROC glVertex3fv;
RGLSYMGLVERTEX3IVPROC glVertex3iv;
RGLSYMGLVERTEX3SVPROC glVertex3sv;
RGLSYMGLVERTEX4DVPROC glVertex4dv;
RGLSYMGLVERTEX4FVPROC glVertex4fv;
RGLSYMGLVERTEX4IVPROC glVertex4iv;
RGLSYMGLVERTEX4SVPROC glVertex4sv;
RGLSYMGLNORMAL3BPROC glNormal3b;
RGLSYMGLNORMAL3DPROC glNormal3d;
RGLSYMGLNORMAL3FPROC glNormal3f;
RGLSYMGLNORMAL3IPROC glNormal3i;
RGLSYMGLNORMAL3SPROC glNormal3s;
RGLSYMGLNORMAL3BVPROC glNormal3bv;
RGLSYMGLNORMAL3DVPROC glNormal3dv;
RGLSYMGLNORMAL3FVPROC glNormal3fv;
RGLSYMGLNORMAL3IVPROC glNormal3iv;
RGLSYMGLNORMAL3SVPROC glNormal3sv;
RGLSYMGLINDEXDPROC glIndexd;
RGLSYMGLINDEXFPROC glIndexf;
RGLSYMGLINDEXIPROC glIndexi;
RGLSYMGLINDEXSPROC glIndexs;
RGLSYMGLINDEXUBPROC glIndexub;
RGLSYMGLINDEXDVPROC glIndexdv;
RGLSYMGLINDEXFVPROC glIndexfv;
RGLSYMGLINDEXIVPROC glIndexiv;
RGLSYMGLINDEXSVPROC glIndexsv;
RGLSYMGLINDEXUBVPROC glIndexubv;
RGLSYMGLCOLOR3BPROC glColor3b;
RGLSYMGLCOLOR3DPROC glColor3d;
RGLSYMGLCOLOR3FPROC glColor3f;
RGLSYMGLCOLOR3IPROC glColor3i;
RGLSYMGLCOLOR3SPROC glColor3s;
RGLSYMGLCOLOR3UBPROC glColor3ub;
RGLSYMGLCOLOR3UIPROC glColor3ui;
RGLSYMGLCOLOR3USPROC glColor3us;
RGLSYMGLCOLOR4BPROC glColor4b;
RGLSYMGLCOLOR4DPROC glColor4d;
RGLSYMGLCOLOR4FPROC glColor4f;
RGLSYMGLCOLOR4IPROC glColor4i;
RGLSYMGLCOLOR4SPROC glColor4s;
RGLSYMGLCOLOR4UBPROC glColor4ub;
RGLSYMGLCOLOR4UIPROC glColor4ui;
RGLSYMGLCOLOR4USPROC glColor4us;
RGLSYMGLCOLOR3BVPROC glColor3bv;
RGLSYMGLCOLOR3DVPROC glColor3dv;
RGLSYMGLCOLOR3FVPROC glColor3fv;
RGLSYMGLCOLOR3IVPROC glColor3iv;
RGLSYMGLCOLOR3SVPROC glColor3sv;
RGLSYMGLCOLOR3UBVPROC glColor3ubv;
RGLSYMGLCOLOR3UIVPROC glColor3uiv;
RGLSYMGLCOLOR3USVPROC glColor3usv;
RGLSYMGLCOLOR4BVPROC glColor4bv;
RGLSYMGLCOLOR4DVPROC glColor4dv;
RGLSYMGLCOLOR4FVPROC glColor4fv;
RGLSYMGLCOLOR4IVPROC glColor4iv;
RGLSYMGLCOLOR4SVPROC glColor4sv;
RGLSYMGLCOLOR4UBVPROC glColor4ubv;
RGLSYMGLCOLOR4UIVPROC glColor4uiv;
RGLSYMGLCOLOR4USVPROC glColor4usv;
RGLSYMGLTEXCOORD1DPROC glTexCoord1d;
RGLSYMGLTEXCOORD1FPROC glTexCoord1f;
RGLSYMGLTEXCOORD1IPROC glTexCoord1i;
RGLSYMGLTEXCOORD1SPROC glTexCoord1s;
RGLSYMGLTEXCOORD2DPROC glTexCoord2d;
RGLSYMGLTEXCOORD2FPROC glTexCoord2f;
RGLSYMGLTEXCOORD2IPROC glTexCoord2i;
RGLSYMGLTEXCOORD2SPROC glTexCoord2s;
RGLSYMGLTEXCOORD3DPROC glTexCoord3d;
RGLSYMGLTEXCOORD3FPROC glTexCoord3f;
RGLSYMGLTEXCOORD3IPROC glTexCoord3i;
RGLSYMGLTEXCOORD3SPROC glTexCoord3s;
RGLSYMGLTEXCOORD4DPROC glTexCoord4d;
RGLSYMGLTEXCOORD4FPROC glTexCoord4f;
RGLSYMGLTEXCOORD4IPROC glTexCoord4i;
RGLSYMGLTEXCOORD4SPROC glTexCoord4s;
RGLSYMGLTEXCOORD1DVPROC glTexCoord1dv;
RGLSYMGLTEXCOORD1FVPROC glTexCoord1fv;
RGLSYMGLTEXCOORD1IVPROC glTexCoord1iv;
RGLSYMGLTEXCOORD1SVPROC glTexCoord1sv;
RGLSYMGLTEXCOORD2DVPROC glTexCoord2dv;
RGLSYMGLTEXCOORD2FVPROC glTexCoord2fv;
RGLSYMGLTEXCOORD2IVPROC glTexCoord2iv;
RGLSYMGLTEXCOORD2SVPROC glTexCoord2sv;
RGLSYMGLTEXCOORD3DVPROC glTexCoord3dv;
RGLSYMGLTEXCOORD3FVPROC glTexCoord3fv;
RGLSYMGLTEXCOORD3IVPROC glTexCoord3iv;
RGLSYMGLTEXCOORD3SVPROC glTexCoord3sv;
RGLSYMGLTEXCOORD4DVPROC glTexCoord4dv;
RGLSYMGLTEXCOORD4FVPROC glTexCoord4fv;
RGLSYMGLTEXCOORD4IVPROC glTexCoord4iv;
RGLSYMGLTEXCOORD4SVPROC glTexCoord4sv;
RGLSYMGLRASTERPOS2DPROC glRasterPos2d;
RGLSYMGLRASTERPOS2FPROC glRasterPos2f;
RGLSYMGLRASTERPOS2IPROC glRasterPos2i;
RGLSYMGLRASTERPOS2SPROC glRasterPos2s;
RGLSYMGLRASTERPOS3DPROC glRasterPos3d;
RGLSYMGLRASTERPOS3FPROC glRasterPos3f;
RGLSYMGLRASTERPOS3IPROC glRasterPos3i;
RGLSYMGLRASTERPOS3SPROC glRasterPos3s;
RGLSYMGLRASTERPOS4DPROC glRasterPos4d;
RGLSYMGLRASTERPOS4FPROC glRasterPos4f;
RGLSYMGLRASTERPOS4IPROC glRasterPos4i;
RGLSYMGLRASTERPOS4SPROC glRasterPos4s;
RGLSYMGLRASTERPOS2DVPROC glRasterPos2dv;
RGLSYMGLRASTERPOS2FVPROC glRasterPos2fv;
RGLSYMGLRASTERPOS2IVPROC glRasterPos2iv;
RGLSYMGLRASTERPOS2SVPROC glRasterPos2sv;
RGLSYMGLRASTERPOS3DVPROC glRasterPos3dv;
RGLSYMGLRASTERPOS3FVPROC glRasterPos3fv;
RGLSYMGLRASTERPOS3IVPROC glRasterPos3iv;
RGLSYMGLRASTERPOS3SVPROC glRasterPos3sv;
RGLSYMGLRASTERPOS4DVPROC glRasterPos4dv;
RGLSYMGLRASTERPOS4FVPROC glRasterPos4fv;
RGLSYMGLRASTERPOS4IVPROC glRasterPos4iv;
RGLSYMGLRASTERPOS4SVPROC glRasterPos4sv;
RGLSYMGLRECTDPROC glRectd;
RGLSYMGLRECTFPROC glRectf;
RGLSYMGLRECTIPROC glRecti;
RGLSYMGLRECTSPROC glRects;
RGLSYMGLRECTDVPROC glRectdv;
RGLSYMGLRECTFVPROC glRectfv;
RGLSYMGLRECTIVPROC glRectiv;
RGLSYMGLRECTSVPROC glRectsv;
RGLSYMGLVERTEXPOINTERPROC glVertexPointer;
RGLSYMGLNORMALPOINTERPROC glNormalPointer;
RGLSYMGLCOLORPOINTERPROC glColorPointer;
RGLSYMGLINDEXPOINTERPROC glIndexPointer;
RGLSYMGLTEXCOORDPOINTERPROC glTexCoordPointer;
RGLSYMGLEDGEFLAGPOINTERPROC glEdgeFlagPointer;
RGLSYMGLGETPOINTERVPROC glGetPointerv;
RGLSYMGLARRAYELEMENTPROC glArrayElement;
RGLSYMGLDRAWARRAYSPROC glDrawArrays;
RGLSYMGLDRAWELEMENTSPROC glDrawElements;
RGLSYMGLINTERLEAVEDARRAYSPROC glInterleavedArrays;
RGLSYMGLSHADEMODELPROC glShadeModel;
RGLSYMGLLIGHTFPROC glLightf;
RGLSYMGLLIGHTIPROC glLighti;
RGLSYMGLLIGHTFVPROC glLightfv;
RGLSYMGLLIGHTIVPROC glLightiv;
RGLSYMGLGETLIGHTFVPROC glGetLightfv;
RGLSYMGLGETLIGHTIVPROC glGetLightiv;
RGLSYMGLLIGHTMODELFPROC glLightModelf;
RGLSYMGLLIGHTMODELIPROC glLightModeli;
RGLSYMGLLIGHTMODELFVPROC glLightModelfv;
RGLSYMGLLIGHTMODELIVPROC glLightModeliv;
RGLSYMGLMATERIALFPROC glMaterialf;
RGLSYMGLMATERIALIPROC glMateriali;
RGLSYMGLMATERIALFVPROC glMaterialfv;
RGLSYMGLMATERIALIVPROC glMaterialiv;
RGLSYMGLGETMATERIALFVPROC glGetMaterialfv;
RGLSYMGLGETMATERIALIVPROC glGetMaterialiv;
RGLSYMGLCOLORMATERIALPROC glColorMaterial;
RGLSYMGLPIXELZOOMPROC glPixelZoom;
RGLSYMGLPIXELSTOREFPROC glPixelStoref;
RGLSYMGLPIXELSTOREIPROC glPixelStorei;
RGLSYMGLPIXELTRANSFERFPROC glPixelTransferf;
RGLSYMGLPIXELTRANSFERIPROC glPixelTransferi;
RGLSYMGLPIXELMAPFVPROC glPixelMapfv;
RGLSYMGLPIXELMAPUIVPROC glPixelMapuiv;
RGLSYMGLPIXELMAPUSVPROC glPixelMapusv;
RGLSYMGLGETPIXELMAPFVPROC glGetPixelMapfv;
RGLSYMGLGETPIXELMAPUIVPROC glGetPixelMapuiv;
RGLSYMGLGETPIXELMAPUSVPROC glGetPixelMapusv;
RGLSYMGLBITMAPPROC glBitmap;
RGLSYMGLREADPIXELSPROC glReadPixels;
RGLSYMGLDRAWPIXELSPROC glDrawPixels;
RGLSYMGLCOPYPIXELSPROC glCopyPixels;
RGLSYMGLSTENCILFUNCPROC glStencilFunc;
RGLSYMGLSTENCILMASKPROC glStencilMask;
RGLSYMGLSTENCILOPPROC glStencilOp;
RGLSYMGLCLEARSTENCILPROC glClearStencil;
RGLSYMGLTEXGENDPROC glTexGend;
RGLSYMGLTEXGENFPROC glTexGenf;
RGLSYMGLTEXGENIPROC glTexGeni;
RGLSYMGLTEXGENDVPROC glTexGendv;
RGLSYMGLTEXGENFVPROC glTexGenfv;
RGLSYMGLTEXGENIVPROC glTexGeniv;
RGLSYMGLGETTEXGENDVPROC glGetTexGendv;
RGLSYMGLGETTEXGENFVPROC glGetTexGenfv;
RGLSYMGLGETTEXGENIVPROC glGetTexGeniv;
RGLSYMGLTEXENVFPROC glTexEnvf;
RGLSYMGLTEXENVIPROC glTexEnvi;
RGLSYMGLTEXENVFVPROC glTexEnvfv;
RGLSYMGLTEXENVIVPROC glTexEnviv;
RGLSYMGLGETTEXENVFVPROC glGetTexEnvfv;
RGLSYMGLGETTEXENVIVPROC glGetTexEnviv;
RGLSYMGLTEXPARAMETERFPROC glTexParameterf;
RGLSYMGLTEXPARAMETERIPROC glTexParameteri;
RGLSYMGLTEXPARAMETERFVPROC glTexParameterfv;
RGLSYMGLTEXPARAMETERIVPROC glTexParameteriv;
RGLSYMGLGETTEXPARAMETERFVPROC glGetTexParameterfv;
RGLSYMGLGETTEXPARAMETERIVPROC glGetTexParameteriv;
RGLSYMGLGETTEXLEVELPARAMETERFVPROC glGetTexLevelParameterfv;
RGLSYMGLGETTEXLEVELPARAMETERIVPROC glGetTexLevelParameteriv;
RGLSYMGLTEXIMAGE1DPROC glTexImage1D;
RGLSYMGLTEXIMAGE2DPROC glTexImage2D;
RGLSYMGLGETTEXIMAGEPROC glGetTexImage;
RGLSYMGLGENTEXTURESPROC glGenTextures;
RGLSYMGLDELETETEXTURESPROC glDeleteTextures;
RGLSYMGLBINDTEXTUREPROC glBindTexture;
RGLSYMGLPRIORITIZETEXTURESPROC glPrioritizeTextures;
RGLSYMGLARETEXTURESRESIDENTPROC glAreTexturesResident;
RGLSYMGLISTEXTUREPROC glIsTexture;
RGLSYMGLTEXSUBIMAGE1DPROC glTexSubImage1D;
RGLSYMGLTEXSUBIMAGE2DPROC glTexSubImage2D;
RGLSYMGLCOPYTEXIMAGE1DPROC glCopyTexImage1D;
RGLSYMGLCOPYTEXIMAGE2DPROC glCopyTexImage2D;
RGLSYMGLCOPYTEXSUBIMAGE1DPROC glCopyTexSubImage1D;
RGLSYMGLCOPYTEXSUBIMAGE2DPROC glCopyTexSubImage2D;
RGLSYMGLMAP1DPROC glMap1d;
RGLSYMGLMAP1FPROC glMap1f;
RGLSYMGLMAP2DPROC glMap2d;
RGLSYMGLMAP2FPROC glMap2f;
RGLSYMGLGETMAPDVPROC glGetMapdv;
RGLSYMGLGETMAPFVPROC glGetMapfv;
RGLSYMGLGETMAPIVPROC glGetMapiv;
RGLSYMGLEVALCOORD1DPROC glEvalCoord1d;
RGLSYMGLEVALCOORD1FPROC glEvalCoord1f;
RGLSYMGLEVALCOORD1DVPROC glEvalCoord1dv;
RGLSYMGLEVALCOORD1FVPROC glEvalCoord1fv;
RGLSYMGLEVALCOORD2DPROC glEvalCoord2d;
RGLSYMGLEVALCOORD2FPROC glEvalCoord2f;
RGLSYMGLEVALCOORD2DVPROC glEvalCoord2dv;
RGLSYMGLEVALCOORD2FVPROC glEvalCoord2fv;
RGLSYMGLMAPGRID1DPROC glMapGrid1d;
RGLSYMGLMAPGRID1FPROC glMapGrid1f;
RGLSYMGLMAPGRID2DPROC glMapGrid2d;
RGLSYMGLMAPGRID2FPROC glMapGrid2f;
RGLSYMGLEVALPOINT1PROC glEvalPoint1;
RGLSYMGLEVALPOINT2PROC glEvalPoint2;
RGLSYMGLEVALMESH1PROC glEvalMesh1;
RGLSYMGLEVALMESH2PROC glEvalMesh2;
RGLSYMGLFOGFPROC glFogf;
RGLSYMGLFOGIPROC glFogi;
RGLSYMGLFOGFVPROC glFogfv;
RGLSYMGLFOGIVPROC glFogiv;
RGLSYMGLFEEDBACKBUFFERPROC glFeedbackBuffer;
RGLSYMGLPASSTHROUGHPROC glPassThrough;
RGLSYMGLSELECTBUFFERPROC glSelectBuffer;
RGLSYMGLINITNAMESPROC glInitNames;
RGLSYMGLLOADNAMEPROC glLoadName;
RGLSYMGLPUSHNAMEPROC glPushName;
RGLSYMGLPOPNAMEPROC glPopName;
RGLSYMGLDRAWRANGEELEMENTSPROC glDrawRangeElements;
RGLSYMGLTEXIMAGE3DPROC glTexImage3D;
RGLSYMGLTEXSUBIMAGE3DPROC glTexSubImage3D;
RGLSYMGLCOPYTEXSUBIMAGE3DPROC glCopyTexSubImage3D;
RGLSYMGLCOLORTABLEPROC glColorTable;
RGLSYMGLCOLORSUBTABLEPROC glColorSubTable;
RGLSYMGLCOLORTABLEPARAMETERIVPROC glColorTableParameteriv;
RGLSYMGLCOLORTABLEPARAMETERFVPROC glColorTableParameterfv;
RGLSYMGLCOPYCOLORSUBTABLEPROC glCopyColorSubTable;
RGLSYMGLCOPYCOLORTABLEPROC glCopyColorTable;
RGLSYMGLGETCOLORTABLEPROC glGetColorTable;
RGLSYMGLGETCOLORTABLEPARAMETERFVPROC glGetColorTableParameterfv;
RGLSYMGLGETCOLORTABLEPARAMETERIVPROC glGetColorTableParameteriv;
RGLSYMGLBLENDEQUATIONPROC glBlendEquation;
RGLSYMGLBLENDCOLORPROC glBlendColor;
RGLSYMGLHISTOGRAMPROC glHistogram;
RGLSYMGLRESETHISTOGRAMPROC glResetHistogram;
RGLSYMGLGETHISTOGRAMPROC glGetHistogram;
RGLSYMGLGETHISTOGRAMPARAMETERFVPROC glGetHistogramParameterfv;
RGLSYMGLGETHISTOGRAMPARAMETERIVPROC glGetHistogramParameteriv;
RGLSYMGLMINMAXPROC glMinmax;
RGLSYMGLRESETMINMAXPROC glResetMinmax;
RGLSYMGLGETMINMAXPROC glGetMinmax;
RGLSYMGLGETMINMAXPARAMETERFVPROC glGetMinmaxParameterfv;
RGLSYMGLGETMINMAXPARAMETERIVPROC glGetMinmaxParameteriv;
RGLSYMGLCONVOLUTIONFILTER1DPROC glConvolutionFilter1D;
RGLSYMGLCONVOLUTIONFILTER2DPROC glConvolutionFilter2D;
RGLSYMGLCONVOLUTIONPARAMETERFPROC glConvolutionParameterf;
RGLSYMGLCONVOLUTIONPARAMETERFVPROC glConvolutionParameterfv;
RGLSYMGLCONVOLUTIONPARAMETERIPROC glConvolutionParameteri;
RGLSYMGLCONVOLUTIONPARAMETERIVPROC glConvolutionParameteriv;
RGLSYMGLCOPYCONVOLUTIONFILTER1DPROC glCopyConvolutionFilter1D;
RGLSYMGLCOPYCONVOLUTIONFILTER2DPROC glCopyConvolutionFilter2D;
RGLSYMGLGETCONVOLUTIONFILTERPROC glGetConvolutionFilter;
RGLSYMGLGETCONVOLUTIONPARAMETERFVPROC glGetConvolutionParameterfv;
RGLSYMGLGETCONVOLUTIONPARAMETERIVPROC glGetConvolutionParameteriv;
RGLSYMGLSEPARABLEFILTER2DPROC glSeparableFilter2D;
RGLSYMGLGETSEPARABLEFILTERPROC glGetSeparableFilter;
RGLSYMGLACTIVETEXTUREPROC glActiveTexture;
RGLSYMGLCLIENTACTIVETEXTUREPROC glClientActiveTexture;
RGLSYMGLCOMPRESSEDTEXIMAGE1DPROC glCompressedTexImage1D;
RGLSYMGLCOMPRESSEDTEXIMAGE2DPROC glCompressedTexImage2D;
RGLSYMGLCOMPRESSEDTEXIMAGE3DPROC glCompressedTexImage3D;
RGLSYMGLCOMPRESSEDTEXSUBIMAGE1DPROC glCompressedTexSubImage1D;
RGLSYMGLCOMPRESSEDTEXSUBIMAGE2DPROC glCompressedTexSubImage2D;
RGLSYMGLCOMPRESSEDTEXSUBIMAGE3DPROC glCompressedTexSubImage3D;
RGLSYMGLGETCOMPRESSEDTEXIMAGEPROC glGetCompressedTexImage;
RGLSYMGLMULTITEXCOORD1DPROC glMultiTexCoord1d;
RGLSYMGLMULTITEXCOORD1DVPROC glMultiTexCoord1dv;
RGLSYMGLMULTITEXCOORD1FPROC glMultiTexCoord1f;
RGLSYMGLMULTITEXCOORD1FVPROC glMultiTexCoord1fv;
RGLSYMGLMULTITEXCOORD1IPROC glMultiTexCoord1i;
RGLSYMGLMULTITEXCOORD1IVPROC glMultiTexCoord1iv;
RGLSYMGLMULTITEXCOORD1SPROC glMultiTexCoord1s;
RGLSYMGLMULTITEXCOORD1SVPROC glMultiTexCoord1sv;
RGLSYMGLMULTITEXCOORD2DPROC glMultiTexCoord2d;
RGLSYMGLMULTITEXCOORD2DVPROC glMultiTexCoord2dv;
RGLSYMGLMULTITEXCOORD2FPROC glMultiTexCoord2f;
RGLSYMGLMULTITEXCOORD2FVPROC glMultiTexCoord2fv;
RGLSYMGLMULTITEXCOORD2IPROC glMultiTexCoord2i;
RGLSYMGLMULTITEXCOORD2IVPROC glMultiTexCoord2iv;
RGLSYMGLMULTITEXCOORD2SPROC glMultiTexCoord2s;
RGLSYMGLMULTITEXCOORD2SVPROC glMultiTexCoord2sv;
RGLSYMGLMULTITEXCOORD3DPROC glMultiTexCoord3d;
RGLSYMGLMULTITEXCOORD3DVPROC glMultiTexCoord3dv;
RGLSYMGLMULTITEXCOORD3FPROC glMultiTexCoord3f;
RGLSYMGLMULTITEXCOORD3FVPROC glMultiTexCoord3fv;
RGLSYMGLMULTITEXCOORD3IPROC glMultiTexCoord3i;
RGLSYMGLMULTITEXCOORD3IVPROC glMultiTexCoord3iv;
RGLSYMGLMULTITEXCOORD3SPROC glMultiTexCoord3s;
RGLSYMGLMULTITEXCOORD3SVPROC glMultiTexCoord3sv;
RGLSYMGLMULTITEXCOORD4DPROC glMultiTexCoord4d;
RGLSYMGLMULTITEXCOORD4DVPROC glMultiTexCoord4dv;
RGLSYMGLMULTITEXCOORD4FPROC glMultiTexCoord4f;
RGLSYMGLMULTITEXCOORD4FVPROC glMultiTexCoord4fv;
RGLSYMGLMULTITEXCOORD4IPROC glMultiTexCoord4i;
RGLSYMGLMULTITEXCOORD4IVPROC glMultiTexCoord4iv;
RGLSYMGLMULTITEXCOORD4SPROC glMultiTexCoord4s;
RGLSYMGLMULTITEXCOORD4SVPROC glMultiTexCoord4sv;
RGLSYMGLLOADTRANSPOSEMATRIXDPROC glLoadTransposeMatrixd;
RGLSYMGLLOADTRANSPOSEMATRIXFPROC glLoadTransposeMatrixf;
RGLSYMGLMULTTRANSPOSEMATRIXDPROC glMultTransposeMatrixd;
RGLSYMGLMULTTRANSPOSEMATRIXFPROC glMultTransposeMatrixf;
RGLSYMGLSAMPLECOVERAGEPROC glSampleCoverage;
RGLSYMGLACTIVETEXTUREARBPROC glActiveTextureARB;
RGLSYMGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB;
RGLSYMGLMULTITEXCOORD1DARBPROC glMultiTexCoord1dARB;
RGLSYMGLMULTITEXCOORD1DVARBPROC glMultiTexCoord1dvARB;
RGLSYMGLMULTITEXCOORD1FARBPROC glMultiTexCoord1fARB;
RGLSYMGLMULTITEXCOORD1FVARBPROC glMultiTexCoord1fvARB;
RGLSYMGLMULTITEXCOORD1IARBPROC glMultiTexCoord1iARB;
RGLSYMGLMULTITEXCOORD1IVARBPROC glMultiTexCoord1ivARB;
RGLSYMGLMULTITEXCOORD1SARBPROC glMultiTexCoord1sARB;
RGLSYMGLMULTITEXCOORD1SVARBPROC glMultiTexCoord1svARB;
RGLSYMGLMULTITEXCOORD2DARBPROC glMultiTexCoord2dARB;
RGLSYMGLMULTITEXCOORD2DVARBPROC glMultiTexCoord2dvARB;
RGLSYMGLMULTITEXCOORD2FARBPROC glMultiTexCoord2fARB;
RGLSYMGLMULTITEXCOORD2FVARBPROC glMultiTexCoord2fvARB;
RGLSYMGLMULTITEXCOORD2IARBPROC glMultiTexCoord2iARB;
RGLSYMGLMULTITEXCOORD2IVARBPROC glMultiTexCoord2ivARB;
RGLSYMGLMULTITEXCOORD2SARBPROC glMultiTexCoord2sARB;
RGLSYMGLMULTITEXCOORD2SVARBPROC glMultiTexCoord2svARB;
RGLSYMGLMULTITEXCOORD3DARBPROC glMultiTexCoord3dARB;
RGLSYMGLMULTITEXCOORD3DVARBPROC glMultiTexCoord3dvARB;
RGLSYMGLMULTITEXCOORD3FARBPROC glMultiTexCoord3fARB;
RGLSYMGLMULTITEXCOORD3FVARBPROC glMultiTexCoord3fvARB;
RGLSYMGLMULTITEXCOORD3IARBPROC glMultiTexCoord3iARB;
RGLSYMGLMULTITEXCOORD3IVARBPROC glMultiTexCoord3ivARB;
RGLSYMGLMULTITEXCOORD3SARBPROC glMultiTexCoord3sARB;
RGLSYMGLMULTITEXCOORD3SVARBPROC glMultiTexCoord3svARB;
RGLSYMGLMULTITEXCOORD4DARBPROC glMultiTexCoord4dARB;
RGLSYMGLMULTITEXCOORD4DVARBPROC glMultiTexCoord4dvARB;
RGLSYMGLMULTITEXCOORD4FARBPROC glMultiTexCoord4fARB;
RGLSYMGLMULTITEXCOORD4FVARBPROC glMultiTexCoord4fvARB;
RGLSYMGLMULTITEXCOORD4IARBPROC glMultiTexCoord4iARB;
RGLSYMGLMULTITEXCOORD4IVARBPROC glMultiTexCoord4ivARB;
RGLSYMGLMULTITEXCOORD4SARBPROC glMultiTexCoord4sARB;
RGLSYMGLMULTITEXCOORD4SVARBPROC glMultiTexCoord4svARB;
RGLSYMGLEGLIMAGETARGETTEXTURE2DOESPROC glEGLImageTargetTexture2DOES;
RGLSYMGLEGLIMAGETARGETRENDERBUFFERSTORAGEOESPROC glEGLImageTargetRenderbufferStorageOES;
RGLSYMGLBINDTEXTURESPROC glBindTextures;

#ifdef __cplusplus
}
#endif
#endif // __NX_GLSYM_H__
