#ifndef __LIBRETRO_SDK_CRT_STRING_H_
#define __LIBRETRO_SDK_CRT_STRING_H_

#include <stdio.h>

void *memcpy(void *dst, const void *src, size_t len);

void *memset(void *b, int c, size_t len);

#endif
