#ifndef _PS4_DEFINES_H
#define _PS4_DEFINES_H

#define PS4_MAX_ORBISPADS 16
#define PS4_MAX_PAD_PORT_TYPES 3

#define	ORBISPAD_L3		        0x00000002
#define	ORBISPAD_R3		        0x00000004
#define	ORBISPAD_OPTIONS	    0x00000008
#define	ORBISPAD_UP		        0x00000010
#define	ORBISPAD_RIGHT		    0x00000020
#define	ORBISPAD_DOWN		      0x00000040
#define	ORBISPAD_LEFT		      0x00000080
#define	ORBISPAD_L2		        0x00000100
#define	ORBISPAD_R2		        0x00000200
#define	ORBISPAD_L1		        0x00000400
#define	ORBISPAD_R1		        0x00000800
#define	ORBISPAD_TRIANGLE	    0x00001000
#define	ORBISPAD_CIRCLE		    0x00002000
#define	ORBISPAD_CROSS		    0x00004000
#define	ORBISPAD_SQUARE		    0x00008000
#define	ORBISPAD_TOUCH_PAD	  0x00100000
#define	ORBISPAD_INTERCEPTED	0x80000000

#define SceUID uint32_t
#define SceKernelStat OrbisKernelStat
#define SCE_KERNEL_PRIO_FIFO_DEFAULT 700
#define SCE_AUDIO_OUT_PORT_TYPE_MAIN   0
#define SCE_AUDIO_OUT_MODE_STEREO      1
#define SCE_MOUSE_BUTTON_PRIMARY 0x00000001
#define SCE_MOUSE_BUTTON_SECONDARY 0x00000002
#define SCE_MOUSE_BUTTON_OPTIONAL 0x00000004
#define SCE_MOUSE_BUTTON_INTERCEPTED 0x80000000
#define SCE_MOUSE_OPEN_PARAM_MERGED	0x01
#define SCE_MOUSE_PORT_TYPE_STANDARD 0
#define SCE_DBG_KEYBOARD_PORT_TYPE_STANDARD	0
#define SCE_USER_SERVICE_MAX_LOGIN_USERS 16
#define SCE_USER_SERVICE_USER_ID_INVALID 0xFFFFFFFF
#define SCE_ORBISPAD_ERROR_ALREADY_OPENED 0x80920004
#define SCE_PAD_PORT_TYPE_STANDARD 0
#define SCE_PAD_PORT_TYPE_SPECIAL	2
#define SCE_PAD_PORT_TYPE_REMOTE_CONTROL 16
#define SCE_KERNEL_PROT_CPU_RW 0x02
#define SCE_KERNEL_MAP_FIXED 0x10

#endif
