//         _________   _________     _________   _________   _________   _________
//      ___\______ /___\____   /_____\____   /___\_____  /___\_____  /___\______ /___
//      \_   |   |   _     |_____/\_   ____    _     |/    _     |/    _   |   |   _/
//       |___|___|___|_____|  sns  |___________|___________|___________|___|___|___|
//      ==[mr.boom]=====================================================[est. 1997]==
//
// Generated by asm2c https://github.com/frranck/asm2c
#include "mrboom.h"
#pragma GCC diagnostic ignored "-Woverlength-strings"
#pragma GCC diagnostic ignored "-Wunused-label"
#define duree_saut 64
#define duree_mort 32
#define ttp2 180
#define triche 0
#define nombre_de_vbl_avant_le_droit_de_poser_bombe2 (60*2)
#define invisibilite_totale 100
#define invinsibilite_bonus 750
#define duree_match 512
#define duree_match2 48
#define duree_match4 512
#define duree_match3 256
#define duree_match5 304
#define time_bouboule 5
#define pic_max 420
#define duree_conta 800
#define duree_draw2 500
#define duree_med2 1200
#define duree_vic2 1200
#define attente_avant_draw2 100
#define attente_avant_med2 100
#define temps_re_menu 15
#define resistance_au_debut_pour_un_dyna 0
#define info1 1
#define info2 1
#define info3 210
#define info4 3
#define taille_header_rec 32
#define taille_bonus_rec 256
#define taille_moire (((((((((2030080+(64000*26)))/4096))+1))*4096))-1)
#define aagb (0*320)
#define aagb2 (((0+38))*320)
#define aagb3 (((0+(38*2)))*320)
#define aagb4 (((0+(38*3)))*320)
#define aagbg ((0*320)+(33*3))
#define aagb2g ((((0+38))*320)+(33*3))
#define aagb3g ((((0+(38*2)))*320)+(33*3))
#define aagb4g ((((0+(38*3)))*320)+(33*3))
#define eaagb5a (((0+(38*4)))*320)
#define aagb5a 198
#define aagb5 (((0+(38*4)))*320)
#define taagb5 229
#define gb (89+(56*320))
#define gb2 ((89+(56*320))+(320*33))
#define gb3 ((89+(56*320))+((320*33)*2))
#define gb4 ((89+(56*320))+((320*33)*3))
#define agb (158*320)
#define agb2 (179*320)
#define agb5 (41+(17*320))
#define agb6 ((41+(17*320))+(33*320))
#define agb7 ((41+(17*320))+((33*320)*2))
#define bleuoio (80+(100*320))
#define machinerie (76+(148*320))
#define gb5 (144*320)
#define gb6 ((144*320)+(24*3))
#define gb7 ((144*320)+(24*6))
#define gb8 ((144*320)+(24*9))
#define gb9 (163*320)
#define of_f0 (72*320)
#define of_f1 ((72*320)+49)
#define of_f2 ((72*320)+(49*2))
#define of_f3 ((72*320)+(49*3))
#define of_f4 ((72*320)+(49*4))
#define of_f5 ((72*320)+(49*5))
#define of_f6 (105*320)
#define of_f7 ((107*320)+49)
#define of_f8 ((107*320)+(49*2))
#define of_f9 ((107*320)+(49*3))
#define of_f10 ((107*320)+(49*4))
#define of_f11 (((107*320)+49)+5)
#define of_f12 (((107*320)+49)+5)
#define nb_unite_donnee4 9
#define nb_sply 16
#define russe (36*640)
#define max_attente 25
#define max_attente4 5
#define min_attente 15
#define bombe_max 16
#define bombe_max2 16
#define taille_dune_info_bombe (6*4)
#define taille_bloc_the_total (1500-packet_header_size)
#define nb_max_ordy 10
#define touches_size (((7*8)+2)+1)
#define nb_dyna 8
#define packet_data_size (576-packet_header_size)
#define packet_header_size 30
#define taille_block_slave_menu ((((32*4)*8)+1)+(14*2))
#define lft 24
#define max_s 16
#define max_s2 8

void program(void) {
#ifndef FALCON
jmp_buf jmpbuffer;
#endif
void * dest;
void * src;
int i;
#ifdef INCLUDEMAIN
dest=NULL;src=NULL;i=0; //to avoid a warning.
#endif
if (m.executionFinished) goto moveToBackGround;
if (m.jumpToBackGround) {
m.jumpToBackGround = 0;
#ifdef MRBOOM
if (m.nosetjmp) m.stackPointer=0; // this an an hack to avoid setJmp in saved state.
if (m.nosetjmp==2) goto directjeu;
if (m.nosetjmp==1) goto directmenu;
#endif
RET;
}
R(JMP(_main));
//PROC num
num:
R(PUSH(16,(READDW(edx))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ecx))));
R(MOV(32,READDD(ebx),32,(dd)READDD(eax)));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,liste_de_machin)))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(READDD(esi), ds))));
errrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr:
R(MOV(16,READDW(eax),16,(dw)0));
rrtetrertertretertterert:
R(CMP(32,READDD(ebx),32,(dd)READDD(ecx)));
R(JB(reerrereerret));
R(SUB(32,READDD(ebx),32,(dd)READDD(ecx)));
R(INC(16,(READDW(eax))));
R(JMP(rrtetrertertretertterert));
reerrereerret:
R(PUSH(16,(READDW(eax))));
R(PUSH(16,(READDW(edx))));
R(ADD(8,READDBl(eax),8,(db)48));
R(MOV(8,READDBl(edx),8,(db)READDBl(eax)));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(POP(16,(READDW(eax))));
R(POP(16,(READDW(edx))));
R(ADD(32,READDD(esi),32,(dd)4));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(READDD(esi), ds))));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(reererreer));
R(JMP(errrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr));
reererreer:
R(MOV(8,READDBl(edx),8,(db)32));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(POP(32,(READDD(ecx))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(16,(READDW(edx))));
RET;

//PROC aff_adresse
aff_adresse:
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSHAD);
R(MOV(16,READDW(ecx),16,(dw)10));
dds:
CALL(hexa);
R(CMP(16,READDW(ecx),16,(dw)1));
R(JE(ertertertertert));
R(MOV(8,READDBl(edx),8,(db)46));
R(CMP(16,READDW(ecx),16,(dw)9));
R(JNE(reterertert));
R(MOV(8,READDBl(edx),8,(db)32));
reterertert:
R(CMP(16,READDW(ecx),16,(dw)3));
R(JNE(reterertertu));
R(MOV(8,READDBl(edx),8,(db)32));
reterertertu:
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(LOOP(dds));
ertertertertert:
R(MOV(8,READDBl(edx),8,(db)10));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(MOV(8,READDBl(edx),8,(db)13));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(POPAD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
RET;

//PROC hexa
hexa:
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(LODSB);
R(PUSH(16,(READDW(eax))));
R(SHR(16,READDW(eax),16,(dw)4));
R(MOVZX(32,READDD(ebx),16,(dw)READDW(eax)));
R(MOV(8,READDBl(edx),8,*((db *) realAddress((offsetof(struct Mem,trucs)+READDD(ebx)), cs))));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(POP(16,(READDW(eax))));
R(AND(8,READDBl(eax),8,(db)15));
R(MOVZX(32,READDD(ebx),16,(dw)READDW(eax)));
R(MOV(8,READDBl(edx),8,*((db *) realAddress((offsetof(struct Mem,trucs)+READDD(ebx)), cs))));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
RET;

//PROC printeax
printeax:
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,ascii)))));
R(MOV(8,READDBl(ecx),8,(db)8));
p1:
R(ROL(32,READDD(eax),32,(dd)4));
R(MOV(8,READDBl(ebx),8,(db)READDBl(eax)));
R(AND(8,READDBl(ebx),8,(db)15));
R(ADD(8,READDBl(ebx),8,(db)48));
R(CMP(8,READDBl(ebx),8,(db)57));
R(JNA(p2));
R(ADD(8,READDBl(ebx),8,(db)7));
p2:
R(MOV(8,*((db *) realAddress(READDD(edi), ds)),8,(db)READDBl(ebx)));
R(INC(32,(READDD(edi))));
R(DEC(8,(READDBl(ecx))));
R(JNZ(p1));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,ascii)))));
R(MOV(8,READDBh(eax),8,(db)9));
R(INT(33));
RET;

//PROC last_color
last_color:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ebx))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,READDD(edi),32,(dd)753664));
R(MOV(32,READDD(esi),32,(dd)753664));
R(MOV(8,READDBh(eax),8,(db)3));
R(MOV(8,READDBh(ebx),8,(db)0));
R(INT(16));
R(DEC(8,(READDBh(edx))));
R(SHR(16,READDW(edx),16,(dw)8));
R(AND(32,READDD(edx),32,(dd)255));
R(MOV(32,READDD(eax),32,(dd)READDD(edx)));
R(SHL(32,READDD(edx),32,(dd)7));
R(SHL(32,READDD(eax),32,(dd)5));
R(ADD(32,READDD(edx),32,(dd)READDD(eax)));
R(ADD(32,READDD(esi),32,(dd)READDD(edx)));
R(ADD(32,READDD(edi),32,(dd)READDD(edx)));
R(MOV(16,READDW(ecx),16,(dw)80));
R(POP(16,(READDW(ebx))));
nooon:
R(INC(32,(READDD(edi))));
R(INC(32,(READDD(esi))));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBh(ebx)));
R(AND(8,READDBl(eax),8,(db)240));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(STOSB);
R(DEC(16,(READDW(ecx))));
R(JNE(nooon));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC affiche_sprites
affiche_sprites:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,blow_what)))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,blow_what2)))));
R(MOV(32,READDD(ecx),32,(dd)14));
R(REP_MOVSW);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)5));
R(JNE(pas_nuages));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,detail), ds)),8,(db)1));
R(JE(pas_nuages));
CALL(gestion_nuage);
pas_nuages:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)2));
R(JNE(retrterterte5rtrze));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,detail), ds)),8,(db)1));
R(JE(retrterterte5rtrze));
CALL(noel2);
retrterterte5rtrze:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)8));
R(JNE(retrterterte5ertertertreteer));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,detail), ds)),8,(db)1));
R(JE(retrterterte5ertertertreteer));
CALL(animfoot_haut);
retrterterte5ertertertreteer:
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,briques)))));
R(LODSW);
R(OR(16,READDW(eax),16,(dw)READDW(eax)));
R(JZ(reertertertertertterrterte));
R(MOV(16,READDW(ecx),16,(dw)READDW(eax)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
libere:
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(LODSW);
R(MOV(16,READDW(ebx),16,(dw)READDW(eax)));
R(LODSW);
R(PUSH(32,(READDD(esi))));
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(esi),32,(dd)1582080));
R(ADD(32,READDD(esi),32,(dd)READDD(ebx)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), es)),8,(db)2));
R(JE(retterterertertertertertertertertertert));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), es)),8,(db)4));
R(JE(retterterertertertertertertertertertert));
R(OR(16,READDW(ebx),16,(dw)READDW(ebx)));
R(JNZ(retterterertertertertertertertertertert));
retrterterte:
CALL(aff_brique);
retterterertertertertertertertertertert2:
R(POP(16,(READDW(ds))));
R(POP(32,(READDD(esi))));
R(DEC(16,(READDW(ecx))));
R(JNZ(libere));
reertertertertertterrterte:
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,bombes)))));
R(LODSW);
R(OR(16,READDW(eax),16,(dw)READDW(eax)));
R(JZ(reertertertertertterrtertet));
R(MOV(16,READDW(ecx),16,(dw)READDW(eax)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
liberet:
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(LODSW);
R(MOV(16,READDW(ebx),16,(dw)READDW(eax)));
R(LODSW);
R(PUSH(32,(READDD(esi))));
R(MOV(32,READDD(esi),32,(dd)(1582080+(64000*5))));
R(ADD(32,READDD(esi),32,(dd)READDD(ebx)));
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(CMP(16,READDW(ebx),16,(dw)(320*172)));
R(JNB(reetrertertert2tyrtyryrtrtyrtyrty));
R(CMP(16,READDW(ebx),16,(dw)((320*16)+112)));
R(JE(reetrertertert2tyrtyryrtrtyrtyrty));
R(SPRITE_16_16);
reetrertertert2tyrtyryrtrtyrtyrty:
R(POP(16,(READDW(ds))));
R(POP(32,(READDD(esi))));
R(DEC(16,(READDW(ecx))));
R(JNZ(liberet));
reertertertertertterrtertet:
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,bombes)))));
R(LODSW);
R(OR(16,READDW(eax),16,(dw)READDW(eax)));
R(JZ(treertertertertertterrtertet));
R(MOV(16,READDW(ecx),16,(dw)READDW(eax)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
tliberet:
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(LODSW);
R(MOV(16,READDW(ebx),16,(dw)READDW(eax)));
R(LODSW);
R(PUSH(32,(READDD(esi))));
R(MOV(32,READDD(esi),32,(dd)(1582080+(64000*5))));
R(ADD(32,READDD(esi),32,(dd)READDD(ebx)));
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(CMP(16,READDW(ebx),16,(dw)(320*172)));
R(JNB(reetrertertert2));
R(CMP(16,READDW(ebx),16,(dw)((320*16)+112)));
R(JE(reetrertertert3));
reetrertertert:
R(POP(16,(READDW(ds))));
R(POP(32,(READDD(esi))));
R(DEC(16,(READDW(ecx))));
R(JNZ(tliberet));
treertertertertertterrtertet:
CALL(aff_ombres);
CALL(expere);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)2));
R(JNE(retrterterte5ertertert));
CALL(noel);
retrterterte5ertertert:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)7));
R(JNE(retrterterte5ertertertrete));
CALL(soucoupe);
retrterterte5ertertertrete:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)8));
R(JNE(retrterterte5ertertertretee));
CALL(animfoot_oblige);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,detail), ds)),8,(db)1));
R(JE(retrterterte5ertertertretee));
CALL(animfoot);
retrterterte5ertertertretee:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)4));
R(JNE(retrterterte5ertertertret));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,detail), ds)),8,(db)1));
R(JE(retrterterte5ertertertret));
CALL(feuillage);
retrterterte5ertertertret:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)6));
R(JNE(retrterterte5ertertertrett));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,detail), ds)),8,(db)1));
R(JE(retrterterte5ertertertrett));
CALL(puzzle2);
retrterterte5ertertertrett:
CALL(tv);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,detail), ds)),8,(db)1));
R(JE(retrterterte5ertertertretterrettereererettrr));
CALL(bdraw);
retrterterte5ertertertretterrettereererettrr:
CALL(horloge);
CALL(pauseur);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
tv:
RET;
ertrteertretterertrteertrte:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,inser_coin), ds))));
R(JNZ(rerteertertyrt));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,inser_coin), ds)),32,(dd)32));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,viseur_ic2), ds)),32,(dd)4));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,viseur_ic2), ds)),32,(dd)(4*4)));
R(JNE(rerteertertyrt));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_ic2), ds)),32,(dd)0));
rerteertertyrt:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,viseur_ic2), ds))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,offset_ic)+READDD(ebx)), ds))));
R(ADD(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,adder_inser_coin), ds))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,adder_inser_coin), ds)),32,(dd)0));
R(JE(ertertertertetrertertert));
R(SUB(32,read_dd(realAddress(offsetof(struct Mem,adder_inser_coin), ds)),32,(dd)320));
ertertertertetrertertert:
R(ADD(32,READDD(esi),32,(dd)(((1582080+(64000*4))+74)+(22*320))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+(((0*320)+256)))))));
R(MOV(32,READDD(ecx),32,(dd)67));
aff_spt2yuertertertrteertix6:
R(MOV(32,READDD(ebx),32,(dd)58));
aff_spt2yuconcertix6:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(aff_spt2rtrtyrtyrtyrtyrtyrtyrtyx6));
R(CMP(8,READDBl(eax),8,(db)1));
R(JE(aff_spt2yuertertertrteix6));
R(CMP(8,READDBl(eax),8,(db)156));
R(JE(aff_spt2yuertertertrteiex6));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
R(JMP(aff_spt2rtrtyrtyrtyrtyrtyrtyrtyx6));
aff_spt2yuertertertrteiex6:
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(READDD(edi), es))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,couleurssss)+READDD(ebx)), es))));
R(ADD(8,READDBl(eax),8,(db)93));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(JMP(aff_spt2rtrtyrtyrtyrtyrtyrtyrtyx6));
aff_spt2yuertertertrteix6:
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(READDD(edi), es))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,couleurssss)+READDD(ebx)), es))));
R(ADD(8,READDBl(eax),8,(db)0));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
aff_spt2rtrtyrtyrtyrtyrtyrtyrtyx6:
R(INC(32,(READDD(edi))));
R(DEC(32,(READDD(ebx))));
R(JNZ(aff_spt2yuconcertix6));
R(ADD(32,READDD(edi),32,(dd)(320-58)));
R(ADD(32,READDD(esi),32,(dd)(320-58)));
R(DEC(32,(READDD(ecx))));
R(JNZ(aff_spt2yuertertertrteertix6));
pas_en_a:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
bdraw:
R(CMP(16,read_dw(realAddress(offsetof(struct Mem,adder_bdraw), ds)),16,(dw)(50*320)));
R(JNE(reterreer233));
RET;
reterreer233:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,bdraw666), ds))));
R(SUB(8,READDBl(eax),8,(db)48));
R(SHL(32,READDD(eax),32,(dd)3));
R(MOV(32,READDD(esi),32,(dd)READDD(eax)));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(ADD(32,READDD(esi),32,(dd)(((1966080+(64000*22))+173)+(320*33))));
R(MOV(32,READDD(edi),32,(dd)(((1966080+(64000*22))+137)+(320*64))));
R(MOV(32,READDD(ecx),32,(dd)5));
aff_spt666yuertertertrteertix10:
R(MOV(32,READDD(ebx),32,(dd)8));
aff_spt666yuconcertix10:
R(MOVSB);
R(DEC(32,(READDD(ebx))));
R(JNZ(aff_spt666yuconcertix10));
R(ADD(32,READDD(edi),32,(dd)(320-8)));
R(ADD(32,READDD(esi),32,(dd)(320-8)));
R(DEC(32,(READDD(ecx))));
R(JNZ(aff_spt666yuertertertrteertix10));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bdraw666)+1), ds))));
R(SUB(8,READDBl(eax),8,(db)48));
R(SHL(32,READDD(eax),32,(dd)3));
R(MOV(32,READDD(esi),32,(dd)READDD(eax)));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(ADD(32,READDD(esi),32,(dd)(((1966080+(64000*22))+173)+(320*33))));
R(MOV(32,READDD(edi),32,(dd)((((1966080+(64000*22))+137)+8)+(320*64))));
R(MOV(32,READDD(ecx),32,(dd)5));
aff_spt666yuertertertrteertix13:
R(MOV(32,READDD(ebx),32,(dd)8));
aff_spt666yuconcertix13:
R(MOVSB);
R(DEC(32,(READDD(ebx))));
R(JNZ(aff_spt666yuconcertix13));
R(ADD(32,READDD(edi),32,(dd)(320-8)));
R(ADD(32,READDD(esi),32,(dd)(320-8)));
R(DEC(32,(READDD(ecx))));
R(JNZ(aff_spt666yuertertertrteertix13));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(XOR(32,READDD(esi),32,(dd)READDD(esi)));
R(MOV(16,READDW(esi),16,read_dw(realAddress(offsetof(struct Mem,adder_bdraw), ds))));
R(ADD(32,READDD(esi),32,(dd)(((1966080+(64000*22))+96)+(41*320))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,slowcpu), ds)),32,(dd)1));
R(JE(cunfalcon));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+8)))));
R(MOV(32,READDD(ecx),32,(dd)43));
aff_spt2yuertertertrteertix15:
R(MOV(32,READDD(ebx),32,(dd)78));
aff_spt2yuconcertix15:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(aff_spt2rtrtyrtyrtyrtyrtyrtyrtyx15));
R(CMP(8,READDBl(eax),8,(db)1));
R(JE(aff_spt2yuertertertrteix15));
R(CMP(8,READDBl(eax),8,(db)156));
R(JE(aff_spt2yuertertertrteiex15));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
R(JMP(aff_spt2rtrtyrtyrtyrtyrtyrtyrtyx15));
aff_spt2yuertertertrteiex15:
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(READDD(edi), es))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,couleurssss)+READDD(ebx)), es))));
R(ADD(8,READDBl(eax),8,(db)93));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(JMP(aff_spt2rtrtyrtyrtyrtyrtyrtyrtyx15));
aff_spt2yuertertertrteix15:
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(READDD(edi), es))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,couleurssss)+READDD(ebx)), es))));
R(ADD(8,READDBl(eax),8,(db)(11-31)));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
aff_spt2rtrtyrtyrtyrtyrtyrtyrtyx15:
R(INC(32,(READDD(edi))));
R(DEC(32,(READDD(ebx))));
R(JNZ(aff_spt2yuconcertix15));
R(ADD(32,READDD(edi),32,(dd)(320-78)));
R(ADD(32,READDD(esi),32,(dd)(320-78)));
R(DEC(32,(READDD(ecx))));
R(JNZ(aff_spt2yuertertertrteertix15));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
cunfalcon:
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+8)))));
R(SPRITE_TIMEOUT);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
pauseur:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds)),8,(db)0));
R(JE(pas_en_pause));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,offset_pause)+READDD(ebx)), ds))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(ADD(32,READDD(esi),32,(dd)1582080));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+((136+(((100-32))*320))))))));
R(SPRITE_64_46);
pas_en_pause:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
animfoot:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), ds))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(AND(32,READDD(ebx),32,(dd)496));
R(SHR(32,READDD(ebx),32,(dd)4));
R(MOV(32,READDD(esi),32,(dd)(1966080+(64000*24))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(8,READDBl(ecx),8,*((db *) realAddress((offsetof(struct Mem,offset_cameraman)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+(140*320))))));
R(SPRITE_23_21);
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(ADD(32,READDD(ebx),32,(dd)(8+16)));
R(AND(32,READDD(ebx),32,(dd)496));
R(SHR(32,READDD(ebx),32,(dd)4));
R(MOV(32,READDD(esi),32,(dd)(1966080+(64000*24))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(8,READDBl(ecx),8,*((db *) realAddress((offsetof(struct Mem,offset_cameraman)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(ADD(32,READDD(esi),32,(dd)(5*24)));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(25*320))+299)))));
R(SPRITE_23_21);
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(AND(32,READDD(ebx),32,(dd)240));
R(SHR(32,READDD(ebx),32,(dd)3));
R(MOV(32,READDD(esi),32,(dd)(1966080+(64000*24))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((offsetof(struct Mem,offset_fille)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(MOV(32,m.edi.dd.val,32,(((dd)(((offsetof(struct Mem,buffer)+(170*320))+140)+32)))));
R(SPRITE_30_48);
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(AND(32,READDD(ebx),32,(dd)240));
R(SHR(32,READDD(ebx),32,(dd)3));
R(MOV(32,READDD(esi),32,(dd)(1966080+(64000*24))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((offsetof(struct Mem,offset_fille)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(MOV(32,m.edi.dd.val,32,(((dd)((((offsetof(struct Mem,buffer)+(170*320))+140)+33)+32)))));
R(SPRITE_30_48);
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(AND(32,READDD(ebx),32,(dd)240));
R(SHR(32,READDD(ebx),32,(dd)3));
R(MOV(32,READDD(esi),32,(dd)(1966080+(64000*24))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((offsetof(struct Mem,offset_fille)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(MOV(32,m.edi.dd.val,32,(((dd)((((offsetof(struct Mem,buffer)+(170*320))+140)+66)+32)))));
R(SPRITE_30_48);
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(AND(32,READDD(ebx),32,(dd)240));
R(SHR(32,READDD(ebx),32,(dd)3));
R(MOV(32,READDD(esi),32,(dd)(1966080+(64000*24))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((offsetof(struct Mem,offset_fille)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(170*320))+10)))));
R(SPRITE_30_48);
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(AND(32,READDD(ebx),32,(dd)240));
R(SHR(32,READDD(ebx),32,(dd)3));
R(MOV(32,READDD(esi),32,(dd)(1966080+(64000*24))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((offsetof(struct Mem,offset_fille)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(170*320))+42)))));
R(SPRITE_30_48);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
animfoot_haut:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(AND(32,READDD(ebx),32,(dd)48));
R(SHR(32,READDD(ebx),32,(dd)4));
R(MOV(32,READDD(esi),32,(dd)(((1966080+(64000*24))+24)+(28*320))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(8,READDBl(ecx),8,*((db *) realAddress((offsetof(struct Mem,offset_supporter)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*2))+2)))));
R(SPRITE_19_23);
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), es))));
R(ADD(32,READDD(ebx),32,(dd)7));
R(AND(32,READDD(ebx),32,(dd)48));
R(SHR(32,READDD(ebx),32,(dd)4));
R(MOV(32,READDD(esi),32,(dd)(((1966080+(64000*24))+144)+(28*320))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(8,READDBl(ecx),8,*((db *) realAddress((offsetof(struct Mem,offset_supporter)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(ecx)));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*3))+295)))));
R(SPRITE_19_23);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
animfoot_oblige:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(esi),32,(dd)(((1966080+(64000*22))+11)+(52*320))));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+11)+(52*320))))));
R(SPRITE_77_12);
R(MOV(32,READDD(esi),32,(dd)(((1966080+(64000*22))+297)+(52*320))));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+297)+(52*320))))));
R(SPRITE_77_12);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
soucoupe:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), ds))));
R(XOR(32,READDD(esi),32,(dd)READDD(esi)));
R(AND(32,READDD(ebx),32,(dd)16));
R(JNZ(mlkjmljktrjklmrejklmtrjklmjklmteerrtrtrrttrrttre));
R(MOV(32,READDD(esi),32,(dd)232));
mlkjmljktrjklmrejklmtrjklmjklmteerrtrtrrttrrttre:
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(ADD(32,READDD(esi),32,(dd)((1966080+(64000*22))+(133*320))));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+232)))));
R(SPRITE_36_88);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
feuillage:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(esi),32,(dd)(((((896000+384000)+46080)+64000)+64000)+11)));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+11)))));
R(SPRITE_192_21);
R(MOV(32,READDD(esi),32,(dd)(((((896000+384000)+46080)+64000)+64000)+288)));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+288)))));
R(SPRITE_192_21);
R(MOV(32,READDD(esi),32,(dd)((((((896000+384000)+46080)+64000)+64000)+36)-6)));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+30)))));
R(SPRITE_16_263);
R(MOV(32,READDD(esi),32,(dd)((((((896000+384000)+46080)+64000)+64000)+69)+(166*320))));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+69)+(166*320))))));
R(SPRITE_26_206);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
puzzle2:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(esi),32,(dd)((1582080+(64000*4))+51)));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+51)))));
R(SPRITE_16_187);
R(MOV(32,READDD(esi),32,(dd)((1582080+(64000*4))+16)));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+16)))));
R(SPRITE_191_16);
R(MOV(32,READDD(esi),32,(dd)((1582080+(64000*4))+288)));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+288)))));
R(SPRITE_92_17);
R(MOV(32,READDD(esi),32,(dd)(((1582080+(64000*4))+288)+(107*320))));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+288)+(107*320))))));
R(SPRITE_85_17);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
gestion_nuage:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), ds))));
R(MOV(32,READDD(ecx),32,(dd)16));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nuage_sympa)))));
teryrtyrtyuuiyuyu:
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+16), ds))));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,compteur_nuage), ds)),32,(dd)READDD(eax)));
R(JNZ(pas_cette_fois));
R(INC(32,read_dd(realAddress((READDD(esi)+4), ds))));
pas_cette_fois:
R(MOV(32,READDD(eax),32,read_dd(realAddress(READDD(esi), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((READDD(esi)+8), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)READDD(ebx)));
R(JB(ertrteterertert));
R(MOV(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)0));
ertrteterertert:
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)319));
R(JA(ertertrteterertert));
R(SUB(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+4), ds))));
R(MOV(32,READDD(edx),32,read_dd(realAddress((READDD(esi)+12), ds))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(esi),32,(dd)(1582080+(64000*5))));
R(ADD(32,READDD(esi),32,(dd)READDD(edx)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(SPRITE_CLOUD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
ertertrteterertert:
R(ADD(32,READDD(esi),32,(dd)(4*5)));
R(DEC(32,(READDD(ecx))));
R(JNZ(teryrtyrtyuuiyuyu));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
noel:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(esi),32,(dd)((64000*8)+(136*320))));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,arbre), es))));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,arbre), es)),32,(dd)32));
R(JZ(retterertertert));
R(ADD(32,READDD(esi),32,(dd)33));
retterertertert:
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+112)+(31*320))))));
R(MOV(32,READDD(ecx),32,(dd)48));
uertertertrteert:
R(MOV(32,READDD(ebx),32,(dd)33));
uconcert:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(uertertertrte));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
uertertertrte:
R(INC(32,(READDD(edi))));
R(DEC(32,(READDD(ebx))));
R(JNZ(uconcert));
R(ADD(32,READDD(edi),32,(dd)(320-33)));
R(ADD(32,READDD(esi),32,(dd)(320-33)));
R(DEC(32,(READDD(ecx))));
R(JNZ(uertertertrteert));
R(MOV(32,READDD(esi),32,(dd)((64000*8)+(77*320))));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+232)+(56*320))))));
R(MOV(32,READDD(ecx),32,(dd)42));
yuertertertrteert:
R(MOV(32,READDD(ebx),32,(dd)48));
yuconcert:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(yuertertertrte));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
yuertertertrte:
R(INC(32,(READDD(edi))));
R(DEC(32,(READDD(ebx))));
R(JNZ(yuconcert));
R(ADD(32,READDD(edi),32,(dd)(320-48)));
R(ADD(32,READDD(esi),32,(dd)(320-48)));
R(DEC(32,(READDD(ecx))));
R(JNZ(yuertertertrteert));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,detail), es)),8,(db)1));
R(JE(retrterterte5));
R(MOV(32,READDD(esi),32,(dd)((((64000*8)+(40*320))+69)+(16*4))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,arbre), es))));
R(AND(32,READDD(ebx),32,(dd)16));
R(JZ(ertertreterterter));
R(ADD(32,READDD(esi),32,(dd)16));
ertertreterterter:
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*184))+50)))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)(((offsetof(struct Mem,buffer)+(320*184))+50)+(17*1))))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)(((offsetof(struct Mem,buffer)+(320*184))+50)+(17*4))))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)(((offsetof(struct Mem,buffer)+(320*184))+50)+(17*9))))));
CALL(oiseau);
R(MOV(32,READDD(esi),32,(dd)((((64000*8)+(40*320))+69)+(16*0))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,arbre), es))));
R(AND(32,READDD(ebx),32,(dd)16));
R(JZ(iertertreterterter));
R(ADD(32,READDD(esi),32,(dd)16));
iertertreterterter:
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*164))+1)))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*148))+1)))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*101))+1)))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*30))+1)))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*47))+1)))));
CALL(oiseau);
retrterterte5:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
noel2:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(esi),32,(dd)(((64000*8)+(23*320))+69)));
R(MOV(32,m.edi.dd.val,32,(((dd)(((offsetof(struct Mem,buffer)+1)+320)+9)))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,arbre), es))));
R(ADD(32,READDD(ebx),32,(dd)(18*16)));
R(AND(32,READDD(ebx),32,(dd)496));
R(SHR(32,READDD(ebx),32,(dd)2));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,offset_oiseau)+READDD(ebx)), es))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)(((((offsetof(struct Mem,buffer)+1)+320)+16)+1)+9)))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)(((((((offsetof(struct Mem,buffer)+1)+320)+16)+1)+16)+1)+9)))));
CALL(oiseau);
R(MOV(32,READDD(esi),32,(dd)(((64000*8)+(23*320))+69)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,arbre), es))));
R(ADD(32,READDD(ebx),32,(dd)(20*16)));
R(AND(32,READDD(ebx),32,(dd)480));
R(SHR(32,READDD(ebx),32,(dd)3));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,offset_oiseau)+READDD(ebx)), es))));
R(MOV(32,m.edi.dd.val,32,(((dd)(((((((offsetof(struct Mem,buffer)+1)+320)+16)+1)+16)+1)+(16*8))))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)(((((((offsetof(struct Mem,buffer)+1)+320)+16)+1)+16)+1)+(16*10))))));
CALL(oiseau);
R(MOV(32,READDD(esi),32,(dd)(((64000*8)+(23*320))+69)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,arbre), es))));
R(ADD(32,READDD(ebx),32,(dd)(20*16)));
R(AND(32,READDD(ebx),32,(dd)240));
R(SHR(32,READDD(ebx),32,(dd)2));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,offset_oiseau)+READDD(ebx)), es))));
R(MOV(32,m.edi.dd.val,32,(((dd)(((((((offsetof(struct Mem,buffer)+1)+320)+16)+1)+16)+1)+(16*5))))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)(((((((offsetof(struct Mem,buffer)+1)+320)+16)+1)+6)+1)+(16*14))))));
CALL(oiseau);
R(MOV(32,READDD(esi),32,(dd)(((64000*8)+(23*320))+69)));
R(MOV(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+112)+(89*320))))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,arbre), es))));
R(AND(32,READDD(ebx),32,(dd)496));
R(SHR(32,READDD(ebx),32,(dd)2));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,offset_oiseau)+READDD(ebx)), es))));
CALL(oiseau);
R(MOV(32,m.edi.dd.val,32,(((dd)((((offsetof(struct Mem,buffer)+112)+(89*320))+16)+1)))));
CALL(oiseau);
retrterterte5rt:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
oiseau:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,(dd)16));
iyuertertertrteert:
R(MOV(32,READDD(ebx),32,(dd)16));
iyuconcert:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(iyuertertertrte));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
iyuertertertrte:
R(INC(32,(READDD(edi))));
R(DEC(32,(READDD(ebx))));
R(JNZ(iyuconcert));
R(ADD(32,READDD(edi),32,(dd)(320-16)));
R(ADD(32,READDD(esi),32,(dd)(320-16)));
R(DEC(32,(READDD(ecx))));
R(JNZ(iyuertertertrteert));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
expere:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,(dd)8));
oooooooo:
R(PUSH(32,(READDD(ecx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(edi),32,(dd)READDD(edi)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+6)+READDD(ebx)), ds))));
R(EAX_X_320);
R(ADD(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+4)+READDD(ebx)), ds))));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+4)+READDD(ebx)), ds)),16,(dw)((0-1))));
R(JE(lapinombnooooiii2x43));
R(PUSH(32,(READDD(ecx))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ecx),32,(dd)READDD(edi)));
R(ADD(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,kel_ombre), ds))));
R(SHR(32,READDD(ebx),8,(db)READDBl(ecx)));
R(AND(32,READDD(ebx),32,(dd)15));
R(JZ(lapinombnan_prend_pas_lombrex43));
R(CMP(32,READDD(ebx),32,(dd)3));
R(JA(lapinombnan_prend_pas_lombrex43));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ecx))));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,ombres)+READDD(edi)), ds))));
R(ADD(16,READDW(eax),16,(dw)(8*320)));
R(JMP(lapinombnooooiiix43));
lapinombnan_prend_pas_lombrex43:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ecx))));
lapinombnooooiiix43:
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
lapinombnooooiii2x43:
R(MOV(32,READDD(ebp),32,(dd)READDD(ebx)));
R(PUSH(32,(READDD(ecx))));
R(MOV(32,READDD(ecx),32,(dd)8));
gogotingo:
R(ADD(32,READDD(ebx),32,(dd)nb_unite_donnee4));
R(ADD(32,READDD(edi),32,(dd)2));
R(DEC(32,(READDD(ecx))));
R(JZ(mwai));
R(XOR(32,READDD(edx),32,(dd)READDD(edx)));
R(MOV(8,READDBl(edx),8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+6)+READDD(ebx)), ds))));
R(EDX_X_320);
R(ADD(16,READDW(edx),16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+4)+READDD(ebx)), ds))));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+4)+READDD(ebx)), ds)),16,(dw)((0-1))));
R(JE(lapinombnooooiii2x44));
R(PUSH(32,(READDD(ecx))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ecx),32,(dd)READDD(edi)));
R(ADD(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,kel_ombre), ds))));
R(SHR(32,READDD(ebx),8,(db)READDBl(ecx)));
R(AND(32,READDD(ebx),32,(dd)15));
R(JZ(lapinombnan_prend_pas_lombrex44));
R(CMP(32,READDD(ebx),32,(dd)3));
R(JA(lapinombnan_prend_pas_lombrex44));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ecx))));
R(MOV(16,READDW(edx),16,read_dw(realAddress((offsetof(struct Mem,ombres)+READDD(edi)), ds))));
R(ADD(16,READDW(edx),16,(dw)(8*320)));
R(JMP(lapinombnooooiiix44));
lapinombnan_prend_pas_lombrex44:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ecx))));
lapinombnooooiiix44:
R(ADD(16,READDW(edx),16,(dw)READDW(ebx)));
R(ADD(16,READDW(edx),16,(dw)READDW(ebx)));
R(ADD(16,READDW(edx),16,(dw)READDW(ebx)));
lapinombnooooiii2x44:
R(CMP(16,READDW(edx),16,(dw)READDW(eax)));
R(JNB(fraiche));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+6)+READDD(ebx)), ds))));
R(EAX_X_320);
R(ADD(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+4)+READDD(ebx)), ds))));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+4)+READDD(ebx)), ds)),16,(dw)((0-1))));
R(JE(lapinombnooooiii2x45));
R(PUSH(32,(READDD(ecx))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ecx),32,(dd)READDD(edi)));
R(ADD(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,kel_ombre), ds))));
R(SHR(32,READDD(ebx),8,(db)READDBl(ecx)));
R(AND(32,READDD(ebx),32,(dd)15));
R(JZ(lapinombnan_prend_pas_lombrex45));
R(CMP(32,READDD(ebx),32,(dd)3));
R(JA(lapinombnan_prend_pas_lombrex45));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ecx))));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,ombres)+READDD(edi)), ds))));
R(ADD(16,READDW(eax),16,(dw)(8*320)));
R(JMP(lapinombnooooiiix45));
lapinombnan_prend_pas_lombrex45:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ecx))));
lapinombnooooiiix45:
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
lapinombnooooiii2x45:
R(MOV(32,READDD(ebp),32,(dd)READDD(ebx)));
fraiche:
R(JMP(gogotingo));
mwai:
R(POP(32,(READDD(ecx))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,donnee4)+READDD(ebp)), ds))));
R(CMP(32,READDD(esi),32,(dd)666));
R(JE(nanananana_il_est_mort));
R(PUSH(32,(READDD(edi))));
R(XOR(32,READDD(edi),32,(dd)READDD(edi)));
R(MOV(16,READDW(edi),16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+4)+READDD(ebp)), ds))));
R(PUSH(16,(READDW(ds))));
R(XOR(16,READDW(ebx),16,(dw)READDW(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+6)+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(MOV(8,READDBl(ecx),8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+7)+READDD(ebp)), ds))));
R(MOV(8,READDBl(edx),8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+8)+READDD(ebp)), ds))));
R(AND(8,READDBl(edx),8,(db)1));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
CALL(affiche_bomby);
R(POP(16,(READDW(ds))));
R(POP(32,(READDD(edi))));
nanananana_il_est_mort:
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+4)+READDD(ebp)), ds)),16,(dw)((0-1))));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+6)+READDD(ebp)), ds)),8,(db)0));
R(POP(32,(READDD(ecx))));
R(DEC(32,(READDD(ecx))));
R(JNZ(oooooooo));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
retterterertertertertertertertertertert:
R(SPRITE_16_16);
R(JMP(retterterertertertertertertertertertert2));
affiche_bomby:
R(CMP(8,READDBl(edx),8,(db)1));
R(JE(blancheur_supreme));
affiche_bomby2:
R(ADD(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(SPRITE_BN);
RET;
blancheur_supreme:
R(ADD(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(SPRITE_BW);
RET;
aff_brique:
R(MOV(32,READDD(eax),32,read_dd(realAddress(READDD(esi), ds))));
R(MOV(32,read_dd(realAddress(READDD(edi), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+4), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+8), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+12), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+12), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+320), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+320), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+320)+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+320)+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+320)+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+320)+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+320)+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+320), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*2)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*2)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*2))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*2))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*2))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*2))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*2))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*2)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*3)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*3)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*3))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*3))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*3))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*3))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*3))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*3)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*4)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*4)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*4))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*4))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*4))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*4))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*4))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*4)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*5)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*5)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*5))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*5))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*5))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*5))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*5))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*5)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*6)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*6)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*6))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*6))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*6))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*6))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*6))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*6)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*7)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*7)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*7))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*7))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*7))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*7))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*7))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*7)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*8)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*8)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*8))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*8))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*8))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*8))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*8))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*8)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*9)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*9)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*9))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*9))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*9))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*9))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*9))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*9)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*10)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*10)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*10))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*10))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*10))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*10))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*10))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*10)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*11)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*11)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*11))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*11))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*11))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*11))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*11))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*11)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*12)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*12)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*12))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*12))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*12))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*12))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*12))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*12)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*13)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*13)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*13))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*13))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*13))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*13))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*13))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*13)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*14)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*14)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*14))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*14))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*14))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*14))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*14))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*14)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+(320*15)), ds))));
R(MOV(32,read_dd(realAddress((READDD(edi)+(320*15)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*15))+4), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*15))+4), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*15))+8), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+(320*15))+8), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+(320*15))+12), ds))));
R(MOV(32,read_dd(realAddress(((READDD(edi)+12)+(320*15)), ds)),32,(dd)READDD(eax)));
RET;
reetrertertert3:
R(SUB(32,READDD(edi),32,(dd)3));
R(MOV(16,READDW(edx),16,(dw)16));
aff_oeufertertrteertrtertex47:
R(MOV(16,READDW(ebx),16,(dw)22));
aff_oeufzerertertterx47:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(aff_oeufretertterertx47));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
aff_oeufretertterertx47:
R(INC(32,(READDD(edi))));
R(DEC(16,(READDW(ebx))));
R(JNZ(aff_oeufzerertertterx47));
R(ADD(32,READDD(edi),32,(dd)(320-22)));
R(ADD(32,READDD(esi),32,(dd)(320-22)));
R(DEC(16,(READDW(edx))));
R(JNZ(aff_oeufertertrteertrtertex47));
R(JMP(reetrertertert));
reetrertertert2:
R(SUB(32,READDD(edi),32,(dd)((11*320)+6)));
R(SPRITE_27_31);
R(JMP(reetrertertert));

//PROC beuh
beuh:
R(PUSH(16,(READDW(ds))));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
R(MOV(16,READDW(ds),16,(dw)READDW(eax)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(MOV(32,READDD(esi),32,(dd)0));
R(MOV(32,READDD(edi),32,(dd)614400));
R(MOV(32,READDD(ecx),32,(dd)76800));
R(REP_MOVSD);
R(POP(16,(READDW(ds))));
RET;

//PROC aff_page
aff_page:
R(PUSH(16,(READDW(ds))));
R(XOR(32,READDD(edi),32,(dd)READDD(edi)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
R(MOV(16,READDW(ds),16,(dw)READDW(eax)));
R(XOR(16,READDW(ebx),16,(dw)READDW(ebx)));
R(XOR(16,READDW(edx),16,(dw)READDW(edx)));
CALL(change_page);
R(PUSH(32,(READDD(edi))));
R(MOV(32,READDD(ecx),32,(dd)16384));
R(REP_MOVSD);
R(POP(32,(READDD(edi))));
CALL(change_page);
R(PUSH(32,(READDD(edi))));
R(MOV(32,READDD(ecx),32,(dd)16384));
R(REP_MOVSD);
R(POP(32,(READDD(edi))));
CALL(change_page);
R(PUSH(32,(READDD(edi))));
R(MOV(32,READDD(ecx),32,(dd)16384));
R(REP_MOVSD);
R(POP(32,(READDD(edi))));
CALL(change_page);
R(PUSH(32,(READDD(edi))));
R(MOV(32,READDD(ecx),32,(dd)16384));
R(REP_MOVSD);
R(POP(32,(READDD(edi))));
CALL(change_page);
R(PUSH(32,(READDD(edi))));
R(MOV(32,READDD(ecx),32,(dd)(45056/4)));
R(REP_MOVSD);
R(POP(32,(READDD(edi))));
R(POP(16,(READDW(ds))));
RET;
change_page:
R(MOV(16,READDW(eax),16,(dw)20229));
R(INT(16));
R(INC(16,(READDW(edx))));
RET;

//PROC aff_page2
aff_page2:
R(RAMBUFFER);
RET;

//PROC load_pcx
load_pcx:
R(PUSHAD);
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,load_pcx_interne), ds)),32,(dd)READDD(ebx)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,(db)0));
R(MOV(8,READDBh(eax),8,(db)61));
R(INT(33));
R(JNC(fileerrorerreur_filecx48));
R(MOV(16,READDW(eax),16,(dw)3));
R(INT(16));
R(LEA(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,loaderror)))));
R(MOV(8,READDBh(eax),8,(db)9));
R(INT(33));
R(MOV(8,READDBl(ebx),8,(db)4));
R(MOV(8,READDBh(ebx),8,(db)128));
CALL(last_color);
R(MOV(16,READDW(eax),16,(dw)19456));
R(INT(33));
fileerrorerreur_filecx48:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds)),32,(dd)READDD(eax)));
R(PUSH(32,(READDD(ecx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(MOV(8,READDBh(eax),8,(db)66));
R(MOV(8,READDBl(eax),8,(db)0));
R(MOV(16,READDW(edx),16,(dw)READDW(ecx)));
R(SHR(32,READDD(ecx),32,(dd)16));
R(INT(33));
R(JNC(fileerrorerreur_filecx49));
R(MOV(16,READDW(eax),16,(dw)3));
R(INT(16));
R(LEA(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,loaderror)))));
R(MOV(8,READDBh(eax),8,(db)9));
R(INT(33));
R(MOV(8,READDBl(ebx),8,(db)4));
R(MOV(8,READDBh(ebx),8,(db)128));
CALL(last_color);
R(MOV(16,READDW(eax),16,(dw)19456));
R(INT(33));
fileerrorerreur_filecx49:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(MOV(8,READDBh(eax),8,(db)63));
R(MOV(32,READDD(ecx),32,(dd)65535));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(INT(33));
R(JNC(fileerrorerreur_filecx50));
R(MOV(16,READDW(eax),16,(dw)3));
R(INT(16));
R(LEA(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,loaderror)))));
R(MOV(8,READDBh(eax),8,(db)9));
R(INT(33));
R(MOV(8,READDBl(ebx),8,(db)4));
R(MOV(8,READDBh(ebx),8,(db)128));
CALL(last_color);
R(MOV(16,READDW(eax),16,(dw)19456));
R(INT(33));
fileerrorerreur_filecx50:
R(POP(32,(READDD(eax))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,buffer), ds))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,isbigendian), ds)),8,(db)1));
R(JNE(bigendianpatchblablablatotox51));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,bigendianin), ds)),32,(dd)READDD(ebx)));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,bigendianin), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+3), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+1), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+2), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+2), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+1), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+3), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+0), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,bigendianout), ds))));
bigendianpatchblablablatotox51:
R(ADD(32,READDD(eax),32,(dd)READDD(ebx)));
R(POP(32,(READDD(ebx))));
R(SUB(32,READDD(eax),32,(dd)768));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+128)))));
encore_un_pixel:
R(CMP(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_pcx_interne), ds))));
R(JNB(cestfini));
R(LODSB);
R(CMP(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+65535)))));
R(JNE(coke));
CALL(charge_encore);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
coke:
R(CMP(8,READDBl(eax),8,(db)192));
R(JB(non_c_un_octet_seul));
R(AND(8,READDBl(eax),8,(db)63));
R(MOVZX(16,READDW(ecx),8,(db)READDBl(eax)));
R(LODSB);
R(CMP(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+65535)))));
R(JNE(cok));
CALL(charge_encore);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
cok:
viennes:
R(STOSB);
R(INC(32,(READDD(ebx))));
R(DEC(16,(READDW(ecx))));
R(JNZ(viennes));
R(JMP(encore_un_pixel));
non_c_un_octet_seul:
R(STOSB);
R(INC(32,(READDD(ebx))));
R(JMP(encore_un_pixel));
cestfini:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(POP(32,(READDD(eax))));
R(MOV(16,READDW(edx),16,(dw)READDW(eax)));
R(SHR(32,READDD(eax),32,(dd)16));
R(MOV(16,READDW(ecx),16,(dw)READDW(eax)));
R(MOV(8,READDBh(eax),8,(db)66));
R(MOV(8,READDBl(eax),8,(db)0));
R(INT(33));
R(JNC(fileerrorerreur_filecx52));
R(MOV(16,READDW(eax),16,(dw)3));
R(INT(16));
R(LEA(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,loaderror)))));
R(MOV(8,READDBh(eax),8,(db)9));
R(INT(33));
R(MOV(8,READDBl(ebx),8,(db)4));
R(MOV(8,READDBh(ebx),8,(db)128));
CALL(last_color);
R(MOV(16,READDW(eax),16,(dw)19456));
R(INT(33));
fileerrorerreur_filecx52:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(MOV(8,READDBh(eax),8,(db)63));
R(MOV(16,READDW(ecx),16,(dw)768));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,pal)))));
R(INT(33));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,pal)))));
R(MOV(16,READDW(ecx),16,(dw)768));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
rtrttr:
R(MOV(8,READDBl(eax),8,*((db *) realAddress(READDD(esi), ds))));
R(SHR(8,READDBl(eax),8,(db)2));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)READDBl(eax)));
R(INC(32,(READDD(esi))));
R(DEC(16,(READDW(ecx))));
R(JNZ(rtrttr));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(MOV(8,READDBh(eax),8,(db)62));
R(INT(33));
R(JNC(fileerrorerreur_filecx53));
R(MOV(16,READDW(eax),16,(dw)3));
R(INT(16));
R(LEA(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,loaderror)))));
R(MOV(8,READDBh(eax),8,(db)9));
R(INT(33));
R(MOV(8,READDBl(ebx),8,(db)4));
R(MOV(8,READDBh(ebx),8,(db)128));
CALL(last_color);
R(MOV(16,READDW(eax),16,(dw)19456));
R(INT(33));
fileerrorerreur_filecx53:
R(POP(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(POPAD);
RET;

//PROC load_raw
load_raw:
R(PUSHAD);
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,(db)0));
R(MOV(8,READDBh(eax),8,(db)61));
R(INT(33));
R(JNC(retyryurttyutyutyuutyyuiiyuuiyuiy));
R(MOV(8,READDBl(edx),8,(db)13));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(LEA(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,suite2)))));
R(MOV(8,READDBh(eax),8,(db)9));
R(INT(33));
R(MOV(8,READDBl(ebx),8,(db)4));
R(MOV(8,READDBh(ebx),8,(db)128));
CALL(last_color);
R(MOV(16,READDW(eax),16,(dw)19456));
R(INT(33));
retyryurttyutyutyuutyyuiiyuuiyuiy:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(MOV(8,READDBh(eax),8,(db)66));
R(MOV(8,READDBl(eax),8,(db)0));
R(MOV(16,READDW(edx),16,(dw)READDW(ecx)));
R(SHR(32,READDD(ecx),32,(dd)16));
R(INT(33));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(MOV(8,READDBh(eax),8,(db)63));
R(MOV(32,READDD(ecx),32,(dd)64000));
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(edx),32,(dd)READDD(edi)));
R(INT(33));
R(POP(16,(READDW(ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(MOV(8,READDBh(eax),8,(db)62));
R(INT(33));
R(POP(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(POPAD);
RET;

//PROC charge_encore
charge_encore:
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(eax))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(16,(READDW(ecx))));
R(PUSH(16,(READDW(edx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,load_handle), ds))));
R(MOV(8,READDBh(eax),8,(db)63));
R(MOV(16,READDW(ecx),16,(dw)65535));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(INT(33));
R(POP(16,(READDW(edx))));
R(POP(16,(READDW(ecx))));
R(POP(32,(READDD(ebx))));
R(POP(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(eax))));
R(POP(16,(READDW(ds))));
RET;

//PROC pal_visage
pal_visage:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)0));
R(JNE(histoire));
rty:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
histoire:
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)1));
R(JNE(zorrro));
R(XOR(16,READDW(ebp),16,(dw)READDW(ebp)));
R(MOV(16,READDW(edx),16,(dw)4));
reeeeee:
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,pal_affiche)))));
R(MOV(16,READDW(ecx),16,(dw)768));
rchanger:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JE(pareil));
R(DEC(8,*((db *) realAddress(READDD(esi), ds))));
R(JMP(pas_pareil));
pareil:
R(INC(16,(READDW(ebp))));
pas_pareil:
R(INC(32,(READDD(esi))));
R(DEC(16,(READDW(ecx))));
R(JNZ(rchanger));
R(DEC(16,(READDW(edx))));
R(JNZ(reeeeee));
R(CMP(16,READDW(ebp),16,(dw)(768*4)));
R(JNE(erterterterrterte));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)2));
erterterterrterte:
CALL(affpal);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
zorrro:
R(XOR(16,READDW(ebp),16,(dw)READDW(ebp)));
R(MOV(16,READDW(edx),16,(dw)4));
yreeeeee:
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pal)))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre), ds)),8,(db)83));
R(JNE(rezterterertrteertyerertrteterter));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JNE(opiopiioiouuiiuiuiuooo));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,pic_time), ds)),32,(dd)0));
R(JE(opiopiioiouuiiuiuiuooo));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pal_pic)))));
opiopiioiouuiiuiuiuooo:
rezterterertrteertyerertrteterter:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre), ds)),8,(db)83));
R(JE(terterertrteertyerertrteterter));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)3));
R(JNE(terrteterertterterertrteertyerertrtetertererer));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pal_jeu)))));
terrteterertterterertrteertyerertrtetertererer:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)86));
R(JNE(terrteterertterterertrteertyerertrteterterererr));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pal_jeu)))));
terrteterertterterertrteertyerertrteterterererr:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)86));
R(JNE(terrteterertterterertrteertyerertrtetertererereerr));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pal_vic)))));
terrteterertterterertrteertyerertrtetertererereerr:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)90));
R(JNE(terrteterertterterertrteertyerertrteterterereree));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pal_med)))));
terrteterertterterertrteertyerertrteterterereree:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)68));
R(JNE(tyr2trtyrtyrtyrtyrtyterterertrteert));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pal_draw)))));
tyr2trtyrtyrtyrtyrtyterterertrteert:
terterertrteertyerertrteterter:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,pic_de_tout_debut), ds)),8,(db)1));
R(JNE(ereetrtrtrtrteeete));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pal_pic2)))));
ereetrtrtrtrteeete:
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,pal_affiche)))));
R(MOV(16,READDW(ecx),16,(dw)768));
yrchanger:
R(MOV(8,READDBl(eax),8,*((db *) realAddress(READDD(edi), ds))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)READDBl(eax)));
R(JE(ypareil));
R(INC(8,*((db *) realAddress(READDD(esi), ds))));
R(JMP(ypas_pareil));
ypareil:
R(INC(16,(READDW(ebp))));
ypas_pareil:
R(INC(32,(READDD(edi))));
R(INC(32,(READDD(esi))));
R(DEC(16,(READDW(ecx))));
R(JNZ(yrchanger));
R(DEC(16,(READDW(edx))));
R(JNZ(yreeeeee));
R(CMP(16,READDW(ebp),16,(dw)(768*4)));
R(JNE(yerterterterrterte));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)0));
yerterterterrterte:
CALL(affpal);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC affpal
affpal:
R(PUSHAD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,pal_affiche)))));
R(MOV(16,READDW(edx),16,(dw)968));
R(XOR(8,READDBl(eax),8,(db)READDBl(eax)));
OUT(READDW(edx),READDBl(eax));
R(MOV(16,READDW(edx),16,(dw)969));
R(MOV(16,READDW(ecx),16,(dw)(256*3)));
arobasearobasesaaccvaaaax:
R(LODSB);
OUT(READDW(edx),READDBl(eax));
R(DEC(16,(READDW(ecx))));
R(JNZ(arobasearobasesaaccvaaaax));
R(POPAD);
RET;

//PROC affpal2
affpal2:
R(PUSHAD);
R(MOV(16,READDW(edx),16,(dw)968));
R(XOR(8,READDBl(eax),8,(db)READDBl(eax)));
OUT(READDW(edx),READDBl(eax));
R(MOV(16,READDW(edx),16,(dw)969));
R(MOV(16,READDW(ecx),16,(dw)(256*3)));
aarobasearobasesaaccvaaaax:
R(LODSB);
OUT(READDW(edx),READDBl(eax));
R(DEC(16,(READDW(ecx))));
R(JNZ(aarobasearobasesaaccvaaaax));
R(POPAD);
RET;

//PROC copie_le_fond
copie_le_fond:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,attente), ds))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente), ds)),32,(dd)0));
R(JNE(erteteteerre));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_fond), ds)),32,(dd)4));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente), ds)),32,(dd)max_attente));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)2));
R(JNE(yertertertertertutyoooooortyyrt));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente), ds)),32,(dd)min_attente));
yertertertertertutyoooooortyyrt:
erteteteerre:
R(MOV(32,READDD(eax),32,(dd)(4*4)));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)2));
R(JNE(yertertertertertutyoooooo));
R(MOV(32,READDD(eax),32,(dd)(3*4)));
yertertertertertutyoooooo:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_fond), ds))));
R(CMP(32,READDD(ebx),32,(dd)READDD(eax)));
R(JB(erterertyurttyu));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_fond), ds)),32,(dd)0));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
erterertyurttyu:
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(SHL(32,READDD(ebx),32,(dd)2));
R(MOV(32,READDD(esi),32,read_dd(realAddress((READDD(ebx)+offsetof(struct Mem,kelle_offset_fond)), ds))));
R(POP(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((READDD(esi)+READDD(ebx)), ds))));
R(RAMESI);
R(COPYBLOCK);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC copie_le_fond_draw
copie_le_fond_draw:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,attente), ds))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente), ds)),32,(dd)0));
R(JNE(yerteteteerre));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_draw), ds)),32,(dd)4));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente), ds)),32,(dd)max_attente));
yerteteteerre:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_draw), ds))));
R(CMP(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_draw), ds))));
R(JNE(yerterertyurttyu));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_draw), ds)),32,(dd)0));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
yerterertyurttyu:
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,adresse_des_draws)+READDD(ebx)), ds))));
R(RAMESI);
R(COPYBLOCK);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC copie_le_fond_vic
copie_le_fond_vic:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,attente), ds))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente), ds)),32,(dd)0));
R(JNE(iyyerteteteerre));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_vic), ds)),32,(dd)4));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente), ds)),32,(dd)max_attente4));
iyyerteteteerre:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_vic), ds))));
R(CMP(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_vic), ds))));
R(JNE(iyerterertyurttyu));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_vic), ds)),32,(dd)0));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
iyerterertyurttyu:
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,adresse_des_vic)+READDD(ebx)), ds))));
R(RAMESI);
R(COPYBLOCK);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,changementzz), ds))));
supreme_victory_group:
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(MOV(32,READDD(edi),32,read_dd(realAddress(offsetof(struct Mem,latest_victory), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(edi)), ds))));
R(XOR(32,READDD(edi),32,(dd)READDD(edi)));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
supreme_victory_group_winners_loop:
R(CMP(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(edi)), ds))));
R(JNE(supreme_victory_group_winners_next));
R(ADD(32,READDD(ecx),32,(dd)(16+8)));
supreme_victory_group_winners_next:
R(ADD(32,READDD(edi),32,(dd)4));
R(DEC(32,(READDD(eax))));
R(JNE(supreme_victory_group_winners_loop));
R(MOV(32,READDD(edx),32,(dd)320));
R(SUB(32,READDD(edx),32,(dd)READDD(ecx)));
R(SHR(32,READDD(edx),32,(dd)1));
R(ADD(32,READDD(edx),32,(dd)(57*320)));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
supreme_victory_group_loop:
R(CMP(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(ecx)), ds))));
R(JNE(supreme_victory_group_next));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(ecx))));
R(PUSH(32,(READDD(edx))));
R(TEST(32,READDD(ecx),32,(dd)4));
R(JNE(supreme_victory_group_sprite));
R(ADD(32,READDD(edx),32,(dd)(3*320)));
supreme_victory_group_sprite:
R(LEA(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,donnee4)+(9*4))))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,ooo546)+READDD(ecx)), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,liste_couleur)+READDD(ecx)), ds))));
R(MOVZX(32,READDD(eax),16,read_dw(realAddress((READDD(ebx)+(0*2)), ds))));
R(ADD(32,READDD(eax),32,(dd)READDD(esi)));
R(STOSD);
R(MOVZX(32,READDD(eax),16,read_dw(realAddress((READDD(ebx)+(1*2)), ds))));
R(ADD(32,READDD(eax),32,(dd)READDD(esi)));
R(STOSD);
R(MOVZX(32,READDD(eax),16,read_dw(realAddress((READDD(ebx)+(2*2)), ds))));
R(ADD(32,READDD(eax),32,(dd)READDD(esi)));
R(STOSD);
R(MOVZX(32,READDD(eax),16,read_dw(realAddress((READDD(ebx)+(3*2)), ds))));
R(ADD(32,READDD(eax),32,(dd)READDD(esi)));
R(STOSD);
R(SHR(32,READDD(ecx),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,dummy1392)+READDD(ecx)), ds))));
R(STOSB);
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,dummy1393)+READDD(ecx)), ds))));
R(STOSB);
R(SHL(32,READDD(ecx),32,(dd)1));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,changementzz), ds))));
R(AND(32,READDD(ebx),32,(dd)48));
R(SHR(32,READDD(ebx),32,(dd)2));
R(MOV(32,READDD(esi),32,read_dd(realAddress(((offsetof(struct Mem,donnee4)+(9*4))+READDD(ebx)), ds))));
R(MOV(32,READDD(edi),32,(dd)READDD(edx)));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(MOV(8,READDBl(ecx),8,*((db *) realAddress((((offsetof(struct Mem,donnee4)+(9*4))+(4*4))+1), ds))));
R(XOR(16,READDW(ebx),16,(dw)READDW(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+(9*4))+(4*4)), ds))));
R(MOV(8,READDBl(edx),8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+8)+READDD(ebp)), ds))));
R(AND(8,READDBl(edx),8,(db)1));
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
CALL(affiche_bomby2);
R(POP(16,(READDW(ds))));
R(POP(32,(READDD(edx))));
R(POP(32,(READDD(ecx))));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(eax))));
R(ADD(32,READDD(edx),32,(dd)(16+8)));
supreme_victory_group_next:
R(ADD(32,READDD(ecx),32,(dd)4));
R(DEC(32,(READDD(eax))));
R(JNZ(supreme_victory_group_loop));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC copie_le_fond_med
copie_le_fond_med:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,blow_what)))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,blow_what2)))));
R(MOV(32,READDD(ecx),32,(dd)14));
R(REP_MOVSW);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(MOV(32,READDD(esi),32,(dd)(64000*7)));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)1));
R(JNE(pas_color));
R(MOV(32,READDD(esi),32,(dd)(1582080+64000)));
pas_color:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)2));
R(JNE(pas_g));
R(MOV(32,READDD(esi),32,(dd)(1582080+(64000*2))));
pas_g:
R(RAMESI);
R(COPYBLOCK);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,m.ebx.dd.val,32,(((dd)offsetof(struct Mem,briques)))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_couleur), ds)),32,(dd)0));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(XOR(32,READDD(edx),32,(dd)READDD(edx)));
pppppppgoeger:
R(PUSH(32,(READDD(edi))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)1));
R(JNE(pas_c3));
R(ADD(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,viseur_namec)+READDD(edx)), ds))));
R(JMP(ihjhuihui));
pas_c3:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)2));
R(JNE(pas_g3));
R(ADD(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,viseur_nameg)+READDD(edx)), ds))));
R(JMP(ihjhuihui));
pas_g3:
R(ADD(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,viseur_name)+READDD(edx)), ds))));
ihjhuihui:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
CALL(affiche_un_caractere);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(ADD(32,READDD(edi),32,(dd)8));
R(INC(32,(READDD(ebx))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
CALL(affiche_un_caractere);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(ADD(32,READDD(edi),32,(dd)8));
R(INC(32,(READDD(ebx))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
CALL(affiche_un_caractere);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(INC(32,read_dd(realAddress(offsetof(struct Mem,viseur_couleur), ds))));
R(ADD(32,READDD(ebx),32,(dd)2));
R(POP(32,(READDD(edi))));
R(ADD(32,READDD(edx),32,(dd)4));
R(DEC(32,(READDD(ecx))));
R(JNZ(pppppppgoeger));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,READDD(esi),32,(dd)(64000*8)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
R(MOV(16,READDW(ds),16,(dw)READDW(eax)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,changementzz), es))));
R(AND(32,READDD(ebx),32,(dd)60));
R(SHR(32,READDD(ebx),32,(dd)1));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,changementzz), es))));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
ertrterteterter:
R(PUSH(32,(READDD(edi))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,donnee4)+READDD(ebp)), es))));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(ertertertertertertert));
R(CMP(32,READDD(ecx),32,(dd)5));
R(JB(rerterteertrte));
R(MOV(32,READDD(ecx),32,(dd)5));
rerterteertrte:
erertererrteert:
R(PUSH(32,(READDD(edi))));
R(PUSH(32,(READDD(esi))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,offset_medaille)+READDD(ebx)), es))));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), es)),8,(db)1));
R(JNE(pas_c32));
R(ADD(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,viseur_victoryc)+READDD(ebp)), es))));
R(JMP(ihjhuihui2));
pas_c32:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), es)),8,(db)2));
R(JNE(pas_c32r));
R(ADD(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,viseur_victoryg)+READDD(ebp)), es))));
R(JMP(ihjhuihui2));
pas_c32r:
R(ADD(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,viseur_victory)+READDD(ebp)), es))));
ihjhuihui2:
R(CMP(32,READDD(ecx),32,(dd)1));
R(JNE(eertterrtrteterert));
draw_skynet_team_medals:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3_sauve), es)),8,(db)4));
R(JNE(check_victory_medal_player));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,latest_victory), es))));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(eax)), es))));
R(CMP(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(ebp)), es))));
R(JE(draw_win_victory_medal));
check_victory_medal_player:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,donnee4)+(4*8)), es)),32,(dd)READDD(ebp)));
R(JNE(eertterrtrteterert));
draw_win_victory_medal:
R(MOV(32,READDD(esi),32,(dd)(64000*8)));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,briques)+(8*4)), es)),8,(db)1));
R(JE(ertterrtrteterert));
eertterrtrteterert:
CALL(aff_sprite);
ertterrtrteterert:
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(edi))));
R(ADD(32,READDD(edi),32,(dd)23));
R(ADD(32,READDD(ebx),32,(dd)(2*2)));
R(AND(32,READDD(ebx),32,(dd)31));
R(DEC(32,(READDD(ecx))));
R(JNZ(erertererrteert));
ertertertertertertert:
R(POP(32,(READDD(edi))));
R(ADD(32,READDD(ebp),32,(dd)4));
R(CMP(32,READDD(ebp),32,(dd)(4*8)));
R(JNE(ertrterteterter));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
aff_sprite:
R(PUSH(32,(READDD(ecx))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(esi))));
R(MOV(32,READDD(ecx),32,(dd)22));
ertertertrteert:
R(MOV(32,READDD(ebx),32,(dd)22));
concert:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(ertertertrte));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
ertertertrte:
R(INC(32,(READDD(edi))));
R(DEC(32,(READDD(ebx))));
R(JNZ(concert));
R(ADD(32,READDD(edi),32,(dd)(320-22)));
R(ADD(32,READDD(esi),32,(dd)(320-22)));
R(DEC(32,(READDD(ecx))));
R(JNZ(ertertertrteert));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ecx))));
RET;

//PROC inst_clavier
inst_clavier:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(16,READDW(ecx),16,(dw)READDW(cs)));
R(MOV(16,READDW(eax),16,(dw)517));
R(MOV(8,READDBl(ebx),8,(db)9));
R(INT(49));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC HANDLER10
HANDLER10:
R(PUSHF);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,(dd)1254486));
R(MOV(32,READDD(edx),32,(dd)1254486));
R(MOV(16,READDW(ds),16,(dw)READDW(eax)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
IN(READDBl(eax),96);
R(TEST(8,READDBl(eax),8,(db)128));
R(JNZ(ureertrteertert2));
ureertrteertert2:
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,clavier)))));
R(CMP(8,READDBl(eax),8,(db)225));
R(JNE(retertert_specialye));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,clavier_stuff2), ds)),8,(db)2));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(retertert_specialye));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pause), ds)),8,(db)1));
R(JMP(tttttttttoooooi));
retertert_specialye:
R(CMP(8,READDBl(eax),8,(db)224));
R(JNE(retertert_special));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,clavier_stuff), ds)),8,(db)1));
R(JMP(tttttttttoooooi));
retertert_special:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,clavier_stuff2), ds)),8,(db)0));
R(JE(reterrteertertter));
R(DEC(8,*((db *) realAddress(offsetof(struct Mem,clavier_stuff2), ds))));
R(JMP(tttttttttoooooi));
reterrteertertter:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,clavier_stuff), ds)),8,(db)1));
R(JNE(pas_extanded));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,clavier_stuff), ds)),8,(db)0));
R(CMP(8,READDBl(eax),8,(db)70));
R(JNE(pas_poauser));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pause), ds)),8,(db)1));
pas_poauser:
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(CMP(8,READDBl(eax),8,(db)128));
R(JB(utou));
R(AND(8,READDBl(eax),8,(db)127));
R(MOV(8,READDBl(ebx),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+READDD(ebx)), ds))));
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JE(tttttttttoooooi));
R(OR(8,READDBl(eax),8,(db)128));
R(JMP(pas_extanded));
utou:
R(MOV(8,READDBl(ebx),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+READDD(ebx)), ds))));
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JE(tttttttttoooooi));
pas_extanded:
R(CMP(8,READDBl(eax),8,(db)128));
R(JB(utouche_appuyee));
R(AND(8,READDBl(eax),8,(db)127));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(MOV(8,*((db *) realAddress(READDD(edi), ds)),8,(db)0));
R(JMP(ube_all));
utouche_appuyee:
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(MOV(8,*((db *) realAddress(READDD(edi), ds)),8,(db)1));
ube_all:
tttttttttoooooi:
R(CMP(32,READDD(eax),32,(dd)1));
R(JNE(uertterertert));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente_nouveau_esc), ds)),32,(dd)0));
R(JNE(uertterertert));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
uertterertert:
uoitreterrtyrty:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,une_touche_a_telle_ete_pressee), ds)),8,(db)1));
R(MOV(8,READDBl(eax),8,(db)32));
OUT(32,READDBl(eax));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(POPF);
RET;

//PROC controle
controle:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,taille_exe_gonfle), ds)),32,(dd)0));
R(JNE(dfgdf222gfghjktrhtkrjjrtyhjkrtyjklrty));
RET;
dfgdf222gfghjktrhtkrjjrtyhjkrtyjklrty:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,donnee2)))));
R(MOV(32,READDD(ecx),32,(dd)touches_size));
R(REP_STOSB);
R(MOV(32,READDD(ecx),32,(dd)8));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,donnee2)))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,touches_)))));
ertyertyjyuiyuiyuiiyu:
R(PUSH(32,(READDD(ecx))));
R(PUSH(32,(READDD(esi))));
R(MOV(32,read_dd(realAddress(READDD(esi), ds)),32,(dd)0));
R(MOV(16,read_dw(realAddress((READDD(esi)+4), ds)),16,(dw)0));
R(PUSH(32,(READDD(edi))));
R(CMP(32,read_dd(realAddress((READDD(edi)+(7*4)), ds)),32,(dd)((0-1))));
R(JNE(retertertrtertetyyrtuui));
CALL(controle_joueur_fleche);
retertertrtertetyyrtuui:
R(POP(32,(READDD(edi))));
R(POP(32,(READDD(esi))));
R(ADD(32,READDD(esi),32,(dd)7));
R(ADD(32,READDD(edi),32,(dd)(8*4)));
R(POP(32,(READDD(ecx))));
R(DEC(32,(READDD(ecx))));
R(JNZ(ertyertyjyuiyuiyuiiyu));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,taille_exe_gonfle), ds)),32,(dd)0));
R(JE(zappelesmachins));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+28), ds)),8,(db)1));
R(JNE(erertertertp));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,donnee2)+(8*7)), ds)),8,(db)1));
erertertertp:
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+1), ds)),8,(db)1));
R(JNE(erertertertpt));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee2)+(8*7))+1), ds)),8,(db)1));
erertertertpt:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,une_touche_a_telle_ete_pressee), ds)),8,(db)1));
R(JNE(erertertertptee));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee2)+(8*7))+2), ds)),8,(db)1));
erertertertptee:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,une_touche_a_telle_ete_pressee), ds)),8,(db)0));
zappelesmachins:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC start_cpu_player
start_cpu_player:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,differents_offset_possible)+(4*8))))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,nb_ai_bombermen), ds))));
R(SHL(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(CMP(32,READDD(ebp),32,(dd)8));
R(JE(exit_function));
R(SHL(32,READDD(ebp),32,(dd)2));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(esi)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(INC(8,*((db *) realAddress(offsetof(struct Mem,nb_ai_bombermen), ds))));
exit_function:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC menu_intelligence
menu_intelligence:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,fx)))));
R(MOV(32,READDD(ecx),32,(dd)14));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(REP_STOSW);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)0));
R(JNZ(rtbrtyjkrtklrtyrtyrtyrty));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,temps_avant_demo), ds))));
R(JNZ(rtbrtyjkrtklrtyrtyrtyrty));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,temps_avant_demo), ds)),32,(dd)ttp2));
R(JMP(rtrtytyutyutyutyuyuttyuyuttyuyutyuyuttyuyuttyuyut));
rtbrtyjkrtklrtyrtyrtyrty:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JE(rtrtytyutyutyutyuyuttyuyuttyuyutyuyuttyuyuttyuyut));
R(JMP(retyeyutyuutyyutyutyuioodfgdfgdfggdf));
rtrtytyutyutyutyuyuttyuyuttyuyutyuyuttyuyuttyuyut:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,differentesply2), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,differentesply2), ds)),32,(dd)4));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,differentesply2), ds)),32,(dd)(4*nb_sply)));
R(JNE(ertyertyuityu));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,differentesply2), ds)),32,(dd)0));
ertyertyuityu:
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,differentesply)+READDD(ebx)), ds))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,replayer_saver4), ds)),32,(dd)READDD(esi)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(MOV(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,replayer_saver4), ds))));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,replayer_saver5), ds)),8,(db)1));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,replayer_saver), ds)),32,(dd)4));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+taille_header_rec)-13), fs))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,isbigendian), ds)),8,(db)1));
R(JNE(bigendianpatchblablablatotox91));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,bigendianin), ds)),32,(dd)READDD(eax)));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,bigendianin), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+3), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+1), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+2), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+2), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+1), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+3), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+0), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,bigendianout), ds))));
bigendianpatchblablablatotox91:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,replayer_saver2), ds)),32,(dd)READDD(eax)));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+taille_header_rec)-17), fs))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,isbigendian), ds)),8,(db)1));
R(JNE(bigendianpatchblablablatotox92));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,bigendianin), ds)),32,(dd)READDD(eax)));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,bigendianin), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+3), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+1), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+2), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+2), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+1), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+3), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+0), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,bigendianout), ds))));
bigendianpatchblablablatotox92:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,replayer_saver3), ds)),32,(dd)READDD(eax)));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre), ds)),8,(db)1));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
retyeyutyuutyyutyutyuioodfgdfgdfggdf:
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,total_t)+(8*7)), ds)),8,(db)1));
R(JNE(erertertertpert));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3_sauve), ds)),8,(db)1));
R(JNE(pascolormode));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)2));
R(JA(startgame));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)14));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx94));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx94:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(erertertertpert));
pascolormode:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)1));
R(JA(startgame));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)0));
R(JNE(nozerodyna));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)14));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx95));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx95:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(erertertertpert));
nozerodyna:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,nb_ai_bombermen), ds)),8,(db)1));
R(JNE(donenottoaddacpuifonlyonecpuregistered));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)14));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx96));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx96:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(erertertertpert));
donenottoaddacpuifonlyonecpuregistered:
CALL(start_cpu_player);
startgame:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)15));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx97));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx97:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre), ds)),8,(db)1));
erertertertpert:
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(OR(32,READDD(ebp),32,(dd)READDD(ebp)));
R(JZ(yen_a_pas));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(edx),32,(dd)READDD(edx)));
ooooooooh:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)4));
R(JE(ms_dos));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(LODSD);
R(MOV(32,read_dd(realAddress(((((offsetof(struct Mem,texte1)+(6*1))+1)+(32*0))+READDD(edx)), ds)),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress(((((offsetof(struct Mem,texte1)+(6*1))+1)+(32*1))+READDD(edx)), ds)),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress(((((offsetof(struct Mem,texte1)+(6*1))+1)+(32*2))+READDD(edx)), ds)),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress(((((offsetof(struct Mem,texte1)+(6*1))+1)+(32*3))+READDD(edx)), ds)),32,(dd)READDD(eax)));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)1));
R(JB(retrterteoooshow));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)3));
R(JA(retrterteoooshow));
R(PUSH(32,(READDD(edx))));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,texte1)+(6*2))+(32*0))+READDD(edx)), ds)),32,(dd)538976288));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,texte1)+(6*2))+(32*1))+READDD(edx)), ds)),32,(dd)538976288));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,texte1)+(6*2))+(32*2))+READDD(edx)), ds)),32,(dd)538976288));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,texte1)+(6*2))+(32*3))+READDD(edx)), ds)),32,(dd)538976288));
R(ADD(32,READDD(edx),32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((((offsetof(struct Mem,texte1)+(6*2))+(32*0))+READDD(edx)), ds)),8,(db)45));
R(MOV(8,*((db *) realAddress((((offsetof(struct Mem,texte1)+(6*2))+(32*1))+READDD(edx)), ds)),8,(db)45));
R(MOV(8,*((db *) realAddress((((offsetof(struct Mem,texte1)+(6*2))+(32*2))+READDD(edx)), ds)),8,(db)45));
R(MOV(8,*((db *) realAddress((((offsetof(struct Mem,texte1)+(6*2))+(32*3))+READDD(edx)), ds)),8,(db)45));
R(POP(32,(READDD(edx))));
retrterteoooshow:
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,temps_joueur)+READDD(ebx)), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,temps_joueur)+READDD(ebx)), ds)),32,(dd)0));
R(JNE(ertterrtytyrrtyrtyrtyrtytuoooooooooo));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)+3), ds)),8,(db)0));
R(JE(pas_flechedu));
R(PUSH(32,(READDD(esi))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds))));
R(DEC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(97-1)));
R(JNE(ertrteertterrterterte));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(122+13)));
ertrteertterrterterte:
R(POP(32,(READDD(esi))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,temps_joueur)+READDD(ebx)), ds)),32,(dd)temps_re_menu));
R(JMP(finito_touches));
pas_flechedu:
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)+0), ds)),8,(db)0));
R(JE(upas_flechedu));
R(PUSH(32,(READDD(esi))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds))));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(122+14)));
R(JNE(uertrteertterrterterte));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)97));
uertrteertterrterterte:
R(POP(32,(READDD(esi))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,temps_joueur)+READDD(ebx)), ds)),32,(dd)temps_re_menu));
R(JMP(finito_touches));
upas_flechedu:
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)+1), ds)),8,(db)0));
R(JE(pas_fleched));
R(INC(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)4));
R(JNE(pertertras_flechedertertertret));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)3));
R(JMP(ooooiio));
pertertras_flechedertertertret:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)5));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruitopx98));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruitopx98:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
ooooiio:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,temps_joueur)+READDD(ebx)), ds)),32,(dd)temps_re_menu));
R(JMP(finito_touches));
pas_fleched:
R(CMP(8,*((db *) realAddress((READDD(esi)+2), ds)),8,(db)0));
R(JE(pas_flechedm));
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)0));
R(JNE(pertertras_flechedertertertretm));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)1));
R(JMP(ootoi));
pertertras_flechedertertertretm:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)5));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruitopx99));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruitopx99:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
ootoi:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,temps_joueur)+READDD(ebx)), ds)),32,(dd)temps_re_menu));
R(JMP(finito_touches));
pas_flechedm:
R(CMP(8,*((db *) realAddress((READDD(esi)+4), ds)),8,(db)1));
R(JE(actionbuttonpushedokx100));
R(CMP(8,*((db *) realAddress((READDD(esi)+5), ds)),8,(db)1));
R(JE(actionbuttonpushedokx100));
R(CMP(8,*((db *) realAddress((READDD(esi)+6), ds)),8,(db)1));
R(JE(actionbuttonpushedokx100));
R(JMP(pas_flechedmy));
actionbuttonpushedokx100:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)3));
R(JE(pertertras_flechedertertertrety));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)5));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruitopx101));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruitopx101:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(INC(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,temps_joueur)+READDD(ebx)), ds)),32,(dd)temps_re_menu));
R(JMP(finito_touches));
pertertras_flechedertertertrety:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)3));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruitopx102));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruitopx102:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebx)), ds)),32,(dd)4));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(SHR(32,READDD(eax),32,(dd)6));
R(INC(32,(READDD(eax))));
R(ADD(8,READDBl(eax),8,(db)48));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(PUSH(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(READDD(esi), ds))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message2)))));
R(MOV(32,read_dd(realAddress(READDD(esi), ds)),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress((READDD(esi)+32), ds)),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_name), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
R(MOV(8,*((db *) realAddress(((READDD(esi)+16)+(32*2)), ds)),8,(db)READDBl(eax)));
R(MOV(8,*((db *) realAddress(((READDD(esi)+16)+(32*3)), ds)),8,(db)READDBl(eax)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,texte1)))));
R(ADD(32,READDD(edi),32,(dd)READDD(edx)));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
enojjojortyrtyrtytyr:
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,love_si)+READDD(ebx)), ds))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_name), ds)),32,(dd)READDD(eax)));
R(JNE(nonon_));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,offset_si)+READDD(ebp)), ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,texte1)))));
R(ADD(32,READDD(edi),32,(dd)READDD(edx)));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(((offsetof(struct Mem,offset_si)+READDD(ebp))+4), ds))));
R(REP_MOVSD);
R(MOV(32,READDD(eax),32,(dd)1179010630));
nonon_:
R(ADD(32,READDD(ebp),32,(dd)8));
R(ADD(32,READDD(ebx),32,(dd)4));
R(CMP(32,READDD(eax),32,(dd)1179010630));
R(JNE(enojjojortyrtyrtytyr));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(finito_touches));
pas_flechedmy:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,temps_joueur)+READDD(ebx)), ds)),32,(dd)1));
finito_touches:
ertterrtytyrrtyrtyrtyrtytuoooooooooo:
ms_dos:
R(ADD(32,READDD(ebx),32,(dd)4));
R(ADD(32,READDD(edx),32,(dd)(32*4)));
R(DEC(32,(READDD(ebp))));
R(JNZ(ooooooooh));
yen_a_pas:
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(CMP(32,READDD(ebp),32,(dd)8));
R(JE(finito_trop_de_dyna));
R(SHL(32,READDD(ebp),32,(dd)2));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,differents_offset_possible)))));
eetterrterterterteertterertert:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(READDD(esi), ds))));
R(CMP(32,READDD(ebx),32,(dd)666));
R(JE(ok_on_en_a_trouve_un));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,total_t)+READDD(ebx))+4), ds)),8,(db)1));
R(JE(actionbuttonpushed2okx105));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,total_t)+READDD(ebx))+5), ds)),8,(db)1));
R(JE(actionbuttonpushed2okx105));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,total_t)+READDD(ebx))+6), ds)),8,(db)1));
R(JE(actionbuttonpushed2okx105));
R(JMP(touche_non_appuyee));
actionbuttonpushed2okx105:
R(PUSH(32,(READDD(ebp))));
R(OR(32,READDD(ebp),32,(dd)READDD(ebp)));
R(JZ(ca_roule_mon_coco));
tretrrtrtrtrtrttr:
R(SUB(32,READDD(ebp),32,(dd)4));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds)),32,(dd)READDD(ebx)));
R(JNE(reetretrert));
R(POP(32,(READDD(ebp))));
R(JMP(touche_non_appuyee));
reetretrert:
R(OR(32,READDD(ebp),32,(dd)READDD(ebp)));
R(JNZ(tretrrtrtrtrtrttr));
ca_roule_mon_coco:
R(POP(32,(READDD(ebp))));
plusieur:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds)),32,(dd)READDD(ebx)));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)3));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruitopx106));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruitopx106:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message3)))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,name_joueur)+READDD(ebp)), ds)),32,(dd)1));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,texte1)))));
R(SHL(32,READDD(ebp),32,(dd)5));
R(ADD(32,READDD(edi),32,(dd)READDD(ebp)));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(JMP(ok_on_en_a_trouve_un));
touche_non_appuyee:
R(ADD(32,READDD(esi),32,(dd)4));
R(JMP(eetterrterterterteertterertert));
finito_trop_de_dyna:
ok_on_en_a_trouve_un:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC controle_joueur_fleche
controle_joueur_fleche:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(READDD(edi), ds))));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+READDD(ebx)), ds)),8,(db)1));
R(JNE(erertertert));
R(MOV(8,*((db *) realAddress((READDD(esi)+2), ds)),8,(db)1));
erertertert:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((READDD(edi)+4), ds))));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+READDD(ebx)), ds)),8,(db)1));
R(JNE(erertertert2));
R(MOV(8,*((db *) realAddress((READDD(esi)+1), ds)),8,(db)1));
erertertert2:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((READDD(edi)+8), ds))));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+READDD(ebx)), ds)),8,(db)1));
R(JNE(erertertert3));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
erertertert3:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((READDD(edi)+12), ds))));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+READDD(ebx)), ds)),8,(db)1));
R(JNE(erertertert4));
R(MOV(8,*((db *) realAddress((READDD(esi)+3), ds)),8,(db)1));
erertertert4:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((READDD(edi)+16), ds))));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+READDD(ebx)), ds)),8,(db)1));
R(JNE(erertertert45));
R(MOV(8,*((db *) realAddress((READDD(esi)+4), ds)),8,(db)1));
erertertert45:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((READDD(edi)+20), ds))));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+READDD(ebx)), ds)),8,(db)1));
R(JNE(ererterter45));
R(MOV(8,*((db *) realAddress((READDD(esi)+5), ds)),8,(db)1));
ererterter45:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((READDD(edi)+24), ds))));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,clavier)+READDD(ebx)), ds)),8,(db)1));
R(JNE(ererterter455));
R(MOV(8,*((db *) realAddress((READDD(esi)+6), ds)),8,(db)1));
ererterter455:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC gestion_jeu
gestion_jeu:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,special_nivo_6), ds)),32,(dd)0));
R(JE(iophrehuiophuioeterterrte));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,special_nivo_6), ds))));
iophrehuiophuioeterterrte:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(ytnononono_onest_en_recordplaye));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(ytnononono_onest_en_recordplaye));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_vbl_avant_le_droit_de_poser_bombe), ds)),32,(dd)(((nombre_de_vbl_avant_le_droit_de_poser_bombe2-10))/2)));
R(JA(erteetetretrerterterter));
R(JMP(klhlkjljkjkljlkjkljkljklkljkljkljkljklj));
ytnononono_onest_en_recordplaye:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_vbl_avant_le_droit_de_poser_bombe), ds)),32,(dd)(nombre_de_vbl_avant_le_droit_de_poser_bombe2-10)));
R(JA(erteetetretrerterterter));
klhlkjljkjkljlkjkljkljklkljkljkljkljklj:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
ertyrtyutyutyutyuioooppp:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino6)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(ertytyuyututyuyyuiyui));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino6)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)1));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)duree_saut));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)1));
ertytyuyututyuyyuiyui:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino6)+READDD(ebp)), ds)),32,(dd)2));
R(JNE(ertytyuyututyuyyuiyuir));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino6)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
ertytyuyututyuyyuiyuir:
R(ADD(32,READDD(ebp),32,(dd)4));
R(CMP(32,READDD(ebp),32,(dd)(4*8)));
R(JNE(ertyrtyutyutyutyuioooppp));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
CALL(gestion_pause);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds)),8,(db)0));
R(JE(erteetetretrerterterter));
CALL(donnee_to_donnee4);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
erteetetretrerterterter:
CALL(transmet_central);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,blow_what2)))));
R(MOV(32,READDD(ecx),32,(dd)14));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(REP_STOSW);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
CALL(dec_temps);
CALL(gestion_bdraw);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(nononono_onest_en_recordplaye));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(nononono_onest_en_recordplaye));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice2), ds)),8,(db)1));
R(JE(nononono_onest_en_recordplaye));
CALL(dec_temps);
CALL(gestion_bdraw);
nononono_onest_en_recordplaye:
CALL(fabrique_monstro_truc);
CALL(gestion_blanchiment);
CALL(contamination);
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
brouter:
R(PUSH(32,(READDD(ecx))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,liste_couleur)+READDD(ebp)), ds))));
CALL(gestion_lapin);
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,patineur)+READDD(ebp)), ds)),32,(dd)0));
R(JE(ooooooooooooooooooooooooooooooooooooooooo));
CALL(gestion_lapin);
ooooooooooooooooooooooooooooooooooooooooo:
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)1));
R(JNE(oooooooooooooooooooooooooooooooooooooooooe));
CALL(gestion_lapin);
oooooooooooooooooooooooooooooooooooooooooe:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(tnononono_onest_en_recordplaye));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(tnononono_onest_en_recordplaye));
CALL(gestion_lapin);
tnononono_onest_en_recordplaye:
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)0));
R(JE(ertterterrterterte));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(ertterterrterterte));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,maladie)+READDD(ebp))+2), ds))));
R(AND(32,READDD(eax),32,(dd)1023));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,blinking)+READDD(eax)), ds)),8,(db)0));
R(JNZ(ertterterrterterte));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,liste_couleur_malade)+READDD(ebp)), ds))));
ertterterrterterte:
CALL(anim_un_joeur);
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(erertert));
CALL(touches_action);
CALL(la_mort);
erertert:
R(ADD(32,READDD(ebp),32,(dd)4));
R(POP(32,(READDD(ecx))));
R(DEC(32,(READDD(ecx))));
R(JNZ(brouter));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_monstres), ds)),32,(dd)0));
R(JE(y_en_a_pas));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_monstres), ds))));
tbrouter:
R(PUSH(32,(READDD(ecx))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(next_monstre));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,blocage)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(ertreteretterter));
CALL(intelligence_monstre);
ertreteretterter:
CALL(la_mort_monstre);
next_monstre:
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,liste_couleur)+READDD(ebp)), ds))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
CALL(anim_un_joeur);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(next_monstre2));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,blocage)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(next_monstre2));
R(JMP(anti_bomb_monstre));
nonononononononon:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)1));
R(JE(retererttzerer));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,touches_save)+READDD(ebp)), ds))));
R(OR(32,READDD(eax),32,(dd)128));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
ouiuouiouiuoi:
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,liste_couleur)+READDD(ebp)), ds))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
CALL(anim_un_joeur);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
retererttzerer:
next_monstre2:
R(ADD(32,READDD(ebp),32,(dd)4));
R(POP(32,(READDD(ecx))));
R(DEC(32,(READDD(ecx))));
R(JNZ(tbrouter));
y_en_a_pas:
CALL(minuteur);
CALL(monsieur_bombe);
CALL(monsieur_brik);
CALL(calc_ombres);
CALL(phase);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)3));
R(JNE(reertertertertert));
CALL(donnee_to_donnee4);
reertertertertert:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
gestion_lapin:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(ertyrttyrrtytyuutyyuiyuiiyuyuityuioouiioyuuioyyuioe));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,lapin_mania1)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
R(JE(ertrterterteert));
ertrterterteertzerzererzzer:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino4)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)0));
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds))));
R(JNZ(erterterterterertrterterteert));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)3));
R(JNE(rttyuooooooo));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino6)+READDD(ebp)), ds)),32,(dd)2));
R(JMP(ertrterterteert));
rttyuooooooo:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
R(JMP(ertrterterteert));
erterterterterertrterterteert:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)(duree_saut-15)));
R(JA(trtyrtyrtyrtyrtyrty));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)15));
R(JB(trtyrtyrtyrtyrtyrty));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)3));
R(JE(ertertertrterterteertzerzererzzerrtyrtrtyyrtrty));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,lapin_mania3)+READDD(ebp)), ds))));
ertertertrterterteertzerzererzzerrtyrtrtyyrtrty:
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds))));
R(SUB(32,READDD(ebx),32,(dd)15));
R(ADD(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,saut_de_lapin)+READDD(ebx)), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(ertrterterteertererer));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,saut_de_lapin2)+READDD(ebx)), ds))));
ertrterterteertererer:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(EAX_X_320);
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino4)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)(duree_saut-15)));
R(JE(ertrterterteert));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)2));
R(JNE(ertrterterteert));
R(PUSH(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(JNB(ertrterterteert_oooo));
R(POP(32,(READDD(eax))));
R(JMP(ertrterterteert));
ertrterterteert_oooo:
R(POP(32,(READDD(eax))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,(dd)READDD(ebp)));
R(SHR(32,READDD(ebx),32,(dd)1));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds))));
R(AND(32,READDD(eax),32,(dd)127));
R(CMP(32,READDD(eax),32,(dd)8));
R(JNE(trrtertyrtytyrtyrrty));
R(INC(16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebx)), ds))));
trrtertyrtytyrtyrrty:
R(CMP(32,READDD(eax),32,(dd)16));
R(JNE(trrtertyrtytyrtyrrtyr));
R(DEC(16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebx)), ds))));
trrtertyrtytyrtyrrtyr:
R(CMP(32,READDD(eax),32,(dd)0));
R(JNE(trrtertyrtytyrtyrrtyy));
R(INC(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(8*2))+READDD(ebx)), ds))));
trrtertyrtytyrtyrrtyy:
R(CMP(32,READDD(eax),32,(dd)24));
R(JNE(trrtertyrtytyrtyrrtyry));
R(DEC(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(8*2))+READDD(ebx)), ds))));
trrtertyrtytyrtyrrtyry:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(eax))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(32,(READDD(ebp))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(POP(32,(READDD(ebp))));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,truc)+READDD(eax))))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+READDD(ebp)), ds)),32,(dd)READDD(esi)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(ertrterterteert));
trtyrtyrtyrtyrtyrty:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)(duree_saut-7)));
R(JA(trtyrtyrtyrtyrtyrtyu));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)7));
R(JB(trtyrtyrtyrtyrtyrtyu));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,lapin_mania4)+READDD(ebp)), ds))));
R(JMP(ertrterterteert));
trtyrtyrtyrtyrtyrtyu:
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,lapin_mania2)+READDD(ebp)), ds))));
ertrterterteert:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)3));
R(JNE(kertrterterteert));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds))));
R(ADD(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,mort_de_lapin)+READDD(ebx)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(EAX_X_320);
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino4)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,lapin_mania5)+READDD(ebp)), ds))));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(eax))));
kertrterterteert:
ertyrttyrrtytyuutyyuiyuiiyuyuityuioouiioyuuioyyuioe:
RET;
gestion_pause:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds)),8,(db)0));
R(JE(pas_en_pauseer));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)15));
R(JNZ(pas_en_pauseer));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds))));
R(ADD(32,READDD(ebx),32,(dd)4));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,offset_pause)+READDD(ebx)), ds)),32,(dd)666));
R(JNE(ihhihiertteretr));
R(MOV(32,READDD(ebx),32,(dd)4));
ihhihiertteretr:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds)),8,(db)READDBl(ebx)));
pas_en_pauseer:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,pause2), ds)),8,(db)0));
R(JE(uerertertert));
R(DEC(8,*((db *) realAddress(offsetof(struct Mem,pause2), ds))));
uerertertert:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,pause), ds)),8,(db)1));
R(JNE(raison_detat));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pause), ds)),8,(db)0));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,pause2), ds)),8,(db)0));
R(JNE(dejapresseyapaslongtemps));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pause2), ds)),8,(db)15));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds)),8,(db)0));
R(JNE(reeertertterert));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds)),8,(db)4));
R(JMP(raison_detat));
reeertertterert:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds)),8,(db)0));
raison_detat:
dejapresseyapaslongtemps:
RET;
anti_bomb_monstre:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(nonononononononon));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(32,(READDD(ebp))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,truc2)+READDD(eax))))));
R(POP(32,(READDD(ebp))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)5));
R(JB(ertterertertertertt));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JNB(ertterertertertertt));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds))));
R(AND(32,READDD(ebx),32,(dd)127));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,anti_bomb)+READDD(ebx)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(ouiuouiouiuoi));
ertterertertertertt:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(nonononononononon));
intelligence_monstre:
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress((READDD(esi)+12), ds))));
R(CMP(32,READDD(ecx),32,(dd)1));
R(JNE(pas_1i));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(non_bouge_pasttryrtytyr));
R(JMP(ok_il_bougert));
pas_1i:
R(CMP(32,READDD(ecx),32,(dd)2));
R(JNE(ok_il_bougert));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(non_bouge_pasttryrtytyr));
R(JMP(ok_il_bougert));
non_bouge_pasttryrtytyr:
RET;
ok_il_bougert:
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(ADD(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,avance2)+READDD(ebp)), ds))));
R(ADD(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,viseur_change_in)+READDD(ebp)), ds))));
R(ADD(32,READDD(eax),32,(dd)READDD(ebp)));
R(AND(32,READDD(eax),32,(dd)127));
R(JNZ(alllagrishna));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,avance2)+READDD(ebp)), ds)),32,(dd)3));
R(JB(arrete__));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance2)+READDD(ebp)), ds)),32,(dd)3));
R(JMP(arrete__));
alllagrishna:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)1));
R(JE(reterertter));
arrete__:
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,avance2)+READDD(ebp)), ds))));
R(JNZ(mendier));
hier2:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance2)+READDD(ebp)), ds)),32,(dd)15));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,viseur_change_in)+READDD(ebp)), ds))));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,changeiny)+READDD(ebx)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,viseur_change_in)+READDD(ebp)), ds)),32,(dd)4));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,viseur_change_in)+READDD(ebp)), ds)),32,(dd)(16*4)));
R(JNE(retkortykokoptrkopkop));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,viseur_change_in)+READDD(ebp)), ds)),32,(dd)0));
retkortykokoptrkopkop:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(mendier));
reterertter:
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches_save)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
mendier:
RET;
donnee_to_donnee4:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
hooooooop:
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebx)), ds))));
R(PUSH(32,(READDD(eax))));
R(SHL(32,READDD(eax),32,(dd)6));
R(MOV(32,READDD(edi),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
R(SHL(32,READDD(eax),32,(dd)8));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebx)), ds))));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,vise_de_ca_haut)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,vise_de_ca_haut2)+READDD(ebp)), ds)),32,(dd)0));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(ertrtyyuttyutyurtyutyutyuyutyut));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebx)), ds))));
R(MOV(32,READDD(edx),32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds))));
R(ADD(32,READDD(edx),32,(dd)13));
R(CMP(16,READDW(eax),16,(dw)READDW(edx)));
R(JA(erertzertertertertertert));
R(NEG(32,(READDD(eax))));
R(ADD(32,READDD(eax),32,(dd)READDD(edx)));
R(INC(32,(READDD(eax))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,vise_de_ca_haut2)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(EAX_X_320);
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,vise_de_ca_haut)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
erertzertertertertertert:
R(POP(32,(READDD(eax))));
R(ADD(32,READDD(edi),32,(dd)((((0-14))*320)-4)));
R(SUB(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,lapipipino4)+READDD(ebp)), ds))));
R(OR(32,READDD(edi),32,(dd)READDD(edi)));
R(JNS(rterteerertertterteryuyyuuuuu));
R(ADD(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,lapipipino4)+READDD(ebp)), ds))));
R(PUSH(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,lapipipino4)+READDD(ebp)), ds))));
R(POP(32,(READDD(eax))));
rterteerertertterteryuyyuuuuu:
R(JMP(tryrtyyrttyutyuyuttyutyutyutyutyutyutyu));
ertrtyyuttyutyurtyutyutyuyutyut:
R(ADD(32,READDD(edi),32,read_dd(realAddress(((offsetof(struct Mem,donnee)+112)+READDD(ebp)), ds))));
tryrtyyrttyutyuyuttyutyutyutyutyutyutyu:
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,donnee4)+READDD(ecx))+4), ds)),16,(dw)READDW(edi)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*4))+READDD(ebx)), ds))));
R(CMP(16,READDW(eax),16,(dw)666));
R(JE(reertertertrte));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(ertyrttyrrtytyuutyyuiyuiiyuyuityuioouiioyuuioyyuio));
R(ADD(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,vise_de_ca_haut)+READDD(ebp)), ds))));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)0));
R(JE(ertterterrtertertertt));
R(MOV(16,READDW(edx),16,read_dw(realAddress(((offsetof(struct Mem,maladie)+READDD(ebp))+2), ds))));
R(AND(32,READDD(edx),32,(dd)1023));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,blinking)+READDD(edx)), ds)),8,(db)0));
R(JNZ(ertterterrtertertertt));
R(ADD(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,lapin_mania_malade)+READDD(ebp)), ds))));
R(JMP(reertertertrte));
ertterterrtertertertt:
R(ADD(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,lapin_mania)+READDD(ebp)), ds))));
R(JMP(reertertertrte));
ertyrttyrrtytyuutyyuiyuiiyuyuityuioouiioyuuioyyuio:
R(ADD(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,donnee)+((8*2)*3))+READDD(ebp)), ds))));
reertertertrte:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,donnee4)+READDD(ecx)), ds)),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+((8*5)*2))+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+6)+READDD(ecx)), ds)),8,(db)READDBl(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+((8*6)*2))+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+7)+READDD(ecx)), ds)),8,(db)READDBl(eax)));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(rertyrttyrrtytyuutyyuiyuiiyuyuityuioouiioyuuioyyuio));
R(PUSH(16,(READDW(eax))));
R(MOV(16,READDW(eax),16,(dw)37));
R(SUB(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,vise_de_ca_haut2)+READDD(ebp)), ds))));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+6)+READDD(ecx)), ds)),8,(db)READDBl(eax)));
R(POP(16,(READDW(eax))));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+7)+READDD(ecx)), ds)),8,(db)32));
rertyrttyrrtytyuutyyuiyuiiyuyuityuioouiioyuuioyyuio:
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,clignotement)+READDD(ebp)), ds))));
R(AND(8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+8)+READDD(ecx)), ds)),8,(db)254));
R(OR(8,*((db *) realAddress(((offsetof(struct Mem,donnee4)+8)+READDD(ecx)), ds)),8,(db)READDBl(eax)));
R(ADD(32,READDD(ecx),32,(dd)nb_unite_donnee4));
R(ADD(32,READDD(ebx),32,(dd)2));
R(ADD(32,READDD(ebp),32,(dd)4));
R(CMP(32,READDD(ebp),32,(dd)(4*8)));
R(JNE(hooooooop));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
la_mort:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(32,(READDD(ebp))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,truc2)+READDD(eax))))));
R(POP(32,(READDD(ebp))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JE(nan_comme_dab));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(nan_laisse_tourner));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)11));
R(JNE(nan_laisse_tourner));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)3));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)duree_mort));
R(JMP(nan_laisse_tourner));
nan_comme_dab:
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)11));
R(JE(microsoft));
nan_laisse_tourner:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)5));
R(JB(ertterertertertert));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JNB(ertterertertertert));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(ertterertertertert));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistanceerterdynanormalitox133));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistancepassaut2x133));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(ertterertertertert));
resistancepassaut2x133:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)12));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx432));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx432:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)34));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)3));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)duree_mort));
resistanceerterdynanormalitox133:
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((READDD(ebx)+(4*4)), ds)),32,(dd)0));
CALL(nike_toutes_ses_bombes);
R(POP(32,(READDD(ebx))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistancefinito_babyx133));
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)invisibilite_totale));
R(AND(32,read_dd(realAddress((offsetof(struct Mem,pousseur)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,patineur)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,tribombe)+READDD(ebp)), ds)),32,(dd)0));
R(JMP(ertterertertertert));
resistancefinito_babyx133:
microsoft:
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((READDD(ebx)+(4*4)), ds)),32,(dd)0));
CALL(nike_toutes_ses_bombes);
R(POP(32,(READDD(ebx))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)7));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx134));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx134:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)35));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)0));
ertterertertertert:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JE(on_est_pas_un_lapin_on_mange_les_bonus));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(baaaaaaaaaaaaaaaaaaaaaaaah_paas_de_bonus));
on_est_pas_un_lapin_on_mange_les_bonus:
R(PUSH(32,(READDD(esi))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JB(bonus_yertterertertertertx135));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(54+10)));
R(JNB(bonus_yertterertertertertx135));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx433));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx433:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+0), ds)),32,(dd)bombe_max));
R(JE(bonus_fireuertkjertjertkljertertertertter2x135));
R(INC(32,read_dd(realAddress((READDD(esi)+0), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+0), ds)),32,(dd)bombe_max));
R(JE(bonus_yertterertertertertx135));
R(PUSH(32,(READDD(esi))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)0));
R(POP(32,(READDD(esi))));
R(JNE(bonus_yertterertertertertx135));
R(INC(32,read_dd(realAddress((READDD(esi)+0), ds))));
bonus_yertterertertertertx135:
R(POP(32,(READDD(esi))));
R(JMP(bonus_fireuertkjertjertkljertertertertterx135));
bonus_fireuertkjertjertkljertertertertter2x135:
R(POP(32,(READDD(esi))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx434));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx434:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
bonus_fireuertkjertjertkljertertertertterx135:
R(PUSH(32,(READDD(esi))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)64));
R(JB(bonus_yertterertertertertx136));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(64+10)));
R(JNB(bonus_yertterertertertertx136));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx435));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx435:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)bombe_max2));
R(JE(bonus_fireuertkjertjertkljertertertertter2x136));
R(INC(32,read_dd(realAddress((READDD(esi)+4), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)bombe_max2));
R(JE(bonus_yertterertertertertx136));
R(PUSH(32,(READDD(esi))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)0));
R(POP(32,(READDD(esi))));
R(JNE(bonus_yertterertertertertx136));
R(INC(32,read_dd(realAddress((READDD(esi)+4), ds))));
bonus_yertterertertertertx136:
R(POP(32,(READDD(esi))));
R(JMP(bonus_fireuertkjertjertkljertertertertterx136));
bonus_fireuertkjertjertkljertertertertter2x136:
R(POP(32,(READDD(esi))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx436));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx436:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
bonus_fireuertkjertjertkljertertertertterx136:
R(PUSH(32,(READDD(eax))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)74));
R(JB(bonus_teteyertterertertertertx137));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(74+10)));
R(JNB(bonus_teteyertterertertertertx137));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx437));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx437:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)15));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,hazard_maladie)+READDD(eax)), ds))));
R(AND(16,READDW(eax),16,(dw)255));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)READDW(eax)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+READDD(ebp))+2), ds)),16,(dw)duree_conta));
bonus_teteyertterertertertertx137:
R(POP(32,(READDD(eax))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)84));
R(JB(bonus_2yertterertertertertx138));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(84+10)));
R(JNB(bonus_2yertterertertertertx138));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx438));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx438:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)invinsibilite_bonus));
R(PUSH(32,(READDD(esi))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)+4), ds)),8,(db)3));
R(POP(32,(READDD(esi))));
R(JNE(bonus_2yertterertertertertx138));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)invinsibilite_bonus));
bonus_2yertterertertertertx138:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)94));
R(JB(bonus_2yertterertertertertx139));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(94+10)));
R(JNB(bonus_2yertterertertertertx139));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx439));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx439:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds)),32,(dd)1));
R(PUSH(32,(READDD(esi))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)+4), ds)),8,(db)3));
R(POP(32,(READDD(esi))));
R(JNE(bonus_2yertterertertertertx139));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds)),32,(dd)1));
bonus_2yertterertertertertx139:
R(PUSH(32,(READDD(esi))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)104));
R(JB(bonus_yertterertertertertx140));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(104+10)));
R(JNB(bonus_yertterertertertertx140));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx440));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx440:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+(4*4)), ds)),32,(dd)1));
R(JE(bonus_fireuertkjertjertkljertertertertter2x140));
R(INC(32,read_dd(realAddress((READDD(esi)+(4*4)), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+(4*4)), ds)),32,(dd)1));
R(JE(bonus_yertterertertertertx140));
R(PUSH(32,(READDD(esi))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)0));
R(POP(32,(READDD(esi))));
R(JNE(bonus_yertterertertertertx140));
R(INC(32,read_dd(realAddress((READDD(esi)+(4*4)), ds))));
bonus_yertterertertertertx140:
R(POP(32,(READDD(esi))));
R(JMP(bonus_fireuertkjertjertkljertertertertterx140));
bonus_fireuertkjertjertkljertertertertter2x140:
R(POP(32,(READDD(esi))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx441));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx441:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
bonus_fireuertkjertjertkljertertertertterx140:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)114));
R(JB(bonus_3yertterertertertertx141));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(114+10)));
R(JNB(bonus_3yertterertertertertx141));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,pousseur)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(bonus_3rteelmkklmertklmertklmertertterterx141));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx442));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx442:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(bonus_3yertterertertertertx141));
bonus_3rteelmkklmertklmertklmertertterterx141:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx443));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx443:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,pousseur)+READDD(ebp)), ds)),32,(dd)1));
bonus_3yertterertertertertx141:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)124));
R(JB(bonus_3yertterertertertertx142));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(124+10)));
R(JNB(bonus_3yertterertertertertx142));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,patineur)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(bonus_3rteelmkklmertklmertklmertertterterx142));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx444));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx444:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(bonus_3yertterertertertertx142));
bonus_3rteelmkklmertklmertklmertertterterx142:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx445));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx445:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,patineur)+READDD(ebp)), ds)),32,(dd)1));
bonus_3yertterertertertertx142:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)134));
R(JB(bonus_4yertterertertertertx143));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(134+10)));
R(JNB(bonus_4yertterertertertertx143));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx446));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx446:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(TEST(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)4095));
R(JNZ(bonus_4pas_zeroerrterteertx143));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx447));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx447:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(bonus_4yertterertertertertx143));
bonus_4pas_zeroerrterteertx143:
R(PUSH(16,(READDW(eax))));
R(PUSH(16,(READDW(ebx))));
R(MOV(16,READDW(eax),16,read_dw(realAddress(offsetof(struct Mem,temps), ds))));
R(MOV(16,READDW(ebx),16,(dw)READDW(eax)));
R(AND(16,READDW(ebx),16,(dw)3840));
R(CMP(16,READDW(ebx),16,(dw)(9*256)));
R(JE(non_fait_rien));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,special_clignotement), ds)),32,(dd)2));
R(ADD(16,READDW(eax),16,(dw)256));
non_fait_rien:
R(MOV(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)READDW(eax)));
R(POP(16,(READDW(ebx))));
R(POP(16,(READDW(eax))));
bonus_4yertterertertertertx143:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)144));
R(JB(bonus_3yertterertertertertx144));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(144+10)));
R(JNB(bonus_3yertterertertertertx144));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,tribombe)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(bonus_3rteelmkklmertklmertklmertertterterx144));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx448));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx448:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(bonus_3yertterertertertertx144));
bonus_3rteelmkklmertklmertklmertertterterx144:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx449));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx449:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,tribombe)+READDD(ebp)), ds)),32,(dd)1));
bonus_3yertterertertertertx144:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)154));
R(JB(bonus_6yertterertertertertx145));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)(154+10)));
R(JNB(bonus_6yertterertertertertx145));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx450));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx450:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
CALL(nike_toutes_les_bombes);
bonus_6yertterertertertertx145:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)193));
R(JNE(bonus_5yertterertertertertx146));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(bonus_5rteelmkklmertklmertklmertertterterx146));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx451));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx451:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(bonus_5yertterertertertertx146));
bonus_5rteelmkklmertklmertklmertertterterx146:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(au_milieu_x_et_ywqtreertterx452));
R(CMP(16,READDW(eax),16,(dw)7));
R(JNE(au_milieu_x_et_ypas_milieux452));
au_milieu_x_et_ywqtreertterx452:
R(CMP(16,READDW(eax),16,(dw)4));
R(JB(au_milieu_x_et_ypas_milieux452));
R(CMP(16,READDW(eax),16,(dw)10));
R(JA(au_milieu_x_et_ypas_milieux452));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)3));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(au_milieu_x_et_ywqtreertter2x452));
R(CMP(16,READDW(eax),16,(dw)7));
R(JNE(au_milieu_x_et_ypas_milieux452));
au_milieu_x_et_ywqtreertter2x452:
R(CMP(16,READDW(eax),16,(dw)4));
R(JB(au_milieu_x_et_ypas_milieux452));
R(CMP(16,READDW(eax),16,(dw)10));
R(JA(au_milieu_x_et_ypas_milieux452));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(au_milieu_x_et_ybooohx452));
au_milieu_x_et_ypas_milieux452:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(bonus_5yertterertertertertx146));
au_milieu_x_et_ybooohx452:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)1));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx453));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx453:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino6)+READDD(ebp)), ds)),32,(dd)1));
R(INC(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)10));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx454));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx454:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)30));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
bonus_5yertterertertertertx146:
baaaaaaaaaaaaaaaaaaaaaaaah_paas_de_bonus:
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_monstres), ds)),32,(dd)0));
R(JE(y_en_a_pas2));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(SHL(32,READDD(ebx),32,(dd)2));
reetrertetert:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebx)), ds)),32,(dd)1));
R(JNE(pas_tue));
R(CMP(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+READDD(ebx)), ds))));
R(JNE(pas_tue));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(pas_tue));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistanceerterdynanormalitox147));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistancepassaut2x147));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(pas_tue));
resistancepassaut2x147:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)12));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx455));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx455:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)34));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)3));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)duree_mort));
resistanceerterdynanormalitox147:
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((READDD(ebx)+(4*4)), ds)),32,(dd)0));
CALL(nike_toutes_ses_bombes);
R(POP(32,(READDD(ebx))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistancefinito_babyx147));
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)invisibilite_totale));
R(AND(32,read_dd(realAddress((offsetof(struct Mem,pousseur)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,patineur)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,tribombe)+READDD(ebp)), ds)),32,(dd)0));
R(JMP(pas_tue));
resistancefinito_babyx147:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)7));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx148));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx148:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)35));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)0));
pas_tue:
R(ADD(32,READDD(ebx),32,(dd)4));
R(CMP(32,READDD(ebx),32,(dd)32));
R(JNE(reetrertetert));
y_en_a_pas2:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
la_mort_monstre:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(32,(READDD(ebp))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,truc2)+READDD(eax))))));
R(POP(32,(READDD(ebp))));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)11));
R(JE(microsoft2));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)5));
R(JB(uertterertertertert));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JNB(uertterertertertert));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(uertterertertertert));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistanceerterdynanormalitox152));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistancepassaut2x152));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(uertterertertertert));
resistancepassaut2x152:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)12));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx456));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx456:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)34));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)3));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)duree_mort));
resistanceerterdynanormalitox152:
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((READDD(ebx)+(4*4)), ds)),32,(dd)0));
CALL(nike_toutes_ses_bombes);
R(POP(32,(READDD(ebx))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds)),32,(dd)0));
R(JE(resistancefinito_babyx152));
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,nombre_de_coups)+READDD(ebp)), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)invisibilite_totale));
R(AND(32,read_dd(realAddress((offsetof(struct Mem,pousseur)+READDD(ebp)), ds)),32,(dd)((0-1))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,patineur)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,tribombe)+READDD(ebp)), ds)),32,(dd)0));
R(JMP(uertterertertertert));
resistancefinito_babyx152:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(SUB(32,READDD(esi),32,(dd)((32*13)-1)));
R(CMP(8,*((db *) realAddress(((READDD(esi)-1)+(32*13)), ds)),8,(db)0));
R(JE(colle_un_bonusdrtytyrrtyrteterertert2x154));
R(CMP(8,*((db *) realAddress(((READDD(esi)-1)+(32*13)), ds)),8,(db)5));
R(JB(colle_un_bonusdrtytyrrtyrteterertertx154));
colle_un_bonusdrtytyrrtyrteterertert2x154:
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,viseur_hazard_bonus2), ds))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,hazard_bonus2)))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(ADD(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,viseur_hazard_bonus2), ds))));
R(CMP(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,viseur_hazard_bonus2)))));
R(JB(colle_un_bonusreertertertx154));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_hazard_bonus2), ds)),32,(dd)0));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,hazard_bonus2)))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
colle_un_bonusreertertertx154:
R(MOV(8,READDBl(eax),8,*((db *) realAddress(READDD(esi), ds))));
R(POP(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,correspondance_bonus2)+READDD(ebx)), ds))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(colle_un_bonuspas_rectx154));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(PUSH(32,(READDD(esi))));
R(MOV(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,replayer_saver4), ds))));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,replayer_saver5), ds))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(((READDD(esi)+taille_header_rec)+READDD(ebx)), fs))));
R(INC(8,*((db *) realAddress(offsetof(struct Mem,replayer_saver5), ds))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(ebx))));
colle_un_bonuspas_rectx154:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,special_nivo_6), ds)),32,(dd)0));
R(JZ(colle_un_bonusuihuiuihhuiouiohuihuiorteerrtyx154));
R(XOR(8,READDBl(eax),8,(db)READDBl(eax)));
colle_un_bonusuihuiuihhuiouiohuihuiorteerrtyx154:
R(MOV(8,*((db *) realAddress(((READDD(esi)-1)+(32*13)), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(eax))));
colle_un_bonusdrtytyrrtyrteterertertx154:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
microsoft2:
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)8));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx156));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx156:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)35));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)0));
uertterertertertert:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
anim_un_joeur:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(rtertterterrterte));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)128));
R(JB(okbouge_pas));
R(AND(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)127));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds))));
R(OR(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)128));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna_x4), ds)),32,(dd)READDD(ebp)));
R(JA(non_c_pas_un_monstre2));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)7));
R(JNE(special_monstres_gigoteurs4));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(ebx),32,(dd)24));
R(SHR(32,READDD(ebx),32,(dd)2));
R(CMP(16,READDW(ebx),16,(dw)2));
R(JNE(trhjllhjkrtlhjrhjkltyrlhjkrty));
R(MOV(16,READDW(eax),16,(dw)(((0-17))*2)));
trhjllhjkrtlhjrhjkltyrlhjkrty:
R(CMP(16,READDW(ebx),16,(dw)6));
R(JNE(trhjllhjkrtlhjrhjkltyrlhjkrty2));
R(MOV(16,READDW(eax),16,(dw)((0-17))));
trhjllhjkrtlhjrhjkltyrlhjkrty2:
R(ADD(16,READDW(eax),16,read_dw(realAddress(READDD(esi), ds))));
R(POP(32,(READDD(ebx))));
R(JMP(rtrtyyrtrtytrytyrrtyzerzerzerzer));
special_monstres_gigoteurs4:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)3));
R(JE(special_monstres_gigoteurs));
non_c_pas_un_monstre2:
R(JMP(rtrtyyrtrtytrytyrrty));
okbouge_pas:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna_x4), ds)),32,(dd)READDD(ebp)));
R(JNA(ui));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)4));
R(JNE(ui));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)8));
R(JNE(tyuo));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)16));
R(JMP(ui));
tyuo:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)16));
R(JNE(tyuu));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)8));
R(JMP(ui));
tyuu:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(tyuo2));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)24));
R(JMP(ui));
tyuo2:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)24));
R(JNE(tyuu1));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)0));
R(JMP(ui));
tyuu1:
ui:
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds))));
special_monstres_gigoteurs:
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna_x4), ds)),32,(dd)READDD(ebp)));
R(JA(non_c_pas_un_monstre));
R(PUSH(32,(READDD(ecx))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,vitesse_monstre)+READDD(ebp)), ds))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(eanononono_onest_en_recordplaye));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(eanononono_onest_en_recordplaye));
R(INC(32,(READDD(ecx))));
eanononono_onest_en_recordplaye:
R(MOV(32,READDD(ebx),32,(dd)3));
R(SHL(32,READDD(ebx),8,(db)READDBl(ecx)));
R(AND(32,READDD(eax),32,(dd)READDD(ebx)));
R(DEC(8,(READDBl(ecx))));
R(SHR(32,READDD(eax),8,(db)READDBl(ecx)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ecx))));
R(JMP(eanononono_onest_en_recordplay));
non_c_pas_un_monstre:
R(AND(32,READDD(eax),32,(dd)48));
R(SHR(32,READDD(eax),32,(dd)3));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,patineur)+READDD(ebp)), ds)),32,(dd)0));
R(JE(tryyyyyyyyyyytyuiouiiuoouiuiooui));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)24));
R(SHR(32,READDD(eax),32,(dd)2));
tryyyyyyyyyyytyuiouiiuoouiuiooui:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(eanononono_onest_en_recordplay));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(eanononono_onest_en_recordplay));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)24));
R(SHR(32,READDD(eax),32,(dd)2));
eanononono_onest_en_recordplay:
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
rtrtyyrtrtytrytyrrty:
R(MOV(16,READDW(eax),16,read_dw(realAddress(READDD(esi), ds))));
rtrtyyrtrtytrytyrrtyzerzerzerzer:
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*4))+READDD(ebp)), ds)),16,(dw)READDW(eax)));
R(POP(32,(READDD(ebp))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)128));
R(JNB(non_bouge_pas));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress((READDD(esi)+12), ds))));
R(CMP(32,READDD(ecx),32,(dd)1));
R(JNE(pas_1));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(non_bouge_pas));
R(JMP(finto_gestion_vitesse));
pas_1:
R(CMP(32,READDD(ecx),32,(dd)2));
R(JNE(pas_2));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(non_bouge_pas));
R(MOV(32,READDD(ecx),32,(dd)1));
R(JMP(finto_gestion_vitesse));
pas_2:
R(CMP(32,READDD(ecx),32,(dd)3));
R(JNE(pas_3));
R(MOV(32,READDD(ecx),32,(dd)1));
R(JMP(finto_gestion_vitesse));
pas_3:
R(CMP(32,READDD(ecx),32,(dd)4));
R(JNE(pas_4));
R(MOV(32,READDD(ecx),32,(dd)2));
R(JMP(finto_gestion_vitesse));
pas_4:
finto_gestion_vitesse:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(anononono_onest_en_recordplay));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(anononono_onest_en_recordplay));
R(INC(32,(READDD(ecx))));
anononono_onest_en_recordplay:
R(ADD(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,patineur)+READDD(ebp)), ds))));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)1));
R(JNE(ertertrterteterrteert2));
R(ADD(32,READDD(ecx),32,(dd)3));
ertertrterteterrteert2:
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)2));
R(JNE(ertertrterteterrteert3));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(non_bouge_pas));
ertertrterteterrteert3:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,blocage)+READDD(ebp)), ds)),32,(dd)0));
R(JE(next_monstre2r2));
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,blocage)+READDD(ebp)), ds))));
R(JMP(non_bouge_pas));
next_monstre2r2:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)1));
ertertrterteterrteert:
R(PUSH(32,(READDD(ecx))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)8));
R(JNE(ererterertert4));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(esi),16,(dw)16));
CALL(remet_milieu_y);
R(JZ(errteterertertrte3));
R(MOV(16,READDW(edx),16,(dw)(8+1)));
R(MOV(16,READDW(ecx),16,(dw)0));
CALL(possible_ou_pas);
R(JNZ(errteterertertrte2));
R(INC(16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(JMP(errteterertertrte3));
errteterertertrte2:
R(PUSH(32,(READDD(ebp))));
R(ADD(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)0));
R(POP(32,(READDD(ebp))));
errteterertertrte3:
R(POP(32,(READDD(ebp))));
ererterertert4:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)16));
R(JNE(ererterertert4e));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(esi),16,(dw)((0-16))));
CALL(remet_milieu_y);
R(JZ(errteterertertrte3e));
R(MOV(16,READDW(edx),16,(dw)((0-8))));
R(MOV(16,READDW(ecx),16,(dw)0));
CALL(possible_ou_pas);
R(JNZ(errteterertertrte2e));
R(DEC(16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(JMP(errteterertertrte3e));
errteterertertrte2e:
R(PUSH(32,(READDD(ebp))));
R(ADD(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)0));
R(POP(32,(READDD(ebp))));
errteterertertrte3e:
R(POP(32,(READDD(ebp))));
ererterertert4e:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)24));
R(JNE(ererterertert4ey));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(esi),16,(dw)((0-16))));
CALL(remet_milieu_x);
R(JZ(errteterertertrte3ey));
R(MOV(16,READDW(edx),16,(dw)0));
R(MOV(16,READDW(ecx),16,(dw)((0-8))));
CALL(possible_ou_pas);
R(JNZ(errteterertertrte2ey));
R(DEC(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(JMP(errteterertertrte3ey));
errteterertertrte2ey:
R(PUSH(32,(READDD(ebp))));
R(ADD(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)0));
R(POP(32,(READDD(ebp))));
errteterertertrte3ey:
R(POP(32,(READDD(ebp))));
ererterertert4ey:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(ererterertert4eyt));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(esi),16,(dw)16));
CALL(remet_milieu_x);
R(JZ(errteterertertrte3eyt));
R(MOV(16,READDW(edx),16,(dw)0));
R(MOV(16,READDW(ecx),16,(dw)(8+1)));
CALL(possible_ou_pas);
R(JNZ(errteterertertrte2eyt));
R(INC(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(JMP(errteterertertrte3eyt));
errteterertertrte2eyt:
R(PUSH(32,(READDD(ebp))));
R(ADD(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)0));
R(POP(32,(READDD(ebp))));
errteterertertrte3eyt:
R(POP(32,(READDD(ebp))));
ererterertert4eyt:
R(POP(32,(READDD(ecx))));
R(DEC(32,(READDD(ecx))));
R(JNZ(ertertrterteterrteert));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(32,(READDD(ebp))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(POP(32,(READDD(ebp))));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,truc)+READDD(eax))))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+READDD(ebp)), ds)),32,(dd)READDD(esi)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
non_bouge_pas:
RET;
rtertterterrterte:
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds))));
R(MOV(16,READDW(eax),16,read_dw(realAddress((READDD(esi)+32), ds))));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*4))+READDD(ebp)), ds)),16,(dw)READDW(eax)));
R(POP(32,(READDD(ebp))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)7));
R(JNZ(ertterrteertrterteertrte));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)16));
R(JE(ertterrteertrterteertrte));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)2));
ertterrteertrterteertrte:
RET;
monsieur_brik:
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc)))));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,briques)+2)))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(ebx),16,(dw)8));
R(MOV(16,READDW(edx),16,(dw)13));
rettertyutyuuityyuityuityuiyuiyuiyui:
R(PUSH(16,(READDW(edx))));
R(MOV(16,READDW(edx),16,(dw)19));
reertertertertertreerertert:
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(zererrte));
R(CMP(8,READDBl(eax),8,(db)1));
R(JE(zererrte));
R(CMP(8,READDBl(eax),8,(db)66));
R(JE(zererrte));
R(CMP(8,READDBl(eax),8,(db)2));
R(JE(ertrtyrtytyytutyutyuuty));
R(CMP(8,READDBl(eax),8,(db)11));
R(JNE(nonononono));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(ADD(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,kel_viseur_brike_fin)+READDD(ebx)), ds))));
R(POP(32,(READDD(ebx))));
R(JMP(brique_dure_rajoutee));
nonononono:
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(erertrteertert));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)11));
R(JNE(erertrteertert));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
R(CMP(8,*((db *) realAddress(((READDD(esi)-1)+(32*13)), ds)),8,(db)0));
R(JE(colle_un_bonusdrtytyrrtyrteterertert2x161));
R(CMP(8,*((db *) realAddress(((READDD(esi)-1)+(32*13)), ds)),8,(db)5));
R(JB(colle_un_bonusdrtytyrrtyrteterertertx161));
colle_un_bonusdrtytyrrtyrteterertert2x161:
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,viseur_hazard_bonus), ds))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,hazard_bonus)))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(ADD(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,viseur_hazard_bonus), ds))));
R(CMP(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,viseur_hazard_bonus)))));
R(JB(colle_un_bonusreertertertx161));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_hazard_bonus), ds)),32,(dd)0));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,hazard_bonus)))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
colle_un_bonusreertertertx161:
R(MOV(8,READDBl(eax),8,*((db *) realAddress(READDD(esi), ds))));
R(POP(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,correspondance_bonus)+READDD(ebx)), ds))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(colle_un_bonuspas_rectx161));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(PUSH(32,(READDD(esi))));
R(MOV(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,replayer_saver4), ds))));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,replayer_saver5), ds))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(((READDD(esi)+taille_header_rec)+READDD(ebx)), fs))));
R(INC(8,*((db *) realAddress(offsetof(struct Mem,replayer_saver5), ds))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(ebx))));
colle_un_bonuspas_rectx161:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,special_nivo_6), ds)),32,(dd)0));
R(JZ(colle_un_bonusuihuiuihhuiouiohuihuiorteerrtyx161));
R(XOR(8,READDBl(eax),8,(db)READDBl(eax)));
colle_un_bonusuihuiuihhuiouiohuihuiorteerrtyx161:
R(MOV(8,*((db *) realAddress(((READDD(esi)-1)+(32*13)), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(eax))));
colle_un_bonusdrtytyrrtyrteterertertx161:
erertrteertert:
ertrtyrtytyytutyutyuuty:
R(SUB(16,READDW(eax),16,(dw)2));
R(SHL(16,READDW(eax),16,(dw)4));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(ADD(16,READDW(ebx),16,(dw)READDW(ebx)));
R(ADD(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,offset_briques)+READDD(ebx)), ds))));
R(POP(32,(READDD(ebx))));
brique_dure_rajoutee:
R(INC(16,(READDW(ecx))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
ertteretretrertrte:
zererrte:
R(ADD(16,READDW(ebx),16,(dw)16));
rettertertertert:
R(DEC(16,(READDW(edx))));
R(JNZ(reertertertertertreerertert));
R(POP(16,(READDW(edx))));
R(ADD(32,READDD(esi),32,(dd)13));
R(ADD(16,READDW(ebx),16,(dw)((320*16)-(16*19))));
R(DEC(16,(READDW(edx))));
R(JNZ(rettertyutyuuityyuityuityuiyuiyuiyui));
R(MOV(16,read_dw(realAddress(offsetof(struct Mem,briques), ds)),16,(dw)READDW(ecx)));
RET;
monsieur_bombe:
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,bombes)+2)))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(ebx),16,(dw)8));
R(MOV(16,READDW(edx),16,(dw)13));
trettertyutyuuityyuityuityuiyuiyuiyui:
R(PUSH(16,(READDW(edx))));
R(MOV(16,READDW(edx),16,(dw)19));
treertertertertertreerertert:
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(tzererrte));
R(CMP(8,READDBl(eax),8,(db)5));
R(JNB(ftttrrtrtyyrtyrtrtyrtytyrrtyrtyrtyrtyrty));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)15));
R(JNZ(terertrteertert));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)5));
R(JNE(terertrteertert));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)1));
terertrteertert:
R(INC(16,(READDW(ecx))));
R(DEC(16,(READDW(eax))));
R(SHL(16,READDW(eax),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)(320*16)));
R(STOSW);
R(PUSH(16,(READDW(ecx))));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(((READDD(esi)-1)+(32*13)), ds))));
R(MOVSX(16,READDW(eax),8,(db)READDBl(eax)));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(MOV(8,READDBl(ecx),8,*((db *) realAddress(((READDD(esi)-1)+((32*13)*2)), ds))));
R(MOVSX(16,READDW(ecx),8,(db)READDBl(ecx)));
R(PUSH(16,(READDW(eax))));
R(MOV(16,READDW(eax),16,(dw)READDW(ecx)));
R(SHL(16,READDW(eax),16,(dw)6));
R(SHL(16,READDW(ecx),16,(dw)8));
R(ADD(16,READDW(ecx),16,(dw)READDW(eax)));
R(POP(16,(READDW(eax))));
R(ADD(16,READDW(eax),16,(dw)READDW(ecx)));
R(STOSW);
R(POP(16,(READDW(ecx))));
R(JMP(tzererrte));
tertteretretrertrte:
ftttrrtrtyyrtyrtrtyrtytyrrtyrtyrtyrtyrty:
R(CMP(16,READDW(eax),16,(dw)12));
R(JNB(bombetetererterertrteerterrteertertrteertrteertertterooix162));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(bombeterertrteertertox162));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)12));
R(JNE(bombeterertrteertertox162));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
bombeterertrteertertox162:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)5));
R(SHL(32,READDD(eax),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,central_b)+READDD(eax)), ds))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bombetetererterertrteerterrteertertrteertrteertertterooix162:
R(CMP(16,READDW(eax),16,(dw)(12+7)));
R(JNB(bombetetererterertrteerterrteertertrteertrteertertterooix163));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(bombeterertrteertertox163));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(12+7)));
R(JNE(bombeterertrteertertox163));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
bombeterertrteertertox163:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)5));
R(SHL(32,READDD(eax),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,central_b)+READDD(eax)), ds))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bombetetererterertrteerterrteertertrteertrteertertterooix163:
R(CMP(16,READDW(eax),16,(dw)((12+7)+7)));
R(JNB(bombetetererterertrteerterrteertertrteertrteertertterooix164));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(bombeterertrteertertox164));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)((12+7)+7)));
R(JNE(bombeterertrteertertox164));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
bombeterertrteertertox164:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)5));
R(SHL(32,READDD(eax),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,central_b)+READDD(eax)), ds))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bombetetererterertrteerterrteertertrteertrteertertterooix164:
R(CMP(16,READDW(eax),16,(dw)(((12+7)+7)+7)));
R(JNB(bombetetererterertrteerterrteertertrteertrteertertterooix165));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(bombeterertrteertertox165));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(((12+7)+7)+7)));
R(JNE(bombeterertrteertertox165));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
bombeterertrteertertox165:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)5));
R(SHL(32,READDD(eax),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,central_b)+READDD(eax)), ds))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bombetetererterertrteerterrteertertrteertrteertertterooix165:
R(CMP(16,READDW(eax),16,(dw)((((12+7)+7)+7)+7)));
R(JNB(bombetetererterertrteerterrteertertrteertrteertertterooix166));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(bombeterertrteertertox166));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)((((12+7)+7)+7)+7)));
R(JNE(bombeterertrteertertox166));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
bombeterertrteertertox166:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)5));
R(SHL(32,READDD(eax),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,central_b)+READDD(eax)), ds))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bombetetererterertrteerterrteertertrteertrteertertterooix166:
R(CMP(16,READDW(eax),16,(dw)(((((12+7)+7)+7)+7)+7)));
R(JNB(bombetetererterertrteerterrteertertrteertrteertertterooix167));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(bombeterertrteertertox167));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(((((12+7)+7)+7)+7)+7)));
R(JNE(bombeterertrteertertox167));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
bombeterertrteertertox167:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)5));
R(SHL(32,READDD(eax),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,central_b)+READDD(eax)), ds))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bombetetererterertrteerterrteertertrteertrteertertterooix167:
R(CMP(16,READDW(eax),16,(dw)((((((12+7)+7)+7)+7)+7)+7)));
R(JNB(bombetetererterertrteerterrteertertrteertrteertertterooix168));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)3));
R(JNZ(bombeterertrteertertox168));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)((((((12+7)+7)+7)+7)+7)+7)));
R(JNE(bombeterertrteertertox168));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
bombeterertrteertertox168:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)5));
R(SHL(32,READDD(eax),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,central_b)+READDD(eax)), ds))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bombetetererterertrteerterrteertertrteertrteertertterooix168:
R(CMP(16,READDW(eax),16,(dw)64));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx169));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx169));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)64));
R(JNE(bonusterertrteertertotx169));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(64-10)));
bonusterertrteertertotx169:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(64-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)160));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx169:
R(CMP(16,READDW(eax),16,(dw)74));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx170));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx170));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)74));
R(JNE(bonusterertrteertertotx170));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(74-10)));
bonusterertrteertertotx170:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(74-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+(320*16))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx170:
R(CMP(16,READDW(eax),16,(dw)84));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx171));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx171));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)84));
R(JNE(bonusterertrteertertotx171));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(84-10)));
bonusterertrteertertotx171:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(84-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+(320*32))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx171:
R(CMP(16,READDW(eax),16,(dw)94));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx172));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx172));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)94));
R(JNE(bonusterertrteertertotx172));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(94-10)));
bonusterertrteertertotx172:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(94-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+(320*48))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx172:
R(CMP(16,READDW(eax),16,(dw)104));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx173));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx173));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)104));
R(JNE(bonusterertrteertertotx173));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(104-10)));
bonusterertrteertertotx173:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(104-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+((320*16)*4))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx173:
R(CMP(16,READDW(eax),16,(dw)114));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx174));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx174));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)114));
R(JNE(bonusterertrteertertotx174));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(114-10)));
bonusterertrteertertotx174:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(114-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+((320*16)*5))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx174:
R(CMP(16,READDW(eax),16,(dw)124));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx175));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx175));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)124));
R(JNE(bonusterertrteertertotx175));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(124-10)));
bonusterertrteertertotx175:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(124-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+((320*16)*6))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx175:
R(CMP(16,READDW(eax),16,(dw)134));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx176));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx176));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)134));
R(JNE(bonusterertrteertertotx176));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(134-10)));
bonusterertrteertertotx176:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(134-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+((320*16)*7))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx176:
R(CMP(16,READDW(eax),16,(dw)144));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx177));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx177));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)144));
R(JNE(bonusterertrteertertotx177));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(144-10)));
bonusterertrteertertotx177:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(144-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+((320*16)*8))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx177:
R(CMP(16,READDW(eax),16,(dw)154));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx178));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx178));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)154));
R(JNE(bonusterertrteertertotx178));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(154-10)));
bonusterertrteertertotx178:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(154-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)(160+((320*16)*9))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx178:
R(CMP(16,READDW(eax),16,(dw)164));
R(JNB(bonustetererterertrteerterrteertertrteertrteertertterooitx179));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(bonusterertrteertertotx179));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)164));
R(JNE(bonusterertrteertertotx179));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(164-10)));
bonusterertrteertertotx179:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)(164-10)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)0));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
bonustetererterertrteerterrteertertrteertrteertertterooitx179:
R(CMP(16,READDW(eax),16,(dw)193));
R(JNE(oeuf_bonustetererterertrteerterrteertertrteertrteertertterooitx180));
R(MOV(16,READDW(eax),16,(dw)(112+(16*320))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
oeuf_bonustetererterertrteerterrteertertrteertrteertertterooitx180:
R(CMP(16,READDW(eax),16,(dw)(194+7)));
R(JNB(explo_bonustetererterertrteerterrteertertrteertrteertertterooitx181));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)7));
R(JNZ(explo_bonusterertrteertertotx181));
R(INC(8,*((db *) realAddress((READDD(esi)-1), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)(194+7)));
R(JNE(explo_bonusterertrteertertotx181));
R(MOV(8,*((db *) realAddress((READDD(esi)-1), ds)),8,(db)0));
explo_bonusterertrteertertotx181:
R(AND(32,READDD(eax),32,(dd)255));
R(SUB(16,READDW(eax),16,(dw)((194+7)-7)));
R(SHL(32,READDD(eax),32,(dd)5));
R(ADD(32,READDD(eax),32,(dd)(0+(172*320))));
R(STOSW);
R(MOV(16,READDW(eax),16,(dw)READDW(ebx)));
R(STOSW);
R(INC(16,(READDW(ecx))));
R(JMP(tzererrte));
explo_bonustetererterertrteerterrteertertrteertrteertertterooitx181:
tzererrte:
R(ADD(16,READDW(ebx),16,(dw)16));
trettertertertert:
R(DEC(16,(READDW(edx))));
R(JNZ(treertertertertertreerertert));
R(POP(16,(READDW(edx))));
R(ADD(32,READDD(esi),32,(dd)13));
R(ADD(16,READDW(ebx),16,(dw)((320*16)-(16*19))));
R(DEC(16,(READDW(edx))));
R(JNZ(trettertyutyuuityyuityuityuiyuiyuiyui));
R(MOV(16,read_dw(realAddress(offsetof(struct Mem,bombes), ds)),16,(dw)READDW(ecx)));
RET;
touches_action:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_vbl_avant_le_droit_de_poser_bombe), ds)),32,(dd)0));
R(JE(okokokok_pas_debut));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_vbl_avant_le_droit_de_poser_bombe), ds))));
R(JMP(treteterrterteterertter));
okokokok_pas_debut:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_med), ds)),32,(dd)attente_avant_med2));
R(JB(treteterrterteterertter));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)3));
R(JE(ertertrterteterrteert2rttyrrty));
R(INC(32,read_dd(realAddress((offsetof(struct Mem,tribombe2)+READDD(ebp)), ds))));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)5));
R(JE(treteterrterteterertter));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,action)+READDD(ebp)), ds)),8,(db)1));
R(JNE(treteterrterteterertter));
ertertrterteterrteert2rttyrrty:
R(MOV(32,READDD(edi),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress(READDD(edi), ds)),32,(dd)0));
R(JE(treteterrterteterertter));
R(PUSH(32,(READDD(ebp))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(POP(32,(READDD(ebp))));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,truc2)+READDD(eax))))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JNE(ya_une_bombeici));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)0));
R(JNE(ya_une_bombeici));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)11));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx457));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx457:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)34));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(READDD(edi), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(8,*((db *) realAddress(READDD(esi), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(DEC(32,read_dd(realAddress(READDD(edi), ds))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
pose_une_bombettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx185:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)0));
R(JE(pose_une_bombeerertertrteterertx185));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(pose_une_bombettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx185));
pose_une_bombeerertertrteterertx185:
R(MOV(32,READDD(edx),32,read_dd(realAddress((READDD(edi)+4), ds))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress((READDD(edi)+8), ds))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(pose_une_bombenononono_onest_en_recordplayx185));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(pose_une_bombenononono_onest_en_recordplayx185));
R(SHR(32,READDD(ecx),32,(dd)1));
pose_une_bombenononono_onest_en_recordplayx185:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(eax)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(3*4)), ds)),16,(dw)READDW(edx)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(eax)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(eax)), ds)),8,(db)0));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(5*4)), ds)),16,(dw)0));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(5*4))+2), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(MOV(32,READDD(edx),32,read_dd(realAddress((READDD(ebx)+(4*4)), ds))));
R(POP(32,(READDD(ebx))));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)6));
R(JNE(pose_une_bombevaniax185));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(3*4)), ds)),16,(dw)1));
pose_une_bombevaniax185:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(3*4))+2), ds)),16,(dw)READDW(edx)));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)READDD(ecx)));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(0*4)), ds)),32,(dd)READDD(edi)));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,liste_bombe), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,tribombe2)+READDD(ebp)), ds)),32,(dd)0));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(treteterrterteterertter));
ya_une_bombeici:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,tribombe)+READDD(ebp)), ds)),32,(dd)0));
R(JE(treteterrterteterertter));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,tribombe2)+READDD(ebp)), ds)),32,(dd)20));
R(JA(treteterrterteterertter));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,tribombe2)+READDD(ebp)), ds)),32,(dd)2));
R(JA(ttrrttreteterrterteterertter));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,tribombe2)+READDD(ebp)), ds)),32,(dd)0));
R(JMP(treteterrterteterertter));
ttrrttreteterrterteterertter:
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds))));
R(AND(32,READDD(eax),32,(dd)127));
R(CMP(32,READDD(eax),32,(dd)16));
R(JNE(direction_du_joueurtrrtertyrtytyrtyrrtyrzx187));
R(MOV(32,READDD(ebx),32,(dd)(((0-1))*1)));
direction_du_joueurtrrtertyrtytyrtyrrtyrzx187:
R(CMP(32,READDD(eax),32,(dd)8));
R(JNE(direction_du_joueurtrrtertyrtytyrtyrrtyrztx187));
R(MOV(32,READDD(ebx),32,(dd)(1*1)));
direction_du_joueurtrrtertyrtytyrtyrrtyrztx187:
R(CMP(32,READDD(eax),32,(dd)0));
R(JNE(direction_du_joueurytrrtertyrtytyrtyrrtyyx187));
R(MOV(32,READDD(ebx),32,(dd)(32*1)));
direction_du_joueurytrrtertyrtytyrtyrrtyyx187:
R(CMP(32,READDD(eax),32,(dd)24));
R(JNE(direction_du_joueurytrrtertyrtytyrtyrrtyryx187));
R(MOV(32,READDD(ebx),32,(dd)(((0-32))*1)));
direction_du_joueurytrrtertyrtytyrtyrrtyryx187:
R(POP(32,(READDD(eax))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
encore_une_toto:
R(CMP(32,read_dd(realAddress(READDD(edi), ds)),32,(dd)0));
R(JNE(nan_c_bon_on_peut_encore_poser));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(treteterrterteterertter));
nan_c_bon_on_peut_encore_poser:
R(ADD(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(eax),32,(dd)READDD(ebx)));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JE(ici_toto));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(treteterrterteterertter));
ici_toto:
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)0));
R(JE(ici_toto2));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(treteterrterteterertter));
ici_toto2:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,playsoundfx), ds)),8,(db)1));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)11));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx458));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx458:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)34));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(READDD(edi), ds))));
R(AND(32,READDD(eax),32,(dd)3));
R(ADD(8,*((db *) realAddress(READDD(esi), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(DEC(32,read_dd(realAddress(READDD(edi), ds))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
pose_une_bombettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx193:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)0));
R(JE(pose_une_bombeerertertrteterertx193));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(pose_une_bombettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx193));
pose_une_bombeerertertrteterertx193:
R(MOV(32,READDD(edx),32,read_dd(realAddress((READDD(edi)+4), ds))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress((READDD(edi)+8), ds))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(pose_une_bombenononono_onest_en_recordplayx193));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(pose_une_bombenononono_onest_en_recordplayx193));
R(SHR(32,READDD(ecx),32,(dd)1));
pose_une_bombenononono_onest_en_recordplayx193:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(eax)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(3*4)), ds)),16,(dw)READDW(edx)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(eax)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(eax)), ds)),8,(db)0));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(5*4)), ds)),16,(dw)0));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(5*4))+2), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(MOV(32,READDD(edx),32,read_dd(realAddress((READDD(ebx)+(4*4)), ds))));
R(POP(32,(READDD(ebx))));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)6));
R(JNE(pose_une_bombevaniax193));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(3*4)), ds)),16,(dw)1));
pose_une_bombevaniax193:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(3*4))+2), ds)),16,(dw)READDW(edx)));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)READDD(ecx)));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(0*4)), ds)),32,(dd)READDD(edi)));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,liste_bombe), ds))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,tribombe2)+READDD(ebp)), ds)),32,(dd)0));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(encore_une_toto));
treteterrterteterertter:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,playsoundfx), ds)),8,(db)1));
R(JNE(dfgjhldfgkhjdflgkjhdfkljhdfglkjhdfglkjhgdfyeah));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)9));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx195));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx195:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)34));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,playsoundfx), ds)),8,(db)0));
dfgjhldfgkhjdflgkjhdfkljhdfglkjhdfglkjhgdfyeah:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,action)+2)+READDD(ebp)), ds)),8,(db)1));
R(JNE(treteterrterteterertteruiotyterertertterert));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JE(treteterrterteterertteruiotyterertertterert));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(treteterrterteterertteruiotyterertertterert));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)10));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx197));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx197:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)32));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds))));
R(AND(32,READDD(eax),32,(dd)127));
R(CMP(32,READDD(eax),32,(dd)16));
R(JNE(direction_du_joueurtrrtertyrtytyrtyrrtyrzx198));
R(MOV(32,READDD(ebx),32,(dd)(((0-1))*2)));
direction_du_joueurtrrtertyrtytyrtyrrtyrzx198:
R(CMP(32,READDD(eax),32,(dd)8));
R(JNE(direction_du_joueurtrrtertyrtytyrtyrrtyrztx198));
R(MOV(32,READDD(ebx),32,(dd)(1*2)));
direction_du_joueurtrrtertyrtytyrtyrrtyrztx198:
R(CMP(32,READDD(eax),32,(dd)0));
R(JNE(direction_du_joueurytrrtertyrtytyrtyrrtyyx198));
R(MOV(32,READDD(ebx),32,(dd)(32*2)));
direction_du_joueurytrrtertyrtytyrtyrrtyyx198:
R(CMP(32,READDD(eax),32,(dd)24));
R(JNE(direction_du_joueurytrrtertyrtytyrtyrrtyryx198));
R(MOV(32,READDD(ebx),32,(dd)(((0-32))*2)));
direction_du_joueurytrrtertyrtytyrtyrrtyryx198:
R(POP(32,(READDD(eax))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,truc)+READDD(eax))))));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(ebp))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)duree_saut));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)1));
R(CMP(32,READDD(ebx),32,(dd)(32*2)));
R(JNE(ertytyyuttyuyuiyuiiyyuiiyuiyu2));
R(CMP(32,READDD(eax),32,(dd)(32*10)));
R(JA(erterererererertert));
ertytyyuttyuyuiyuiiyyuiiyuiyu2:
R(CMP(32,READDD(ebx),32,(dd)(((0-32))*2)));
R(JNE(ertytyyuttyuyuiyuiiyyuiiyuiyuo));
R(CMP(32,READDD(eax),32,(dd)(32*2)));
R(JB(erterererererertert));
ertytyyuttyuyuiyuiiyyuiiyuiyuo:
R(CMP(32,READDD(ebx),32,(dd)(((0-32))*2)));
R(JNE(biooooiiii));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+32), ds)),8,(db)0));
R(JE(biooooiiii));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(16,READDW(eax),16,(dw)7));
R(JE(au_milieu_y2ertzertertax200));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(erterererererertert2));
au_milieu_y2ertzertertax200:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
biooooiiii:
R(CMP(32,READDD(ebx),32,(dd)(32*2)));
R(JNE(biooooiiiie2));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))-32), ds)),8,(db)0));
R(JE(biooooiiiie2));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(16,READDW(eax),16,(dw)7));
R(JE(au_milieu_y2ertzertertax201));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(erterererererertert2));
au_milieu_y2ertzertertax201:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
biooooiiiie2:
R(CMP(32,READDD(ebx),32,(dd)2));
R(JNE(biooooiiiie24));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))-1), ds)),8,(db)0));
R(JE(biooooiiiie24));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)3));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(16,READDW(eax),16,(dw)7));
R(JE(au_milieu_x2ertzertertax202));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(erterererererertert2));
au_milieu_x2ertzertertax202:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
biooooiiiie24:
R(CMP(32,READDD(ebx),32,(dd)((0-2))));
R(JNE(biooooiiiie24y));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+1), ds)),8,(db)0));
R(JE(biooooiiiie24y));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)3));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(16,READDW(eax),16,(dw)7));
R(JE(au_milieu_x2ertzertertax203));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(erterererererertert2));
au_milieu_x2ertzertertax203:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
biooooiiiie24y:
R(CMP(8,*((db *) realAddress((READDD(esi)+READDD(ebx)), ds)),8,(db)0));
R(JNE(erterererererertert));
R(CMP(32,READDD(ebx),32,(dd)((0-2))));
R(JNE(iuertytyyuttyuyuiyuiiyyuiiyuiyu));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(16,READDW(eax),16,(dw)7));
R(JNE(au_milieu_yertzertertax204));
R(POP(32,(READDD(ebp))));
R(JMP(au_milieu_yerterererererertertyutyuyutyuux204));
au_milieu_yertzertertax204:
R(POP(32,(READDD(ebp))));
R(CMP(16,READDW(eax),16,(dw)7));
R(JB(au_milieu_yretreterertertterertertertx204));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+32), ds)),8,(db)0));
R(JNE(erterererererertert2));
R(JMP(au_milieu_yerterererererertertyutyuyutyuux204));
au_milieu_yretreterertertterertertertx204:
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))-32), ds)),8,(db)0));
R(JNE(erterererererertert2));
au_milieu_yerterererererertertyutyuyutyuux204:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)2));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds)),32,(dd)17));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)3));
R(AND(16,READDW(eax),16,(dw)15));
R(POP(32,(READDD(ebp))));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))-1), ds)),8,(db)0));
R(JE(iuokeeey));
R(CMP(16,READDW(eax),16,(dw)7));
R(JA(iuokeeey));
R(PUSH(32,(READDD(eax))));
R(SUB(32,READDD(eax),32,(dd)7));
R(SUB(32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
iuokeeey:
R(POP(32,(READDD(eax))));
iuertytyyuttyuyuiyuiiyyuiiyuiyu:
R(CMP(32,READDD(ebx),32,(dd)((0+2))));
R(JNE(iiuertytyyuttyuyuiyuiiyyuiiyuiyu));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(16,READDW(eax),16,(dw)7));
R(JNE(au_milieu_yertzertertax205));
R(POP(32,(READDD(ebp))));
R(JMP(au_milieu_yerterererererertertyutyuyutyuux205));
au_milieu_yertzertertax205:
R(POP(32,(READDD(ebp))));
R(CMP(16,READDW(eax),16,(dw)7));
R(JB(au_milieu_yretreterertertterertertertx205));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+32), ds)),8,(db)0));
R(JNE(erterererererertert2));
R(JMP(au_milieu_yerterererererertertyutyuyutyuux205));
au_milieu_yretreterertertterertertertx205:
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))-32), ds)),8,(db)0));
R(JNE(erterererererertert2));
au_milieu_yerterererererertertyutyuyutyuux205:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)2));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds)),32,(dd)17));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)3));
R(AND(16,READDW(eax),16,(dw)15));
R(POP(32,(READDD(ebp))));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+1), ds)),8,(db)0));
R(JE(iiuokeeey));
R(CMP(16,READDW(eax),16,(dw)8));
R(JB(iiuokeeey));
R(PUSH(32,(READDD(eax))));
R(SUB(32,READDD(eax),32,(dd)7));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
iiuokeeey:
R(POP(32,(READDD(eax))));
iiuertytyyuttyuyuiyuiiyyuiiyuiyu:
R(CMP(32,READDD(ebx),32,(dd)(32*2)));
R(JNE(nooooooooooooooooooooi));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)3));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(16,READDW(eax),16,(dw)7));
R(JNE(au_milieu_xertzertertax206));
R(POP(32,(READDD(ebp))));
R(JMP(au_milieu_xerterererererertertyutyuyutyuuty222x206));
au_milieu_xertzertertax206:
R(POP(32,(READDD(ebp))));
R(CMP(16,READDW(eax),16,(dw)7));
R(JB(au_milieu_xretreterertertterertertertty222x206));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+1), ds)),8,(db)0));
R(JNE(erterererererertert2));
R(JMP(au_milieu_xerterererererertertyutyuyutyuuty222x206));
au_milieu_xretreterertertterertertertty222x206:
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))-1), ds)),8,(db)0));
R(JNE(erterererererertert2));
au_milieu_xerterererererertertyutyuyutyuuty222x206:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)2));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds)),32,(dd)17));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(AND(16,READDW(eax),16,(dw)15));
R(POP(32,(READDD(ebp))));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+32), ds)),8,(db)0));
R(JE(uokeeey));
R(CMP(16,READDW(eax),16,(dw)8));
R(JB(uokeeey));
R(PUSH(32,(READDD(eax))));
R(SUB(32,READDD(eax),32,(dd)7));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
uokeeey:
R(POP(32,(READDD(eax))));
nooooooooooooooooooooi:
R(CMP(32,READDD(ebx),32,(dd)(((0-32))*2)));
R(JNE(knooooooooooooooooooooi));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)3));
R(AND(16,READDW(eax),16,(dw)15));
R(CMP(16,READDW(eax),16,(dw)7));
R(JNE(au_milieu_xertzertertax207));
R(POP(32,(READDD(ebp))));
R(JMP(au_milieu_xerterererererertertyutyuyutyuuty222x207));
au_milieu_xertzertertax207:
R(POP(32,(READDD(ebp))));
R(CMP(16,READDW(eax),16,(dw)7));
R(JB(au_milieu_xretreterertertterertertertty222x207));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+1), ds)),8,(db)0));
R(JNE(erterererererertert2));
R(JMP(au_milieu_xerterererererertertyutyuyutyuuty222x207));
au_milieu_xretreterertertterertertertty222x207:
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))-1), ds)),8,(db)0));
R(JNE(erterererererertert2));
au_milieu_xerterererererertertyutyuyutyuuty222x207:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)2));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds)),32,(dd)17));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebp))));
R(SHR(32,READDD(ebp),32,(dd)1));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(AND(16,READDW(eax),16,(dw)15));
R(POP(32,(READDD(ebp))));
R(CMP(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))-32), ds)),8,(db)0));
R(JE(kuokeeey));
R(CMP(16,READDW(eax),16,(dw)7));
R(JA(kuokeeey));
R(PUSH(32,(READDD(eax))));
R(SUB(32,READDD(eax),32,(dd)7));
R(NEG(32,(READDD(eax))));
R(ADD(32,read_dd(realAddress((offsetof(struct Mem,lapipipino7)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
kuokeeey:
R(POP(32,(READDD(eax))));
knooooooooooooooooooooi:
erterererererertert:
R(JMP(baaaaaaaaaaa));
erterererererertert2:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)127));
R(JA(baaaaaaaaaaa));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
baaaaaaaaaaa:
treteterrterteterertteruiotyterertertterert:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress((READDD(ebx)+(4*4)), ds)),32,(dd)1));
R(JNE(treteterrterteterertteruioty));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,action)+1)+READDD(ebp)), ds)),8,(db)1));
R(JNE(treteterrterteterertteruioty));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,liste_bombe), ds))));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(pasdutout));
R(LEA(32,m.esi.dd.val,32,(((dd)((offsetof(struct Mem,liste_bombe)+4)-taille_dune_info_bombe)))));
next_bomby:
R(ADD(32,READDD(esi),32,(dd)taille_dune_info_bombe));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)0));
R(JNE(ya_bombe_ici));
R(JMP(next_bomby));
ya_bombe_ici:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebp)), ds))));
R(CMP(32,read_dd(realAddress(READDD(esi), ds)),32,(dd)READDD(ebx)));
R(JNE(non_dejar));
R(CMP(16,read_dw(realAddress(((READDD(esi)+(4*3))+2), ds)),16,(dw)0));
R(JE(non_dejar));
R(MOV(16,read_dw(realAddress(((READDD(esi)+(4*3))+2), ds)),16,(dw)0));
R(MOV(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)1));
non_dejar:
R(DEC(32,(READDD(ecx))));
R(JNZ(next_bomby));
pasdutout:
treteterrterteterertteruioty:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
minuteur:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,liste_bombe), ds))));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(ertrteertertrterteertertertrteertertert));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
tetrrtyrtyrtyrtyrtyrtytyrtyr:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)0));
R(JNE(rtytyrrtytyrtyuutyiyuuiouiopuiouiopioppiopiopp2));
R(INC(32,(READDD(ecx))));
R(JMP(rtytyrrtytyrtyuutyiyuuiouiopuiouiopioppiopiopp));
rtytyrrtytyrtyuutyiyuuiouiopuiouiopioppiopiopp2:
CALL(deplacement_bombes);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JNE(nononono_onest_en_recordplayzrezerzeezr));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,twice), ds)),8,(db)1));
R(JNE(nononono_onest_en_recordplayzrezerzeezr));
CALL(deplacement_bombes);
nononono_onest_en_recordplayzrezerzeezr:
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)11));
R(JNE(kklmjjkjklmklmjmjklmjklmjklmjkl));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(3*4))+2), ds)),16,(dw)0));
R(JMP(finis_la));
kklmjjkjklmklmjmjklmjklmjklmjkl:
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(3*4))+2), ds)),16,(dw)1));
R(JE(rertertertert));
finis_la:
R(DEC(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds))));
R(JNZ(rertertertert));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,liste_bombe), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)11));
R(JE(nononononiioiouuio));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)5));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)2));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx211));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx211:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
nononononiioiouuio:
R(MOV(32,READDD(eax),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(0*4)), ds))));
R(INC(32,read_dd(realAddress(READDD(eax), ds))));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)11));
R(JE(nononononiioiouuiorytrty));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(3*4)), ds))));
explosiontrrtyyrtrytrtyrytrytrtyyrtyrtx212:
R(ADD(32,READDD(esi),32,(dd)((0-32))));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)0));
R(JE(explosionreerertterertteretrx212));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)2));
R(JNE(explosionferretertrterteetrrteertrterteretrteretertertrteertx212));
R(MOV(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)3));
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx212));
explosionreerertterertteretrx212:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
R(JB(explosionnononononono_rien_pas_de_bombex212));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)4));
R(JA(explosionnononononono_rien_pas_de_bombex212));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
explosionttyrrtyrtyrtyrtytyrrtyrtyyrtrtyx212:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(explosionerertertrteterertx212));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(explosionttyrrtyrtyrtyrtytyrrtyrtyyrtrtyx212));
explosionerertertrteterertx212:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx212));
explosionnononononono_rien_pas_de_bombex212:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JB(explosionnononononono_rien_pas_de_bonusx212));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(JA(explosionnononononono_rien_pas_de_bonusx212));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx462));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx462:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx212));
explosionnononononono_rien_pas_de_bonusx212:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JE(explosionerterertrteertrex212));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)5));
R(JB(explosionytyutyuiityuityux212));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)12));
R(JB(explosionferretertrterteetrrteertrterteretrteretertertrteertx212));
explosionytyutyuiityuityux212:
explosionerterertrteertrex212:
R(MOV(8,READDBl(eax),8,(db)33));
R(CMP(32,READDD(ecx),32,(dd)1));
R(JNE(explosionretertertertx212));
R(MOV(8,READDBl(eax),8,(db)40));
explosionretertertertx212:
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)READDBl(eax)));
explosionerterertrteertx212:
R(DEC(32,(READDD(ecx))));
R(JNZ(explosiontrrtyyrtrytrtyrytrytrtyyrtyrtx212));
explosionferretertrterteetrrteertrterteretrteretertertrteertx212:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(3*4)), ds))));
explosiontrrtyyrtrytrtyrytrytrtyyrtyrtx213:
R(ADD(32,READDD(esi),32,(dd)32));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)0));
R(JE(explosionreerertterertteretrx213));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)2));
R(JNE(explosionferretertrterteetrrteertrterteretrteretertertrteertx213));
R(MOV(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)3));
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx213));
explosionreerertterertteretrx213:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
R(JB(explosionnononononono_rien_pas_de_bombex213));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)4));
R(JA(explosionnononononono_rien_pas_de_bombex213));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
explosionttyrrtyrtyrtyrtytyrrtyrtyyrtrtyx213:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(explosionerertertrteterertx213));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(explosionttyrrtyrtyrtyrtytyrrtyrtyyrtrtyx213));
explosionerertertrteterertx213:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx213));
explosionnononononono_rien_pas_de_bombex213:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JB(explosionnononononono_rien_pas_de_bonusx213));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(JA(explosionnononononono_rien_pas_de_bonusx213));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx467));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx467:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx213));
explosionnononononono_rien_pas_de_bonusx213:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JE(explosionerterertrteertrex213));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)5));
R(JB(explosionytyutyuiityuityux213));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)12));
R(JB(explosionferretertrterteetrrteertrterteretrteretertertrteertx213));
explosionytyutyuiityuityux213:
explosionerterertrteertrex213:
R(MOV(8,READDBl(eax),8,(db)33));
R(CMP(32,READDD(ecx),32,(dd)1));
R(JNE(explosionretertertertx213));
R(MOV(8,READDBl(eax),8,(db)47));
explosionretertertertx213:
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)READDBl(eax)));
explosionerterertrteertx213:
R(DEC(32,(READDD(ecx))));
R(JNZ(explosiontrrtyyrtrytrtyrytrytrtyyrtyrtx213));
explosionferretertrterteetrrteertrterteretrteretertertrteertx213:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(3*4)), ds))));
explosiontrrtyyrtrytrtyrytrytrtyyrtyrtx214:
R(ADD(32,READDD(esi),32,(dd)1));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)0));
R(JE(explosionreerertterertteretrx214));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)2));
R(JNE(explosionferretertrterteetrrteertrterteretrteretertertrteertx214));
R(MOV(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)3));
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx214));
explosionreerertterertteretrx214:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
R(JB(explosionnononononono_rien_pas_de_bombex214));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)4));
R(JA(explosionnononononono_rien_pas_de_bombex214));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
explosionttyrrtyrtyrtyrtytyrrtyrtyyrtrtyx214:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(explosionerertertrteterertx214));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(explosionttyrrtyrtyrtyrtytyrrtyrtyyrtrtyx214));
explosionerertertrteterertx214:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx214));
explosionnononononono_rien_pas_de_bombex214:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JB(explosionnononononono_rien_pas_de_bonusx214));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(JA(explosionnononononono_rien_pas_de_bonusx214));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx472));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx472:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx214));
explosionnononononono_rien_pas_de_bonusx214:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JE(explosionerterertrteertrex214));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)5));
R(JB(explosionytyutyuiityuityux214));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)12));
R(JB(explosionferretertrterteetrrteertrterteretrteretertertrteertx214));
explosionytyutyuiityuityux214:
explosionerterertrteertrex214:
R(MOV(8,READDBl(eax),8,(db)12));
R(CMP(32,READDD(ecx),32,(dd)1));
R(JNE(explosionretertertertx214));
R(MOV(8,READDBl(eax),8,(db)26));
explosionretertertertx214:
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)READDBl(eax)));
explosionerterertrteertx214:
R(DEC(32,(READDD(ecx))));
R(JNZ(explosiontrrtyyrtrytrtyrytrytrtyyrtyrtx214));
explosionferretertrterteetrrteertrterteretrteretertertrteertx214:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(MOV(16,READDW(ecx),16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(3*4)), ds))));
explosiontrrtyyrtrytrtyrytrytrtyyrtyrtx215:
R(ADD(32,READDD(esi),32,(dd)((0-1))));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)0));
R(JE(explosionreerertterertteretrx215));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)2));
R(JNE(explosionferretertrterteetrrteertrterteretrteretertertrteertx215));
R(MOV(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)3));
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx215));
explosionreerertterertteretrx215:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
R(JB(explosionnononononono_rien_pas_de_bombex215));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)4));
R(JA(explosionnononononono_rien_pas_de_bombex215));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
explosionttyrrtyrtyrtyrtytyrrtyrtyyrtrtyx215:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(explosionerertertrteterertx215));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(explosionttyrrtyrtyrtyrtytyrrtyrtyyrtrtyx215));
explosionerertertrteterertx215:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx215));
explosionnononononono_rien_pas_de_bombex215:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JB(explosionnononononono_rien_pas_de_bonusx215));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(JA(explosionnononononono_rien_pas_de_bonusx215));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx477));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx477:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)194));
R(JMP(explosionferretertrterteetrrteertrterteretrteretertertrteertx215));
explosionnononononono_rien_pas_de_bonusx215:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JE(explosionerterertrteertrex215));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)5));
R(JB(explosionytyutyuiityuityux215));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)12));
R(JB(explosionferretertrterteetrrteertrterteretrteretertertrteertx215));
explosionytyutyuiityuityux215:
explosionerterertrteertrex215:
R(MOV(8,READDBl(eax),8,(db)12));
R(CMP(32,READDD(ecx),32,(dd)1));
R(JNE(explosionretertertertx215));
R(MOV(8,READDBl(eax),8,(db)19));
explosionretertertertx215:
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)READDBl(eax)));
explosionerterertrteertx215:
R(DEC(32,(READDD(ecx))));
R(JNZ(explosiontrrtyyrtrytrtyrytrytrtyyrtyrtx215));
explosionferretertrterteetrrteertrterteretrteretertertrteertx215:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
nononononiioiouuiorytrty:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(0*4)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(3*4)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4)), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4)), ds)),32,(dd)0));
rertertertert:
rtytyrrtytyrtyuutyiyuuiouiopuiouiopioppiopiopp:
R(ADD(32,READDD(ebp),32,(dd)taille_dune_info_bombe));
R(DEC(32,(READDD(ecx))));
R(JNZ(tetrrtyrtyrtyrtyrtyrtytyrtyr));
ertrteertertrterteertertertrteertertert:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
phase:
R(PUSHAD);
R(CMP(16,read_dw(realAddress(offsetof(struct Mem,bdraw666), ds)),16,(dw)14649));
R(JE(dikgrhrfhgrrethghkgh));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,vie)))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
yretrteertertert:
R(CMP(32,read_dd(realAddress(READDD(edi), ds)),32,(dd)1));
R(JE(et_non_pas_de_draw));
R(ADD(32,READDD(edi),32,(dd)4));
R(DEC(32,(READDD(ecx))));
R(JNZ(yretrteertertert));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_draw), ds))));
R(JNZ(pas_encore));
dikgrhrfhgrrethghkgh:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)68));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)1));
R(JNE(pas_encore));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)77));
pas_encore:
et_non_pas_de_draw:
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(edx),32,(dd)READDD(edx)));
R(XOR(32,READDD(edi),32,(dd)READDD(edi)));
R(MOV(32,READDD(eax),32,(dd)((0-1))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
uyretrteertertert:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(edi)), ds)),32,(dd)1));
R(JNE(uil_est_mort));
R(CMP(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(edi)), ds))));
R(JE(deja_dernier_un_gagnant_comme_ca));
R(INC(32,(READDD(edx))));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(edi)), ds))));
R(MOV(32,READDD(ebx),32,(dd)READDD(edi)));
deja_dernier_un_gagnant_comme_ca:
uil_est_mort:
R(ADD(32,READDD(edi),32,(dd)4));
R(DEC(32,(READDD(ecx))));
R(JNZ(uyretrteertertert));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)1));
R(JNE(pasmode1player));
R(MOV(32,READDD(ecx),32,(dd)8));
R(MOV(32,READDD(edx),32,(dd)0));
R(XOR(32,READDD(edi),32,(dd)READDD(edi)));
comptemonstresvivants:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(edi)), ds)),32,(dd)1));
R(JNE(monstremort));
R(INC(32,(READDD(edx))));
monstremort:
R(ADD(32,READDD(edi),32,(dd)4));
R(DEC(32,(READDD(ecx))));
R(JNZ(comptemonstresvivants));
pasmode1player:
R(CMP(32,READDD(edx),32,(dd)1));
R(JNE(terertyrtytyrrtyrty));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(ebx))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
ertterertuuuuu:
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,infojoueur)+READDD(ebx)), ds))));
CALL(nike_toutes_ses_bombes);
R(POP(32,(READDD(ebx))));
R(ADD(32,READDD(ebx),32,(dd)4));
R(CMP(32,READDD(ebx),32,(dd)32));
R(JNE(ertterertuuuuu));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_med), ds)),32,(dd)attente_avant_med2));
R(JNE(reterertertertr89));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_med), ds)),32,(dd)(attente_avant_med2-1)));
reterertertertr89:
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(MOV(32,READDD(ecx),32,(dd)(32*13)));
zertertrtertetyrtyutyuiuiy:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JE(terertyrtytyrrtyrtyert));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)54));
R(JB(terertyrtytyrrtyrty));
terertyrtytyrrtyrtyert:
R(INC(32,(READDD(esi))));
R(DEC(32,(READDD(ecx))));
R(JNZ(zertertrtertetyrtyutyuiuiy));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(SHL(32,READDD(ebx),32,(dd)2));
R(CMP(32,read_dd(realAddress((READDD(ebx)+offsetof(struct Mem,kelle_fin)), ds)),32,(dd)0));
R(JE(pas_attente_fin_apocalypse));
R(POP(32,(READDD(ebx))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,in_the_apocalypse), ds)),8,(db)1));
R(JE(terertyrtytyrrtyrty));
R(PUSH(32,(READDD(ebx))));
pas_attente_fin_apocalypse:
R(POP(32,(READDD(ebx))));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_med), ds))));
R(JNZ(terertyrtytyrrtyrty));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)90));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
pas_celui_la:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(ebx)), ds)),32,(dd)READDD(eax)));
R(JE(reertertertertert90));
R(ADD(32,READDD(ebx),32,(dd)4));
R(JMP(pas_celui_la));
reertertertertert90:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)1));
R(JNE(dfdfdfgkldgflkdgflkdlgklgdfl));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,victoires)+READDD(ebx)), ds))));
dfdfdfgkldgflkdgflkdlgklgdfl:
R(INC(32,read_dd(realAddress((offsetof(struct Mem,victoires)+READDD(ebx)), ds))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,latest_victory), ds)),32,(dd)READDD(ebx)));
skynet_team_victory:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3_sauve), ds)),8,(db)4));
R(JNE(copy_victory_data));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(ebx)), ds))));
R(XOR(32,READDD(edi),32,(dd)READDD(edi)));
skynet_team_victory_loop:
R(CMP(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,team)+READDD(edi)), ds))));
R(JNE(skynet_team_victory_next));
R(CMP(32,READDD(edi),32,read_dd(realAddress(offsetof(struct Mem,latest_victory), ds))));
R(JE(skynet_team_victory_next));
R(INC(32,read_dd(realAddress((offsetof(struct Mem,victoires)+READDD(edi)), ds))));
skynet_team_victory_next:
R(ADD(32,READDD(edi),32,(dd)4));
R(DEC(32,(READDD(ecx))));
R(JNE(skynet_team_victory_loop));
copy_victory_data:
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,victoires)))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,donnee4)))));
R(MOV(32,READDD(ecx),32,(dd)9));
R(REP_MOVSD);
R(MOV(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,liste_couleur)+READDD(ebx)), ds))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(READDD(esi), ds))));
R(ADD(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,donnee)+((8*3)*2))+READDD(ebx)), ds))));
R(STOSD);
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((READDD(esi)+2), ds))));
R(ADD(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,donnee)+((8*3)*2))+READDD(ebx)), ds))));
R(STOSD);
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((READDD(esi)+4), ds))));
R(ADD(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,donnee)+((8*3)*2))+READDD(ebx)), ds))));
R(STOSD);
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((READDD(esi)+6), ds))));
R(ADD(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,donnee)+((8*3)*2))+READDD(ebx)), ds))));
R(STOSD);
R(SHR(32,READDD(ebx),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+((8*5)*2))+READDD(ebx)), ds))));
R(STOSB);
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+((8*6)*2))+READDD(ebx)), ds))));
R(STOSB);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,briques)))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
erertcharmant:
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nick_t)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebx)), ds))));
R(MOVSD);
R(ADD(32,READDD(ebx),32,(dd)4));
R(CMP(32,READDD(ebx),32,(dd)(4*8)));
R(JNE(erertcharmant));
terertyrtytyrrtyrty:
R(POPAD);
RET;

//PROC load_data
load_data:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,dataloaded), ds)),8,(db)1));
R(JNE(dotheload));
RET;
dotheload:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,dataloaded), ds)),8,(db)2));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
CALL(load_gus);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*0)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)0));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*1)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(64000*2)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*2)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(64000*3)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*7)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(64000*5)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*8)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(64000*6)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*9)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(64000*7)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*10)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(64000*8)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*3)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)576000));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*6)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)640000));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*11)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)704000));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*12)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)768000));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*13)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)832000));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*14)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)896000));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*15)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(896000+(64000*3))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*16)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(896000+(64000*2))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*18)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(896000+(64000*1))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*19)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*21))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*20)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)((896000+(64000*4))+(640*((36*2))))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*21)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(((896000+(64000*4))+(640*((36*2))))+64000)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*22)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(((896000+(64000*4))+(640*((36*2))))+(64000*2))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*23)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(((896000+384000)+46080)+64000)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*24)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)((((896000+384000)+46080)+64000)+64000)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*17)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(((896000+384000)+46080)+(64000*3))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*25)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)1582080));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*26)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1582080+64000)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*27)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1582080+(64000*2))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*30)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+64000)));
CALL(load_raw);
naoinoirzeniozerrzeerzzererz:
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*31)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*2))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*32)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*3))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*33)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*4))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*34)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*5))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*37)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*6))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*35)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1582080+(64000*3))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*36)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1582080+(64000*4))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*38)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*7))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*39)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*8))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*40)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*9))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*41)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*10))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*42)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*11))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*43)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*12))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*44)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*13))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*45)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*14))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*46)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*15))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*47)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*16))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*48)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*17))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*49)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*18))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*50)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*19))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*51)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*20))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*52)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*22))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*53)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*23))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*54)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*24))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*55)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*25))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*56)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1966080+(64000*26))));
CALL(load_raw);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*4)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(1582080+(64000*5))));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,packed_liste)+(4*5)), ds))));
R(MOV(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,iff_file_name)))));
R(MOV(32,READDD(edi),32,(dd)(64000*4)));
R(MOV(32,READDD(ebx),32,(dd)(320*200)));
R(MOV(16,READDW(eax),16,(dw)READDW(fs)));
CALL(load_pcx);
eretterertrerzet:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC transmet_central
transmet_central:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
rterteterrterteertert:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,control_joueur)+READDD(ebp)), ds))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(pas_recertterteretrreertyutyyut));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,control_joueur2)+READDD(ebp)), ds))));
pas_recertterteretrreertyutyyut:
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,total_t)+READDD(ebx))))));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,action)+READDD(ebp)), ds)),32,(dd)0));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JE(dynanormalito));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)0));
R(JNE(errereerreerretertertrtertetyyrtuui));
dynanormalito:
R(CMP(8,*((db *) realAddress((READDD(esi)+3), ds)),8,(db)1));
R(JNE(erterertertertert));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)0));
R(MOV(16,READDW(eax),16,(dw)1));
erterertertertert:
R(CMP(8,*((db *) realAddress((READDD(esi)+1), ds)),8,(db)1));
R(JNE(erterertertertert2));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)8));
R(MOV(16,READDW(eax),16,(dw)2));
erterertertertert2:
R(CMP(8,*((db *) realAddress((READDD(esi)+2), ds)),8,(db)1));
R(JNE(erterertertertert3));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)16));
R(MOV(16,READDW(eax),16,(dw)3));
erterertertertert3:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
R(JNE(erterertertertert4));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)24));
R(MOV(16,READDW(eax),16,(dw)4));
erterertertertert4:
R(CMP(8,*((db *) realAddress((READDD(esi)+4), ds)),8,(db)1));
R(JNE(ertertertertrte));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,action)+READDD(ebp)), ds)),8,(db)1));
ertertertertrte:
R(CMP(8,*((db *) realAddress((READDD(esi)+6), ds)),8,(db)1));
R(JNE(ertertertertrte3));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,action)+READDD(ebp))+2), ds)),8,(db)1));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+READDD(ebp)), ds)),16,(dw)2));
R(JNE(ertertertertrte3));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,action)+READDD(ebp))+2), ds)),8,(db)0));
ertertertertrte3:
errereerreerretertertrtertetyyrtuui:
R(CMP(8,*((db *) realAddress((READDD(esi)+5), ds)),8,(db)1));
R(JNE(tertertertertrte));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,action)+READDD(ebp))+1), ds)),8,(db)1));
tertertertertrte:
R(OR(16,READDW(eax),16,(dw)READDW(eax)));
R(JNZ(reerrteertyut));
R(OR(32,read_dd(realAddress((offsetof(struct Mem,touches)+READDD(ebp)), ds)),32,(dd)128));
reerrteertyut:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(ADD(32,READDD(ebp),32,(dd)4));
R(DEC(32,(READDD(ecx))));
R(JNZ(rterteterrterteertert));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC possible_ou_pas
possible_ou_pas:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)READDW(edx)));
R(ADD(16,READDW(eax),16,(dw)3));
R(SHR(16,READDW(eax),16,(dw)4));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(16,READDW(ebx),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(ebx),16,(dw)14));
R(ADD(16,READDW(ebx),16,(dw)READDW(ecx)));
R(AND(16,READDW(ebx),16,(dw)65520));
R(SHL(16,READDW(ebx),16,(dw)1));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc)))));
R(ADD(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JNE(zerrzerzeerzer));
R(CMP(8,*((db *) realAddress((READDD(esi)+(32*13)), ds)),8,(db)1));
R(JB(efrerrereterter));
R(CMP(8,*((db *) realAddress((READDD(esi)+(32*13)), ds)),8,(db)4));
R(JA(efrerrereterter));
R(ADD(32,READDD(ebp),32,(dd)READDD(ebp)));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+READDD(ebp)), ds)),32,(dd)READDD(esi)));
R(JE(efrerrereterter));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,pousseur)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(zerrzerzeerzer));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(16,READDW(edx),16,(dw)0));
R(JE(pas_eee));
R(JNS(tjyuyu6754oooi));
R(MOV(16,READDW(eax),16,(dw)((0-1))));
tjyuyu6754oooi:
R(CMP(16,READDW(edx),16,(dw)0));
R(JS(tjyuyu6754oooir));
R(MOV(16,READDW(eax),16,(dw)1));
tjyuyu6754oooir:
R(MOV(32,READDD(ecx),32,(dd)0));
CALL(pousse_la_bombe);
R(JMP(okeeeeeyiui));
pas_eee:
R(CMP(16,READDW(ecx),16,(dw)0));
R(JE(pas_eeerty));
R(JNS(tjyuyu6754oooirr));
R(MOV(16,READDW(eax),16,(dw)((0-1))));
tjyuyu6754oooirr:
R(CMP(16,READDW(ecx),16,(dw)0));
R(JS(tjyuyu6754oooirt));
R(MOV(16,READDW(eax),16,(dw)1));
tjyuyu6754oooirt:
R(MOV(32,READDD(ecx),32,(dd)2));
CALL(pousse_la_bombe);
pas_eeerty:
okeeeeeyiui:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(JMP(zerrzerzeerzer));
efrerrereterter:
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
zerrzerzeerzer:
R(MOV(16,READDW(ecx),16,(dw)1));
R(OR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC remet_milieu_x
remet_milieu_x:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(ADD(16,READDW(ebx),16,(dw)3));
R(AND(16,READDW(ebx),16,(dw)15));
R(CMP(16,READDW(ebx),16,(dw)7));
R(JE(iookokokokok_deja_milieux));
R(MOV(16,READDW(edx),16,(dw)0));
R(MOV(16,READDW(ecx),16,(dw)READDW(esi)));
CALL(possible_ou_pas);
R(JNZ(ertertirtyrtyyrtrtyrtyxer));
eheheh_mur:
R(CMP(16,READDW(ebx),16,(dw)7));
R(JA(zererzerrteertrterteert));
R(INC(16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
zererzerrteertrterteert:
R(DEC(16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
ertertirtyrtyyrtrtyrtyxer:
R(PUSH(32,(READDD(ebp))));
R(ADD(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)0));
R(POP(32,(READDD(ebp))));
R(CMP(16,READDW(ebx),16,(dw)7));
R(JA(uzererzerrteertrterteert));
R(MOV(16,READDW(edx),16,(dw)((0-16))));
R(MOV(16,READDW(ecx),16,(dw)READDW(esi)));
CALL(possible_ou_pas);
R(JNZ(eheheh_mur));
R(MOV(16,READDW(edx),16,(dw)((0-16))));
R(MOV(16,READDW(ecx),16,(dw)0));
CALL(possible_ou_pas);
R(JNZ(eheheh_mur));
R(DEC(16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
uzererzerrteertrterteert:
R(MOV(16,READDW(edx),16,(dw)16));
R(MOV(16,READDW(ecx),16,(dw)READDW(esi)));
CALL(possible_ou_pas);
R(JNZ(eheheh_mur));
R(MOV(16,READDW(edx),16,(dw)16));
R(MOV(16,READDW(ecx),16,(dw)0));
CALL(possible_ou_pas);
R(JNZ(eheheh_mur));
R(INC(16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
iookokokokok_deja_milieux:
R(MOV(16,READDW(ecx),16,(dw)1));
R(OR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC remet_milieu_y
remet_milieu_y:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(16,READDW(ebx),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(ebx),16,(dw)14));
R(AND(16,READDW(ebx),16,(dw)15));
R(CMP(16,READDW(ebx),16,(dw)7));
R(JE(yiookokokokok_deja_milieux));
R(MOV(16,READDW(edx),16,(dw)READDW(esi)));
R(MOV(16,READDW(ecx),16,(dw)0));
CALL(possible_ou_pas);
R(JNZ(yertertirtyrtyyrtrtyrtyxer));
impossible_2_fois:
R(CMP(16,READDW(ebx),16,(dw)7));
R(JA(yzererzerrteertrterteert));
R(INC(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
yzererzerrteertrterteert:
R(DEC(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
yertertirtyrtyyrtrtyrtyxer:
R(PUSH(32,(READDD(ebp))));
R(ADD(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,avance)+READDD(ebp)), ds)),32,(dd)0));
R(POP(32,(READDD(ebp))));
R(CMP(16,READDW(ebx),16,(dw)7));
R(JA(yuzererzerrteertrterteert));
R(MOV(16,READDW(edx),16,(dw)READDW(esi)));
R(MOV(16,READDW(ecx),16,(dw)((0-16))));
CALL(possible_ou_pas);
R(JNZ(impossible_2_fois));
R(MOV(16,READDW(edx),16,(dw)0));
R(MOV(16,READDW(ecx),16,(dw)((0-16))));
CALL(possible_ou_pas);
R(JNZ(impossible_2_fois));
R(DEC(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
yuzererzerrteertrterteert:
R(MOV(16,READDW(edx),16,(dw)READDW(esi)));
R(MOV(16,READDW(ecx),16,(dw)16));
CALL(possible_ou_pas);
R(JNZ(impossible_2_fois));
R(MOV(16,READDW(edx),16,(dw)0));
R(MOV(16,READDW(ecx),16,(dw)16));
CALL(possible_ou_pas);
R(JNZ(impossible_2_fois));
R(INC(16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(XOR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
yiookokokokok_deja_milieux:
R(MOV(16,READDW(ecx),16,(dw)1));
R(OR(16,READDW(ecx),16,(dw)READDW(ecx)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC nouvelle_partie
nouvelle_partie:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pauseur2), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pause), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,pause2), ds)),8,(db)50));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(PUSH(32,(READDD(eax))));
R(SHL(32,READDD(eax),32,(dd)2));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna_x4), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
R(SUB(32,READDD(eax),32,(dd)8));
R(NEG(32,(READDD(eax))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_monstres), ds)),32,(dd)READDD(eax)));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_monstres), ds)),32,(dd)READDD(eax)));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JE(trtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2erte));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
trtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2erte:
R(MOV(32,READDD(ecx),32,(dd)8));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,victoires)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(REP_STOSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC nouvelle_manche
nouvelle_manche:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,inser_coin), ds)),32,(dd)120));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_ic2), ds)),32,(dd)4));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,adder_inser_coin), ds)),32,(dd)(320*((67+50)))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_sur_fond), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,duree_vic), ds)),32,(dd)duree_vic2));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_draw), ds)),32,(dd)attente_avant_draw2));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_med), ds)),32,(dd)attente_avant_med2));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,duree_draw), ds)),32,(dd)duree_draw2));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,duree_med), ds)),32,(dd)duree_med2));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente_nouveau_esc), ds)),32,(dd)0));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)1));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)3));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)0));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JE(trtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2ert));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
trtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2ert:
R(MOV(16,read_dw(realAddress(offsetof(struct Mem,bdraw666), ds)),16,(dw)(m.isLittle?12339:13104)));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,bdraw1), ds)),8,(db)60));
R(MOV(16,read_dw(realAddress(offsetof(struct Mem,adder_bdraw), ds)),16,(dw)(50*320)));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,balance_le_bdrawn), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,temps2), ds)),8,(db)59));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,special_nivo_6), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,acceleration), ds)),32,(dd)0));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,in_the_apocalypse), ds)),8,(db)0));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_vbl_avant_le_droit_de_poser_bombe), ds)),32,(dd)nombre_de_vbl_avant_le_droit_de_poser_bombe2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)1));
R(JNE(fdlkjdfkljdfglkgdjf));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_vbl_avant_le_droit_de_poser_bombe), ds)),32,(dd)0));
fdlkjdfkljdfglkgdjf:
R(PUSH(16,(READDW(eax))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,team3_sauve), ds))));
R(AND(8,READDBl(eax),8,(db)3));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)READDBl(eax)));
R(POP(16,(READDW(eax))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,viseur_liste_terrain), ds))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,liste_terrain)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)READDBl(eax)));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,viseur_liste_terrain), ds))));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,liste_terrain)+1)+READDD(ebx)), ds)),8,(db)66));
R(JNE(coolio));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_liste_terrain), ds)),32,(dd)0));
coolio:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(ertrtertertyetyuyutyut));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)0));
R(PUSH(32,(READDD(eax))));
R(MOV(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,replayer_saver4), ds))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((READDD(esi)+taille_header_rec)-9), fs))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,isbigendian), ds)),8,(db)1));
R(JNE(bigendianpatchblablablatotox245));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,bigendianin), ds)),32,(dd)READDD(eax)));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,bigendianin), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+3), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+1), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+2), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+2), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+1), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+3), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+0), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,bigendianout), ds))));
bigendianpatchblablablatotox245:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(((READDD(esi)+taille_header_rec)-1), fs))));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
ertrtertertyetyuyutyut:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)0));
R(JNE(etrtyertyrdfgdfggdffgdgdfgy));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,n_team)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,team)))));
R(MOV(32,READDD(ecx),32,(dd)9));
R(REP_MOVSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
etrtyertyrdfgdfggdffgdgdfgy:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)2));
R(JNE(etrtyertyrdfgdfggdffgdgdf));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,s_team)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,team)))));
R(MOV(32,READDD(ecx),32,(dd)9));
R(REP_MOVSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
etrtyertyrdfgdfggdffgdgdf:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)1));
R(JNE(etrtyertyrdfgdfggdffgdgdfe));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,c_team)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,team)))));
R(MOV(32,READDD(ecx),32,(dd)9));
R(REP_MOVSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
etrtyertyrdfgdfggdffgdgdfe:
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(SHL(32,READDD(ebx),32,(dd)2));
R(MOV(32,READDD(eax),32,read_dd(realAddress((READDD(ebx)+offsetof(struct Mem,kelle_duree)), ds))));
R(MOV(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)READDW(eax)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)(((64/4))*8)));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_play)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)(64/4)));
R(REP_STOSD);
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)8));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,touches_save)))));
R(REP_STOSD);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,donnee)))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(SHL(32,READDD(ebx),32,(dd)2));
R(MOV(32,READDD(esi),32,read_dd(realAddress((READDD(ebx)+offsetof(struct Mem,kelle_donnee)), ds))));
R(XOR(32,READDD(edx),32,(dd)READDD(edx)));
R(MOV(32,READDD(edx),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(edx),32,(dd)15));
R(SHL(32,READDD(edx),32,(dd)5));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(pashjktrkhjerterttyr));
R(PUSH(32,(READDD(esi))));
R(MOV(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,replayer_saver4), ds))));
R(MOV(32,READDD(edx),32,read_dd(realAddress(((READDD(esi)+taille_header_rec)-5), fs))));
R(POP(32,(READDD(esi))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,isbigendian), ds)),8,(db)1));
R(JNE(bigendianpatchblablablatotox252));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,bigendianin), ds)),32,(dd)READDD(edx)));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,bigendianin), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+3), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+1), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+2), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+2), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+1), ds)),8,(db)READDBl(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,bigendianin)+3), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bigendianout)+0), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(MOV(32,READDD(edx),32,read_dd(realAddress(offsetof(struct Mem,bigendianout), ds))));
bigendianpatchblablablatotox252:
pashjktrkhjerterttyr:
R(MOV(32,READDD(ecx),32,(dd)8));
oooiiooiioio:
R(MOV(32,READDD(ebx),32,read_dd(realAddress((offsetof(struct Mem,random_place)+READDD(edx)), ds))));
R(MOV(16,READDW(eax),16,read_dw(realAddress((READDD(esi)+READDD(ebx)), ds))));
R(MOV(16,read_dw(realAddress(READDD(edi), ds)),16,(dw)READDW(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((READDD(esi)+READDD(ebx))+(8*2)), ds))));
R(MOV(16,read_dw(realAddress((READDD(edi)+(8*2)), ds)),16,(dw)READDW(eax)));
R(ADD(32,READDD(edi),32,(dd)2));
R(ADD(32,READDD(edx),32,(dd)4));
R(DEC(32,(READDD(ecx))));
R(JNZ(oooiiooiioio));
R(ADD(32,READDD(esi),32,(dd)(8*4)));
R(ADD(32,READDD(edi),32,(dd)(8*2)));
R(MOV(32,READDD(ecx),32,(dd)(8*14)));
R(REP_MOVSB);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,liste_couleur)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_MOVSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,nombre_de_coups)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_MOVSD);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,infos_j_n)))));
R(MOV(32,READDD(ecx),32,(dd)5));
R(REP_MOVSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,infos_m_n)))));
R(MOV(32,READDD(ecx),32,(dd)(5*8)));
R(REP_MOVSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,invinsible)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_MOVSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,blocage)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_MOVSD);
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(LODSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,invinsible)))));
R(REP_STOSD);
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(LODSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,blocage)))));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pousseur)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_MOVSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,vitesse_monstre)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_MOVSD);
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(LODSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,pousseur)))));
R(REP_STOSD);
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(LODSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,patineur)))));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,correspondance_bonus)))));
R(MOV(32,READDD(ecx),32,(dd)(32/4)));
R(REP_MOVSD);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,j1)))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(PUSH(32,(READDD(ecx))));
koaiouiouiouiououiuio:
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,infos_j_n), ds))));
R(STOSD);
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,infos_j_n)+4), ds))));
R(STOSD);
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,infos_j_n)+8), ds))));
R(STOSD);
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,infos_j_n)+12), ds))));
R(STOSD);
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,infos_j_n)+16), ds))));
R(STOSD);
R(DEC(32,(READDD(ecx))));
R(JNZ(koaiouiouiouiououiuio));
R(POP(32,(READDD(ecx))));
R(MOV(32,READDD(eax),32,(dd)READDD(ecx)));
R(SUB(32,READDD(ecx),32,(dd)8));
R(NEG(32,(READDD(ecx))));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(centralol));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,infos_m_n)))));
erterertertert:
R(ADD(32,READDD(esi),32,(dd)(4*5)));
R(DEC(32,(READDD(eax))));
R(JNZ(erterertertert));
monstro4:
R(PUSH(32,(READDD(ecx))));
R(MOV(32,READDD(ecx),32,(dd)5));
R(REP_MOVSD);
R(POP(32,(READDD(ecx))));
R(DEC(32,(READDD(ecx))));
R(JNZ(monstro4));
centralol:
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,s_normal)))));
R(LEA(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,donnee)+(8*6))))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(REP_MOVSD);
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,liste_couleur_normal)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,liste_couleur)))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(REP_MOVSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,l_normal)))));
R(LEA(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,donnee)+(8*10))))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(REP_MOVSW);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,c_normal)))));
R(LEA(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,donnee)+(8*12))))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(REP_MOVSW);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,a_normal)))));
R(LEA(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,donnee)+(8*14))))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(REP_MOVSD);
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,r_normal)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,nombre_de_coups)))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds))));
R(REP_MOVSD);
i:
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,briques)))));
R(MOV(32,READDD(ecx),32,(dd)(1+((19*13)*2))));
R(REP_STOSW);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,bombes)))));
R(MOV(32,READDD(ecx),32,(dd)(1+((19*13)*2))));
R(REP_STOSW);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,maladie)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,clignotement)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,tribombe)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,tribombe2)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,lapipipino)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,lapipipino2)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,lapipipino3)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,lapipipino4)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,lapipipino5)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,lapipipino6)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(SHL(32,READDD(ebx),32,(dd)2));
R(MOV(32,READDD(esi),32,read_dd(realAddress((READDD(ebx)+offsetof(struct Mem,kelle_truc)), ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,truc)))));
R(MOV(32,READDD(ecx),32,(dd)(32*13)));
R(REP_MOVSB);
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(SHL(32,READDD(ebx),32,(dd)2));
R(MOV(32,READDD(esi),32,read_dd(realAddress((READDD(ebx)+offsetof(struct Mem,kelle_bonus)), ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(MOV(32,READDD(ecx),32,(dd)(32*13)));
R(REP_MOVSB);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,liste_bombe)))));
R(MOV(32,READDD(ecx),32,(dd)(1+(247*((taille_dune_info_bombe/4))))));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,avance2)))));
R(MOV(32,READDD(eax),32,(dd)1));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,touches)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,action)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress(offsetof(struct Mem,terrain), ds))));
R(DEC(8,(READDBl(ebx))));
R(SHL(32,READDD(ebx),32,(dd)2));
R(MOV(32,READDD(esi),32,read_dd(realAddress((READDD(ebx)+offsetof(struct Mem,kelle_apocalypse)), ds))));
R(POP(32,(READDD(ebx))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,truc_fin)))));
R(MOV(32,READDD(ecx),32,(dd)((13*32)+4)));
R(REP_MOVSB);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,vie)))));
R(MOV(32,READDD(ecx),32,(dd)8));
R(MOV(32,READDD(eax),32,(dd)1));
R(REP_STOSD);
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
a6ans:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(32,(READDD(ebp))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(POP(32,(READDD(ebp))));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,truc)+READDD(eax))))));
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+READDD(ebp)), ds)),32,(dd)READDD(esi)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(ADD(32,READDD(ebp),32,(dd)4));
R(CMP(32,READDD(ebp),32,(dd)(4*8)));
R(JNE(a6ans));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(JE(pas_action));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,liste_bombbbb2), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente_entre_chake_bombe), ds)),32,(dd)0));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur__nouvelle_attente_entre_chake_bombe), ds)),32,(dd)0));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,avance)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,viseur_change_in_save)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,viseur_change_in)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_MOVSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
pas_action:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC rec_play_touches
rec_play_touches:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(pas_rec4));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,special_on_a_loadee_nivo), ds)),8,(db)2));
R(JNE(rertterterrtertytyrrtyrtyrtyrtyrtyrtyrty));
erererjhrejhreerlhehelej:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sors_du_menu_aussitot), ds)),8,(db)1));
rertterterrtertytyrrtyrtyrtyrtyrtyrtyrty:
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,replayer_saver), ds))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_play)))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,replayer_saver3), ds))));
iencoermnjklrtrtytyuyuisdfgrht345:
R(PUSH(32,(READDD(esi))));
R(MOV(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,replayer_saver4), ds))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((((READDD(esi)+taille_header_rec)+taille_bonus_rec)+READDD(ebp)), fs))));
R(POP(32,(READDD(esi))));
R(MOV(8,*((db *) realAddress((READDD(esi)+READDD(ebx)), ds)),8,(db)0));
R(TEST(8,READDBl(eax),8,(db)1));
R(JZ(bhrebherterteeeeee));
R(MOV(8,*((db *) realAddress((READDD(esi)+READDD(ebx)), ds)),8,(db)1));
bhrebherterteeeeee:
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+1), ds)),8,(db)0));
R(TEST(8,READDBl(eax),8,(db)2));
R(JZ(bhrebherterteeeeeei));
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+1), ds)),8,(db)1));
bhrebherterteeeeeei:
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+2), ds)),8,(db)0));
R(TEST(8,READDBl(eax),8,(db)4));
R(JZ(bhrebherterteeeeeeii));
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+2), ds)),8,(db)1));
bhrebherterteeeeeeii:
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+3), ds)),8,(db)0));
R(TEST(8,READDBl(eax),8,(db)8));
R(JZ(bhrebherterteeeeeeiii));
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+3), ds)),8,(db)1));
bhrebherterteeeeeeiii:
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+4), ds)),8,(db)0));
R(TEST(8,READDBl(eax),8,(db)16));
R(JZ(bhrebherterteeeeeeooo));
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+4), ds)),8,(db)1));
bhrebherterteeeeeeooo:
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+5), ds)),8,(db)0));
R(TEST(8,READDBl(eax),8,(db)32));
R(JZ(bhrebherterteeeeeep));
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+5), ds)),8,(db)1));
bhrebherterteeeeeep:
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+6), ds)),8,(db)0));
R(TEST(8,READDBl(eax),8,(db)64));
R(JZ(bhrebherterteeeeeept));
R(MOV(8,*((db *) realAddress(((READDD(esi)+READDD(ebx))+6), ds)),8,(db)1));
bhrebherterteeeeeept:
R(INC(32,(READDD(ebp))));
R(ADD(32,READDD(ebx),32,(dd)7));
R(DEC(32,(READDD(ecx))));
R(JNZ(iencoermnjklrtrtytyuyuisdfgrht345));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,replayer_saver), ds)),32,(dd)READDD(ebp)));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nb_ordy_connected), ds))));
R(INC(32,(READDD(ecx))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
touche_presseretertertertertetrtrertertertx260:
R(CMP(8,*((db *) realAddress(((READDD(esi)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseerterertertertx260));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
touche_presseerterertertertx260:
R(ADD(32,READDD(esi),32,(dd)64));
R(DEC(32,(READDD(ecx))));
R(JNZ(touche_presseretertertertertetrtrertertertx260));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
R(JNE(rertttttttttttttttttttttt345));
R(PUSH(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,ttp), ds))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,temps_avant_demo), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
rertttttttttttttttttttttt345:
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,replayer_saver2), ds))));
R(JNZ(continueeeee));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
continueeeee:
pas_rec4:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC load_gus
load_gus:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,iff_liste)))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,total_liste), ds))));
uencoremasuperdeliredemaker:
R(CMP(32,read_dd(realAddress(READDD(edi), ds)),32,(dd)((0-1))));
R(JE(ufinitobabyr));
R(ADD(32,READDD(eax),32,read_dd(realAddress(READDD(edi), ds))));
R(MOV(32,read_dd(realAddress(READDD(edi), ds)),32,(dd)READDD(eax)));
R(ADD(32,READDD(edi),32,(dd)4));
R(JMP(uencoremasuperdeliredemaker));
ufinitobabyr:
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,READDD(eax),32,(dd)4));
tyu:
R(MOV(32,READDD(ecx),32,read_dd(realAddress((offsetof(struct Mem,iff_liste)+READDD(ebp)), ds))));
R(CMP(32,READDD(ecx),32,(dd)((0-1))));
R(JE(ooo));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(ADD(32,READDD(ebp),32,(dd)4));
R(ADD(32,READDD(eax),32,(dd)4));
R(JMP(tyu));
ooo:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC init_packed_liste
init_packed_liste:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,packed_liste)))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,taille_exe_gonfle), ds))));
encoremasuperdeliredemaker:
R(CMP(32,read_dd(realAddress(READDD(edi), ds)),32,(dd)((0-1))));
R(JE(finitobabyr));
R(ADD(32,READDD(eax),32,read_dd(realAddress(READDD(edi), ds))));
R(MOV(32,read_dd(realAddress(READDD(edi), ds)),32,(dd)READDD(eax)));
R(ADD(32,READDD(edi),32,(dd)4));
R(JMP(encoremasuperdeliredemaker));
finitobabyr:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,total_liste), ds)),32,(dd)READDD(eax)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC menu
menu:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JNE(reerttrtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2r));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,pic_time), ds)),32,(dd)0));
R(JZ(reerttrtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2r));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,pic_time), ds))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,pic_time), ds)),32,(dd)16));
R(JNE(erterertyyuuutyutyutyutyutyutyutyuty));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)1));
erterertyyuuutyutyutyutyutyutyutyuty:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,pic_time), ds)),32,(dd)17));
R(JB(erttertertertertert));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,pic_time), ds)),32,(dd)(pic_max-34)));
R(JA(erttertertertertert));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nb_ordy_connected), ds))));
R(INC(32,(READDD(ecx))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
touche_presseretertertertertetrtrertertertx269:
R(CMP(8,*((db *) realAddress(((READDD(esi)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseerterertertertx269));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,pic_time), ds)),32,(dd)17));
touche_presseerterertertertx269:
R(ADD(32,READDD(esi),32,(dd)64));
R(DEC(32,(READDD(ecx))));
R(JNZ(touche_presseretertertertertetrtrertertertx269));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
erttertertertertert:
R(MOV(32,READDD(esi),32,(dd)(896000+(64000*1))));
R(RAMESI);
R(COPYBLOCK);
CALL(aff_page2);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
reerttrtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2r:
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,last_sucker), ds)),8,(db)0));
R(JNE(kjhkjhfhgfhgfhgfghfghfhgfhgfhgfhgfhgfghf));
R(MOV(32,READDD(esi),32,(dd)(896000+(64000*3))));
R(MOV(32,READDD(esi),32,(dd)(64000*4)));
R(RAMESI);
R(COPYBLOCK);
kjhkjhfhgfhgfhgfghfghfhgfhgfhgfhgfhgfghf:
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
CALL(scroll);
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur_couleur), ds)),32,(dd)0));
R(LEA(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+((((((((((30*320)+(36*320))+14)+(320*3))+2)-6)+2)+(320*8))+1)))))));
R(MOV(32,m.ebx.dd.val,32,(((dd)offsetof(struct Mem,texte1)))));
R(MOV(32,READDD(ecx),32,(dd)4));
rrteertertrteert:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,scrollyf), ds))));
R(AND(32,READDD(eax),32,(dd)96));
R(ADD(32,READDD(ebx),32,(dd)READDD(eax)));
CALL(aff_texte);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(ADD(32,READDD(ebx),32,(dd)(32*4)));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,viseur_couleur), ds))));
R(ADD(32,READDD(edi),32,(dd)80));
R(DEC(32,(READDD(ecx))));
R(JNZ(rrteertertrteert));
R(LEA(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+((((((((((101*320)+(36*320))+14)+(320*3))+2)-6)+2)+(320*8))+1)))))));
R(MOV(32,m.ebx.dd.val,32,(((dd)(offsetof(struct Mem,texte1)+((32*4)*4))))));
R(MOV(32,READDD(ecx),32,(dd)4));
oorrteertertrteert:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,scrollyf), ds))));
R(AND(32,READDD(eax),32,(dd)96));
R(ADD(32,READDD(ebx),32,(dd)READDD(eax)));
CALL(aff_texte);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(ADD(32,READDD(ebx),32,(dd)(32*4)));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,viseur_couleur), ds))));
R(ADD(32,READDD(edi),32,(dd)80));
R(DEC(32,(READDD(ecx))));
R(JNZ(oorrteertertrteert));
R(POP(16,(READDW(es))));
CALL(aff_page2);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
erettererttrtyrtyyrtrty:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
aff_lettre:
R(PUSH(32,(READDD(ecx))));
R(MOV(32,READDD(ecx),32,(dd)6));
oertterertertertert:
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,viseur_couleur), es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), es)),8,(db)90));
R(JNE(rertetyutyuyuttuyyuttyu));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,couleur)+READDD(ebx)), es))));
R(JMP(rtyutyuyuttyutyuyutyutyut));
rertetyutyuyuttuyyuttyu:
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,couleur_menu)+READDD(ebx)), es))));
rtyutyuyuttyutyuyutyutyut:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(crocroeretretrertertertx277));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(ebx)));
crocroeretretrertertertx277:
R(INC(32,(READDD(edi))));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(crocroeretretrertertertx278));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(ebx)));
crocroeretretrertertertx278:
R(INC(32,(READDD(edi))));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(crocroeretretrertertertx279));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(ebx)));
crocroeretretrertertertx279:
R(INC(32,(READDD(edi))));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(crocroeretretrertertertx280));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(ebx)));
crocroeretretrertertertx280:
R(INC(32,(READDD(edi))));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(crocroeretretrertertertx281));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(ebx)));
crocroeretretrertertertx281:
R(INC(32,(READDD(edi))));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(crocroeretretrertertertx282));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(ebx)));
crocroeretretrertertertx282:
R(INC(32,(READDD(edi))));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(crocroeretretrertertertx283));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(ebx)));
crocroeretretrertertertx283:
R(INC(32,(READDD(edi))));
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(crocroeretretrertertertx284));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(ebx)));
crocroeretretrertertertx284:
R(INC(32,(READDD(edi))));
R(POP(32,(READDD(ebx))));
R(ADD(32,READDD(edi),32,(dd)(320-8)));
R(ADD(32,READDD(esi),32,(dd)(320-8)));
R(DEC(32,(READDD(ecx))));
R(JNZ(oertterertertertert));
R(POP(32,(READDD(ecx))));
RET;
aff_texte:
R(MOV(32,READDD(edx),32,(dd)3));
reretertertrte:
R(MOV(32,READDD(ecx),32,(dd)6));
ererrteertertertertert:
CALL(affiche_un_caractere);
R(ADD(32,READDD(edi),32,(dd)8));
R(DEC(32,(READDD(ecx))));
R(JNZ(ererrteertertertertert));
R(ADD(32,READDD(edi),32,(dd)((320*10)-(8*6))));
R(DEC(32,(READDD(edx))));
R(JNZ(reretertertrte));
RET;
rerteertertertertertrteertertretrerertertert:
R(INC(32,(READDD(ebx))));
R(JMP(rtrtytyryrtrtysepcialespace));
affiche_un_caractere:
R(PUSH(32,(READDD(edi))));
R(PUSH(16,(READDW(ds))));
R(MOV(32,READDD(esi),32,(dd)((1582080+(64000*5))+(165*320))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(READDD(ebx), es))));
R(CMP(8,READDBl(eax),8,(db)32));
R(JE(rerteertertertertertrteertertretrerertertert));
R(CMP(8,READDBl(eax),8,(db)(122+13)));
R(JE(rerteertertertertertrteertertretrerertertert));
R(CMP(8,READDBl(eax),8,(db)63));
R(JNE(erttrerteertertertertertrteertertretrerertertertyyfr));
R(MOV(32,READDD(esi),32,(dd)((((1582080+(64000*5))+232)+(8*3))+(172*320))));
R(JMP(reerertrteertertrteertrtertertrteert));
erttrerteertertertertertrteertertretrerertertertyyfr:
R(CMP(8,READDBl(eax),8,(db)45));
R(JNE(erttrerteertertertertertrteertertretrerertertertyyfrt));
R(MOV(32,READDD(esi),32,(dd)(((1582080+(64000*5))+(167*320))+304)));
R(SUB(32,READDD(edi),32,(dd)(320*3)));
R(JMP(reerertrteertertrteertrtertertrteert));
erttrerteertertertertertrteertertretrerertertertyyfrt:
R(SUB(8,READDBl(eax),8,(db)97));
R(CMP(8,READDBl(eax),8,(db)(33-97)));
R(JNE(erterterert));
R(MOV(16,READDW(eax),16,(dw)(288/8)));
R(JMP(rtrtyrtyrty));
erterterert:
R(CMP(8,READDBl(eax),8,(db)(46-97)));
R(JNE(erterterertz));
R(MOV(16,READDW(eax),16,(dw)(296/8)));
R(JMP(rtrtyrtyrty));
erterterertz:
R(CMP(8,READDBl(eax),8,(db)(58-97)));
R(JNE(derterterertz));
R(MOV(16,READDW(eax),16,(dw)(312/8)));
R(JMP(rtrtyrtyrty));
derterterertz:
R(CMP(8,READDBl(eax),8,(db)(48-97)));
R(JB(rderterterertz));
R(ADD(8,READDBl(eax),8,(db)(26-((48-97)))));
rderterterertz:
rtrtyrtyrty:
R(SHL(16,READDW(eax),16,(dw)3));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
reerertrteertertrteertrtertertrteert:
R(INC(32,(READDD(ebx))));
CALL(aff_lettre);
rtrtytyryrtrtysepcialespace:
R(POP(16,(READDW(ds))));
R(POP(32,(READDD(edi))));
RET;

//PROC scroll
scroll:
R(INC(32,read_dd(realAddress(offsetof(struct Mem,scrollyf), ds))));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,scrollyf), ds)),32,(dd)7));
R(JNZ(trerteertertertertertrteertertretrerertertert));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,m.ebx.dd.val,32,(((dd)offsetof(struct Mem,tected)))));
R(ADD(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,tecte2), ds))));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,tecte2), ds))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(READDD(ebx), ds))));
R(CMP(8,READDBl(eax),8,(db)42));
R(JNE(dtrerteertertertertertrteertertretrererterterte));
R(LEA(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,tected)))));
R(SUB(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,tecte)))));
R(NEG(32,(READDD(edx))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,tecte2), ds)),32,(dd)READDD(edx)));
R(MOV(32,READDD(esi),32,(dd)(((1582080+(64000*5))+223)+(171*320))));
R(JMP(reerertrteertertrteert));
dtrerteertertertertertrteertertretrererterterte:
R(MOV(8,READDBl(eax),8,*((db *) realAddress(READDD(ebx), ds))));
R(CMP(8,READDBl(eax),8,(db)219));
R(JNE(dtrerteertertertertertrteertertretrerertertert));
R(LEA(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,tected)))));
R(SUB(32,m.edx.dd.val,32,(((dd)offsetof(struct Mem,tecte)))));
R(NEG(32,(READDD(edx))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,tecte2), ds)),32,(dd)READDD(edx)));
R(MOV(32,READDD(esi),32,(dd)(((1582080+(64000*5))+223)+(171*320))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)1));
R(JNE(reerertrteertertrteert));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,tecte2), ds)),32,(dd)0));
R(JMP(reerertrteertertrteert));
dtrerteertertertertertrteertertretrerertertert:
R(CMP(8,READDBl(eax),8,(db)32));
R(JNE(erttrerteertertertertertrteertertretrerertertert));
R(MOV(32,READDD(esi),32,(dd)(((1582080+(64000*5))+223)+(171*320))));
R(JMP(reerertrteertertrteert));
erttrerteertertertertertrteertertretrerertertert:
R(CMP(8,READDBl(eax),8,(db)47));
R(JNE(erttrerteertertertertertrteertertretrererterterty));
R(MOV(32,READDD(esi),32,(dd)(((((1582080+(64000*5))+223)+(172*320))-8)+16)));
R(JMP(reerertrteertertrteert));
erttrerteertertertertertrteertertretrererterterty:
R(CMP(8,READDBl(eax),8,(db)40));
R(JNE(erttrerteertertertertertrteertertretrerertertertyr));
R(MOV(32,READDD(esi),32,(dd)((((((1582080+(64000*5))+223)+8)+(172*320))-8)+16)));
R(JMP(reerertrteertertrteert));
erttrerteertertertertertrteertertretrerertertertyr:
R(CMP(8,READDBl(eax),8,(db)41));
R(JNE(erttrerteertertertertertrteertertretrerertertertyy));
R(MOV(32,READDD(esi),32,(dd)((((((1582080+(64000*5))+223)+(8*2))+(172*320))-8)+16)));
R(JMP(reerertrteertertrteert));
erttrerteertertertertertrteertertretrerertertertyy:
R(CMP(8,READDBl(eax),8,(db)63));
R(JNE(erttrerteertertertertertrteertertretrerertertertyyf));
R(MOV(32,READDD(esi),32,(dd)((((((1582080+(64000*5))+223)+(8*3))+(172*320))-8)+16)));
R(JMP(reerertrteertertrteert));
erttrerteertertertertertrteertertretrerertertertyyf:
R(SUB(8,READDBl(eax),8,(db)97));
R(CMP(8,READDBl(eax),8,(db)(33-97)));
R(JNE(terterterert));
R(MOV(16,READDW(eax),16,(dw)(288/8)));
R(JMP(trtrtyrtyrty));
terterterert:
R(CMP(8,READDBl(eax),8,(db)(46-97)));
R(JNE(terterterertz));
R(MOV(16,READDW(eax),16,(dw)(296/8)));
R(JMP(trtrtyrtyrty));
terterterertz:
R(CMP(8,READDBl(eax),8,(db)(58-97)));
R(JNE(tderterterertz));
R(MOV(16,READDW(eax),16,(dw)(312/8)));
R(JMP(trtrtyrtyrty));
tderterterertz:
R(CMP(8,READDBl(eax),8,(db)(45-97)));
R(JNE(tderterterertzt));
R(MOV(16,READDW(eax),16,(dw)(304/8)));
R(JMP(trtrtyrtyrty));
tderterterertzt:
R(CMP(8,READDBl(eax),8,(db)(48-97)));
R(JB(trderterterertz));
R(ADD(8,READDBl(eax),8,(db)(26-((48-97)))));
trderterterertz:
trtrtyrtyrty:
R(SHL(16,READDW(eax),16,(dw)3));
R(MOV(32,READDD(esi),32,(dd)((1582080+(64000*5))+(165*320))));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
reerertrteertertrteert:
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,scrolly)+320)))));
R(MOV(32,READDD(ecx),32,(dd)6));
frrr:
R(MOVSD);
R(MOVSD);
R(ADD(32,READDD(edi),32,(dd)320));
R(ADD(32,READDD(esi),32,(dd)(320-8)));
R(DEC(32,(READDD(ecx))));
R(JNZ(frrr));
R(POP(16,(READDW(ds))));
trerteertertertertertrteertertretrerertertert:
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,scrolly)+1)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,scrolly)))));
R(MOV(32,READDD(ecx),32,(dd)(328*6)));
R(REP_MOVSB);
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,scrolly)))));
R(MOV(32,m.edi.dd.val,32,(((dd)(offsetof(struct Mem,buffer)+(320*192))))));
R(MOV(32,READDD(edx),32,(dd)6));
ddd:
R(MOV(32,READDD(ecx),32,(dd)320));
oooooooooi:
R(LODSB);
R(OR(8,READDBl(eax),8,(db)READDBl(eax)));
R(JZ(retertetyooo));
R(MOV(8,*((db *) realAddress(READDD(edi), es)),8,(db)READDBl(eax)));
retertetyooo:
R(INC(32,(READDD(edi))));
R(DEC(32,(READDD(ecx))));
R(JNZ(oooooooooi));
R(ADD(32,READDD(esi),32,(dd)8));
R(DEC(32,(READDD(edx))));
R(JNZ(ddd));
RET;

//PROC contamination
contamination:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)0));
R(JE(decremertertrterteterrteertx286));
R(DEC(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds))));
R(JMP(decremooox286));
decremertertrterteterrteertx286:
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
decremooox286:
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)0));
R(JE(decremertertrterteterrteertx287));
R(DEC(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds))));
R(JMP(decremooox287));
decremertertrterteterrteertx287:
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
decremooox287:
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)0));
R(JE(decremertertrterteterrteertx288));
R(DEC(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds))));
R(JMP(decremooox288));
decremertertrterteterrteertx288:
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
decremooox288:
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)0));
R(JE(decremertertrterteterrteertx289));
R(DEC(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds))));
R(JMP(decremooox289));
decremertertrterteterrteertx289:
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
decremooox289:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)0));
R(JE(decremertertrterteterrteertx290));
R(DEC(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds))));
R(JMP(decremooox290));
decremertertrterteterrteertx290:
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
decremooox290:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)0));
R(JE(decremertertrterteterrteertx291));
R(DEC(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds))));
R(JMP(decremooox291));
decremertertrterteterrteertx291:
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
decremooox291:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)0));
R(JE(decremertertrterteterrteertx292));
R(DEC(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds))));
R(JMP(decremooox292));
decremertertrterteterrteertx292:
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
decremooox292:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)0));
R(JE(decremertertrterteterrteertx293));
R(DEC(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds))));
R(JMP(decremooox293));
decremertertrterteterrteertx293:
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
decremooox293:
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JE(contareterterrtertertx294));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(contareterterrtertertx294));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds))));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x483));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(fooo2x483));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JNE(fooo2x483));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)1));
R(JE(fooo2x483));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)2));
R(JE(fooo2x483));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)duree_conta));
fooo2x483:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x484));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(fooo2x484));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JNE(fooo2x484));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)1));
R(JE(fooo2x484));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)2));
R(JE(fooo2x484));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)duree_conta));
fooo2x484:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x485));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(fooo2x485));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JNE(fooo2x485));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)1));
R(JE(fooo2x485));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)2));
R(JE(fooo2x485));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)duree_conta));
fooo2x485:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x486));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(fooo2x486));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JNE(fooo2x486));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)1));
R(JE(fooo2x486));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)2));
R(JE(fooo2x486));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)duree_conta));
fooo2x486:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x487));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(fooo2x487));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JNE(fooo2x487));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)1));
R(JE(fooo2x487));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)2));
R(JE(fooo2x487));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)duree_conta));
fooo2x487:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x488));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(fooo2x488));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JNE(fooo2x488));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)1));
R(JE(fooo2x488));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)2));
R(JE(fooo2x488));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)duree_conta));
fooo2x488:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x489));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(fooo2x489));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JNE(fooo2x489));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)1));
R(JE(fooo2x489));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)2));
R(JE(fooo2x489));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)duree_conta));
fooo2x489:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x490));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(fooo2x490));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JNE(fooo2x490));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)1));
R(JE(fooo2x490));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)2));
R(JE(fooo2x490));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)duree_conta));
fooo2x490:
contareterterrtertertx294:
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JE(contareterterrtertertx295));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(contareterterrtertertx295));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds))));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x491));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(fooo2x491));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JNE(fooo2x491));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)1));
R(JE(fooo2x491));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)2));
R(JE(fooo2x491));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)duree_conta));
fooo2x491:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x492));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(fooo2x492));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JNE(fooo2x492));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)1));
R(JE(fooo2x492));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)2));
R(JE(fooo2x492));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)duree_conta));
fooo2x492:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x493));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(fooo2x493));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JNE(fooo2x493));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)1));
R(JE(fooo2x493));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)2));
R(JE(fooo2x493));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)duree_conta));
fooo2x493:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x494));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(fooo2x494));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JNE(fooo2x494));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)1));
R(JE(fooo2x494));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)2));
R(JE(fooo2x494));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)duree_conta));
fooo2x494:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x495));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(fooo2x495));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JNE(fooo2x495));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)1));
R(JE(fooo2x495));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)2));
R(JE(fooo2x495));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)duree_conta));
fooo2x495:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x496));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(fooo2x496));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JNE(fooo2x496));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)1));
R(JE(fooo2x496));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)2));
R(JE(fooo2x496));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)duree_conta));
fooo2x496:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x497));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(fooo2x497));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JNE(fooo2x497));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)1));
R(JE(fooo2x497));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)2));
R(JE(fooo2x497));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)duree_conta));
fooo2x497:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x498));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(fooo2x498));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JNE(fooo2x498));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)1));
R(JE(fooo2x498));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)2));
R(JE(fooo2x498));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)duree_conta));
fooo2x498:
contareterterrtertertx295:
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JE(contareterterrtertertx296));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(contareterterrtertertx296));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds))));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x499));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(fooo2x499));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JNE(fooo2x499));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)1));
R(JE(fooo2x499));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)2));
R(JE(fooo2x499));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)duree_conta));
fooo2x499:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x500));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(fooo2x500));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JNE(fooo2x500));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)1));
R(JE(fooo2x500));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)2));
R(JE(fooo2x500));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)duree_conta));
fooo2x500:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x501));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(fooo2x501));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JNE(fooo2x501));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)1));
R(JE(fooo2x501));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)2));
R(JE(fooo2x501));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)duree_conta));
fooo2x501:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x502));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(fooo2x502));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JNE(fooo2x502));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)1));
R(JE(fooo2x502));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)2));
R(JE(fooo2x502));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)duree_conta));
fooo2x502:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x503));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(fooo2x503));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JNE(fooo2x503));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)1));
R(JE(fooo2x503));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)2));
R(JE(fooo2x503));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)duree_conta));
fooo2x503:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x504));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(fooo2x504));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JNE(fooo2x504));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)1));
R(JE(fooo2x504));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)2));
R(JE(fooo2x504));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)duree_conta));
fooo2x504:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x505));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(fooo2x505));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JNE(fooo2x505));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)1));
R(JE(fooo2x505));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)2));
R(JE(fooo2x505));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)duree_conta));
fooo2x505:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x506));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(fooo2x506));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JNE(fooo2x506));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)1));
R(JE(fooo2x506));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)2));
R(JE(fooo2x506));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)duree_conta));
fooo2x506:
contareterterrtertertx296:
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JE(contareterterrtertertx297));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(contareterterrtertertx297));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds))));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x507));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(fooo2x507));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JNE(fooo2x507));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)1));
R(JE(fooo2x507));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)2));
R(JE(fooo2x507));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)duree_conta));
fooo2x507:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x508));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(fooo2x508));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JNE(fooo2x508));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)1));
R(JE(fooo2x508));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)2));
R(JE(fooo2x508));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)duree_conta));
fooo2x508:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x509));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(fooo2x509));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JNE(fooo2x509));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)1));
R(JE(fooo2x509));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)2));
R(JE(fooo2x509));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)duree_conta));
fooo2x509:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x510));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(fooo2x510));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JNE(fooo2x510));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)1));
R(JE(fooo2x510));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)2));
R(JE(fooo2x510));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)duree_conta));
fooo2x510:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x511));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(fooo2x511));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JNE(fooo2x511));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)1));
R(JE(fooo2x511));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)2));
R(JE(fooo2x511));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)duree_conta));
fooo2x511:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x512));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(fooo2x512));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JNE(fooo2x512));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)1));
R(JE(fooo2x512));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)2));
R(JE(fooo2x512));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)duree_conta));
fooo2x512:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x513));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(fooo2x513));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JNE(fooo2x513));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)1));
R(JE(fooo2x513));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)2));
R(JE(fooo2x513));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)duree_conta));
fooo2x513:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x514));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(fooo2x514));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JNE(fooo2x514));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)1));
R(JE(fooo2x514));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)2));
R(JE(fooo2x514));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)duree_conta));
fooo2x514:
contareterterrtertertx297:
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JE(contareterterrtertertx298));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(contareterterrtertertx298));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds))));
R(MOV(16,READDW(ebx),16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x515));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(fooo2x515));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JNE(fooo2x515));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)1));
R(JE(fooo2x515));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)2));
R(JE(fooo2x515));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)duree_conta));
fooo2x515:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x516));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(fooo2x516));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JNE(fooo2x516));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)1));
R(JE(fooo2x516));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)2));
R(JE(fooo2x516));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)duree_conta));
fooo2x516:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x517));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(fooo2x517));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JNE(fooo2x517));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)1));
R(JE(fooo2x517));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)2));
R(JE(fooo2x517));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)duree_conta));
fooo2x517:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x518));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(fooo2x518));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JNE(fooo2x518));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)1));
R(JE(fooo2x518));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)2));
R(JE(fooo2x518));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)duree_conta));
fooo2x518:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x519));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(fooo2x519));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JNE(fooo2x519));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)1));
R(JE(fooo2x519));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)2));
R(JE(fooo2x519));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)duree_conta));
fooo2x519:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x520));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(fooo2x520));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JNE(fooo2x520));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)1));
R(JE(fooo2x520));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)2));
R(JE(fooo2x520));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)duree_conta));
fooo2x520:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x521));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(fooo2x521));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JNE(fooo2x521));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)1));
R(JE(fooo2x521));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)2));
R(JE(fooo2x521));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)duree_conta));
fooo2x521:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x522));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(fooo2x522));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JNE(fooo2x522));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)1));
R(JE(fooo2x522));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)2));
R(JE(fooo2x522));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)duree_conta));
fooo2x522:
contareterterrtertertx298:
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JE(contareterterrtertertx299));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(contareterterrtertertx299));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds))));
R(MOV(16,READDW(ebx),16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x523));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(fooo2x523));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JNE(fooo2x523));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)1));
R(JE(fooo2x523));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)2));
R(JE(fooo2x523));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)duree_conta));
fooo2x523:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x524));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(fooo2x524));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JNE(fooo2x524));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)1));
R(JE(fooo2x524));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)2));
R(JE(fooo2x524));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)duree_conta));
fooo2x524:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x525));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(fooo2x525));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JNE(fooo2x525));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)1));
R(JE(fooo2x525));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)2));
R(JE(fooo2x525));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)duree_conta));
fooo2x525:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x526));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(fooo2x526));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JNE(fooo2x526));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)1));
R(JE(fooo2x526));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)2));
R(JE(fooo2x526));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)duree_conta));
fooo2x526:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x527));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(fooo2x527));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JNE(fooo2x527));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)1));
R(JE(fooo2x527));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)2));
R(JE(fooo2x527));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)duree_conta));
fooo2x527:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x528));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(fooo2x528));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JNE(fooo2x528));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)1));
R(JE(fooo2x528));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)2));
R(JE(fooo2x528));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)duree_conta));
fooo2x528:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x529));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(fooo2x529));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JNE(fooo2x529));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)1));
R(JE(fooo2x529));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)2));
R(JE(fooo2x529));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)duree_conta));
fooo2x529:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x530));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(fooo2x530));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JNE(fooo2x530));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)1));
R(JE(fooo2x530));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)2));
R(JE(fooo2x530));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)duree_conta));
fooo2x530:
contareterterrtertertx299:
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JE(contareterterrtertertx300));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(contareterterrtertertx300));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds))));
R(MOV(16,READDW(ebx),16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x531));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(fooo2x531));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JNE(fooo2x531));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)1));
R(JE(fooo2x531));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)2));
R(JE(fooo2x531));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)duree_conta));
fooo2x531:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x532));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(fooo2x532));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JNE(fooo2x532));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)1));
R(JE(fooo2x532));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)2));
R(JE(fooo2x532));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)duree_conta));
fooo2x532:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x533));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(fooo2x533));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JNE(fooo2x533));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)1));
R(JE(fooo2x533));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)2));
R(JE(fooo2x533));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)duree_conta));
fooo2x533:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x534));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(fooo2x534));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JNE(fooo2x534));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)1));
R(JE(fooo2x534));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)2));
R(JE(fooo2x534));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)duree_conta));
fooo2x534:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x535));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(fooo2x535));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JNE(fooo2x535));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)1));
R(JE(fooo2x535));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)2));
R(JE(fooo2x535));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)duree_conta));
fooo2x535:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x536));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(fooo2x536));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JNE(fooo2x536));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)1));
R(JE(fooo2x536));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)2));
R(JE(fooo2x536));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)duree_conta));
fooo2x536:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x537));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(fooo2x537));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JNE(fooo2x537));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)1));
R(JE(fooo2x537));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)2));
R(JE(fooo2x537));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)duree_conta));
fooo2x537:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x538));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(fooo2x538));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JNE(fooo2x538));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)1));
R(JE(fooo2x538));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)2));
R(JE(fooo2x538));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)duree_conta));
fooo2x538:
contareterterrtertertx300:
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JE(contareterterrtertertx301));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(contareterterrtertertx301));
R(MOV(32,READDD(eax),32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds))));
R(MOV(16,READDW(ebx),16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+0), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x539));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+0), ds)),32,(dd)1));
R(JNE(fooo2x539));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)0));
R(JNE(fooo2x539));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)1));
R(JE(fooo2x539));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+0), ds)),16,(dw)2));
R(JE(fooo2x539));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+0), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+2), ds)),16,(dw)duree_conta));
fooo2x539:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+4), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x540));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+4), ds)),32,(dd)1));
R(JNE(fooo2x540));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)0));
R(JNE(fooo2x540));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)1));
R(JE(fooo2x540));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+4), ds)),16,(dw)2));
R(JE(fooo2x540));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+4), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+2), ds)),16,(dw)duree_conta));
fooo2x540:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+8), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x541));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+8), ds)),32,(dd)1));
R(JNE(fooo2x541));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)0));
R(JNE(fooo2x541));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)1));
R(JE(fooo2x541));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+8), ds)),16,(dw)2));
R(JE(fooo2x541));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+8), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+2), ds)),16,(dw)duree_conta));
fooo2x541:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,last_bomb)+12), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x542));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+12), ds)),32,(dd)1));
R(JNE(fooo2x542));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)0));
R(JNE(fooo2x542));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)1));
R(JE(fooo2x542));
R(CMP(16,read_dw(realAddress((offsetof(struct Mem,lapipipino2)+12), ds)),16,(dw)2));
R(JE(fooo2x542));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,maladie)+12), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+2), ds)),16,(dw)duree_conta));
fooo2x542:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+0)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x543));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+0)+16), ds)),32,(dd)1));
R(JNE(fooo2x543));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)0));
R(JNE(fooo2x543));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)1));
R(JE(fooo2x543));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+0)+16), ds)),16,(dw)2));
R(JE(fooo2x543));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+0)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+0)+16)+2), ds)),16,(dw)duree_conta));
fooo2x543:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+4)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x544));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+4)+16), ds)),32,(dd)1));
R(JNE(fooo2x544));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)0));
R(JNE(fooo2x544));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)1));
R(JE(fooo2x544));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+4)+16), ds)),16,(dw)2));
R(JE(fooo2x544));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+4)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+4)+16)+2), ds)),16,(dw)duree_conta));
fooo2x544:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+8)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x545));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+8)+16), ds)),32,(dd)1));
R(JNE(fooo2x545));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)0));
R(JNE(fooo2x545));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)1));
R(JE(fooo2x545));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+8)+16), ds)),16,(dw)2));
R(JE(fooo2x545));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+8)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+8)+16)+2), ds)),16,(dw)duree_conta));
fooo2x545:
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,last_bomb)+12)+16), ds)),32,(dd)READDD(eax)));
R(JNE(fooo2x546));
R(CMP(32,read_dd(realAddress(((offsetof(struct Mem,vie)+12)+16), ds)),32,(dd)1));
R(JNE(fooo2x546));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)0));
R(JNE(fooo2x546));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)1));
R(JE(fooo2x546));
R(CMP(16,read_dw(realAddress(((offsetof(struct Mem,lapipipino2)+12)+16), ds)),16,(dw)2));
R(JE(fooo2x546));
R(MOV(16,read_dw(realAddress(((offsetof(struct Mem,maladie)+12)+16), ds)),16,(dw)READDW(ebx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,maladie)+12)+16)+2), ds)),16,(dw)duree_conta));
fooo2x546:
contareterterrtertertx301:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC horloge
horloge:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(TEST(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)32768));
R(JZ(clignote));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
clignote:
R(MOV(16,READDW(ebp),16,read_dw(realAddress(offsetof(struct Mem,temps), ds))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(LEA(32,m.edi.dd.val,32,(((dd)((offsetof(struct Mem,buffer)+(320*183))+277)))));
R(PUSH(32,(READDD(edi))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,(dw)READDW(ebp)));
R(SHR(16,READDW(eax),16,(dw)8));
R(AND(16,READDW(eax),16,(dw)15));
R(MOV(32,READDD(esi),32,(dd)(((((896000+384000)+46080)+128000)+(83*320))+80)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(SPRITE_16_11);
R(MOV(32,READDD(esi),32,(dd)((((((896000+384000)+46080)+128000)+(83*320))+80)+(10*16))));
R(POP(32,(READDD(edi))));
R(PUSH(32,(READDD(edi))));
R(ADD(32,READDD(edi),32,(dd)12));
R(TEST(16,read_dw(realAddress(offsetof(struct Mem,temps), es)),16,(dw)16384));
R(JNZ(affiche_pas_deuxpoint));
R(SPRITE_16_5);
affiche_pas_deuxpoint:
R(POP(32,(READDD(edi))));
R(PUSH(32,(READDD(edi))));
R(ADD(32,READDD(edi),32,(dd)(12+6)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,(dw)READDW(ebp)));
R(SHR(16,READDW(eax),16,(dw)4));
R(AND(16,READDW(eax),16,(dw)15));
R(MOV(32,READDD(esi),32,(dd)(((((896000+384000)+46080)+128000)+(83*320))+80)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(SPRITE_16_11);
R(POP(32,(READDD(edi))));
R(ADD(32,READDD(edi),32,(dd)(24+6)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,(dw)READDW(ebp)));
R(AND(16,READDW(eax),16,(dw)15));
R(MOV(32,READDD(esi),32,(dd)(((((896000+384000)+46080)+128000)+(83*320))+80)));
R(SHL(32,READDD(eax),32,(dd)4));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(SPRITE_16_11);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC gestion_bdraw
gestion_bdraw:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,balance_le_bdrawn), ds)),8,(db)0));
R(JNE(ereretereterreer233));
RET;
ereretereterreer233:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_med), ds)),32,(dd)attente_avant_med2));
R(JNE(ertertertertetrertertertzet));
R(CMP(16,read_dw(realAddress(offsetof(struct Mem,bdraw666), ds)),16,(dw)14649));
R(JE(kjmlkjjkmlkjlmjklmjmkl));
R(DEC(8,*((db *) realAddress(offsetof(struct Mem,bdraw1), ds))));
R(JNZ(kjmlkjjkmlkjlmjklmjmkl));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,bdraw1), ds)),8,(db)60));
R(DEC(8,*((db *) realAddress((offsetof(struct Mem,bdraw666)+1), ds))));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,bdraw666)+1), ds)),8,(db)(48-1)));
R(JNE(kjmlkjjkmlkjlmjklmjmkl));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,bdraw666)+1), ds)),8,(db)57));
R(DEC(8,*((db *) realAddress(offsetof(struct Mem,bdraw666), ds))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,bdraw666), ds)),8,(db)(48-1)));
R(JNE(kjmlkjjkmlkjlmjklmjmkl));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,bdraw666), ds)),8,(db)57));
kjmlkjjkmlkjlmjklmjmkl:
R(CMP(16,read_dw(realAddress(offsetof(struct Mem,adder_bdraw), ds)),16,(dw)0));
R(JE(ertertertertetrertertertzet));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)1));
R(JE(ertertertertetrertertertzet));
R(SUB(16,read_dw(realAddress(offsetof(struct Mem,adder_bdraw), ds)),16,(dw)320));
ertertertertetrertertertzet:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC dec_temps
dec_temps:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(TEST(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)4095));
R(JZ(zerooo));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,temps2), ds)),8,(db)15));
R(JNE(nonononoiuioiohjrr));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,special_clignotement), ds)),32,(dd)0));
R(JE(dommage_pp));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,special_clignotement), ds))));
R(JMP(special_fete));
dommage_pp:
R(OR(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)16384));
R(MOV(16,READDW(eax),16,read_dw(realAddress(offsetof(struct Mem,temps), ds))));
R(AND(16,READDW(eax),16,(dw)16383));
R(CMP(16,READDW(eax),16,(dw)17));
R(JA(nonononoiuioiohjrr));
special_fete:
R(OR(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)32768));
nonononoiuioiohjrr:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,temps2), ds)),8,(db)1));
R(JNE(nonononoiuioiohjrrt));
R(AND(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)49151));
R(TEST(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)32768));
R(JZ(nonononoiuioiohjrrt));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)6));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx309));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx309:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(AND(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)32767));
nonononoiuioiohjrrt:
R(DEC(8,*((db *) realAddress(offsetof(struct Mem,temps2), ds))));
R(JZ(ertterrtyrtyrtyyrt));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
zerooo:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)6));
R(JNE(ertrtytyuyuiiyuughfdfgdfgfgdrtyrtyrtyerertertert));
CALL(pose_une_bombe_bonus);
ertrtytyuyuiiyuughfdfgdfgfgdrtyrtyrtyerertertert:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,in_the_apocalypse), ds)),8,(db)0));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc_fin)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,truc)))));
R(MOV(32,READDD(ecx),32,(dd)(32*13)));
nextooi:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JE(nextooo));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,in_the_apocalypse), ds)),8,(db)1));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,truc_fin)+(32*13)), ds))));
R(TEST(32,read_dd(realAddress(offsetof(struct Mem,changement), ds)),32,(dd)READDD(eax)));
R(JNZ(nextooo));
R(DEC(8,*((db *) realAddress(READDD(esi), ds))));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)133));
R(JNE(nextooo567888888_));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,special_nivo_6), ds)),32,(dd)60));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,balance_le_bdrawn), ds)),8,(db)1));
ertrtytyuyuiiyuughfdfgdfgfgdrtyrtyrtyertetttrrttrt:
R(CMP(8,*((db *) realAddress(READDD(edi), ds)),8,(db)2));
R(JNE(nextoooy));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,terrain), ds)),8,(db)6));
R(JE(ertrtytyuyuiiyuughfdfgdfgfgdrtyrtyrty));
R(MOV(8,*((db *) realAddress(READDD(edi), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((READDD(edi)+(32*13)), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)4));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx311));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx311:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(nextoooy));
ertrtytyuyuiiyuughfdfgdfgfgdrtyrtyrty:
R(MOV(8,*((db *) realAddress(READDD(edi), ds)),8,(db)3));
nextoooy:
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JMP(nextooo));
nextooo567888888_:
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JNE(nextooo));
R(CMP(8,*((db *) realAddress(READDD(edi), ds)),8,(db)1));
R(JE(nextooo));
R(MOV(8,*((db *) realAddress(READDD(edi), ds)),8,(db)11));
R(MOV(8,*((db *) realAddress((READDD(edi)+(32*13)), ds)),8,(db)194));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)4));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx312));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx312:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
nextooo:
R(INC(32,(READDD(edi))));
R(INC(32,(READDD(esi))));
R(DEC(32,(READDD(ecx))));
R(JNZ(nextooi));
finto_pas_cetelmk:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
ertterrtyrtyrtyyrt:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,temps2), ds)),8,(db)59));
R(MOV(16,READDW(eax),16,read_dw(realAddress(offsetof(struct Mem,temps), ds))));
R(AND(16,READDW(eax),16,(dw)15));
R(DEC(16,(READDW(eax))));
R(CMP(16,READDW(eax),16,(dw)((0-1))));
R(JNE(pas_zeroret));
R(MOV(16,READDW(eax),16,(dw)9));
R(JMP(canal_sux));
pas_zeroret:
R(AND(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)65520));
R(OR(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)READDW(eax)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
canal_sux:
R(AND(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)65520));
R(OR(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)READDW(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(offsetof(struct Mem,temps), ds))));
R(SHR(16,READDW(eax),16,(dw)4));
R(AND(16,READDW(eax),16,(dw)15));
R(DEC(16,(READDW(eax))));
R(CMP(16,READDW(eax),16,(dw)((0-1))));
R(JNE(pas_zeroret7));
R(MOV(16,READDW(eax),16,(dw)5));
R(JMP(stade));
pas_zeroret7:
R(SHL(16,READDW(eax),16,(dw)4));
R(AND(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)65295));
R(OR(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)READDW(eax)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
stade:
R(SHL(16,READDW(eax),16,(dw)4));
R(AND(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)65295));
R(OR(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)READDW(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(offsetof(struct Mem,temps), ds))));
R(SHR(16,READDW(eax),16,(dw)8));
R(DEC(16,(READDW(eax))));
R(CMP(16,READDW(eax),16,(dw)((0-1))));
R(JNE(pas_zeroret72));
R(MOV(16,READDW(eax),16,(dw)9));
R(JMP(stade));
pas_zeroret72:
R(SHL(16,READDW(eax),16,(dw)8));
R(AND(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)61695));
R(OR(16,read_dw(realAddress(offsetof(struct Mem,temps), ds)),16,(dw)READDW(eax)));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC gestion_blanchiment
gestion_blanchiment:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
verite:
R(MOV(32,READDD(eax),32,(dd)0));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds)),32,(dd)0));
R(JE(bababh));
R(DEC(32,read_dd(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds))));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,invinsible)+READDD(ebp)), ds))));
R(AND(32,READDD(eax),32,(dd)1023));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,blinking)+READDD(eax)), ds))));
R(AND(32,READDD(eax),32,(dd)1));
bababh:
R(MOV(32,read_dd(realAddress((offsetof(struct Mem,clignotement)+READDD(ebp)), ds)),32,(dd)READDD(eax)));
R(ADD(32,READDD(ebp),32,(dd)4));
R(CMP(32,READDD(ebp),32,(dd)(4*8)));
R(JNE(verite));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC nike_toutes_ses_bombes
nike_toutes_ses_bombes:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,liste_bombe), ds))));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(upasdutout));
R(LEA(32,m.esi.dd.val,32,(((dd)((offsetof(struct Mem,liste_bombe)+4)-taille_dune_info_bombe)))));
unext_bomby:
R(ADD(32,READDD(esi),32,(dd)taille_dune_info_bombe));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)0));
R(JNE(uya_bombe_ici));
R(JMP(unext_bomby));
uya_bombe_ici:
R(CMP(32,read_dd(realAddress(READDD(esi), ds)),32,(dd)READDD(ebx)));
R(JNE(unon_dejar));
R(CMP(16,read_dw(realAddress(((READDD(esi)+(4*3))+2), ds)),16,(dw)1));
R(JNE(unon_dejar));
R(MOV(16,read_dw(realAddress(((READDD(esi)+(4*3))+2), ds)),16,(dw)2));
unon_dejar:
R(DEC(32,(READDD(ecx))));
R(JNZ(unext_bomby));
upasdutout:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC nike_toutes_les_bombes
nike_toutes_les_bombes:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,liste_bombe), ds))));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(upasdutoutu));
R(LEA(32,m.esi.dd.val,32,(((dd)((offsetof(struct Mem,liste_bombe)+4)-taille_dune_info_bombe)))));
unext_bombyu:
R(ADD(32,READDD(esi),32,(dd)taille_dune_info_bombe));
R(CMP(32,read_dd(realAddress((READDD(esi)+4), ds)),32,(dd)0));
R(JNE(uya_bombe_iciu));
R(JMP(unext_bombyu));
uya_bombe_iciu:
R(MOV(16,read_dw(realAddress(((READDD(esi)+(4*3))+2), ds)),16,(dw)0));
R(MOV(32,read_dd(realAddress((READDD(esi)+(1*4)), ds)),32,(dd)1));
R(DEC(32,(READDD(ecx))));
R(JNZ(unext_bombyu));
upasdutoutu:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC pousse_la_bombe
pousse_la_bombe:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc)))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
ttyrrtyrtyrtyrtytyrrtyrtyyrtrtye:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)0));
R(JE(cherche_encore));
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JNE(cherche_encore));
R(JMP(okey_on_puse));
cherche_encore:
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(ttyrrtyrtyrtyrtytyrrtyrtyyrtrtye));
okey_on_puse:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(5*4)), ds)),16,(dw)0));
R(JNE(peu_pas_pousser));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(5*4))+2), ds)),16,(dw)0));
R(JNE(peu_pas_pousser));
R(CMP(32,READDD(esi),32,(dd)(32*11)));
R(JB(pas_ce_cas_larrr));
R(CMP(32,READDD(ecx),32,(dd)2));
R(JE(peu_pas_pousser));
pas_ce_cas_larrr:
R(CMP(32,READDD(esi),32,(dd)(32*2)));
R(JNB(pas_ce_cas_larrr2));
R(CMP(32,READDD(ecx),32,(dd)2));
R(JE(peu_pas_pousser));
pas_ce_cas_larrr2:
R(AND(32,READDD(esi),32,(dd)31));
R(CMP(32,READDD(esi),32,(dd)1));
R(JNE(pas_ce_cas3));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(peu_pas_pousser));
pas_ce_cas3:
R(CMP(32,READDD(esi),32,(dd)17));
R(JNE(pas_ce_cas5));
R(OR(32,READDD(ecx),32,(dd)READDD(ecx)));
R(JZ(peu_pas_pousser));
pas_ce_cas5:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),32,(dd)0));
R(ADD(32,READDD(ebx),32,(dd)READDD(ecx)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)READDW(eax)));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)13));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx324));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx324:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)34));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
peu_pas_pousser:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC fabrique_monstro_truc
fabrique_monstro_truc:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,truc_monstre)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)((32*13)/4)));
R(REP_STOSD);
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
zecompetion:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,vie)+READDD(ebp)), ds)),32,(dd)1));
R(JNE(fdggrtetyrklmjyurtmkljrtymjklyut));
R(PUSH(32,(READDD(ebp))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(SHR(32,READDD(ebp),32,(dd)1));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(nb_dyna*2))+READDD(ebp)), ds))));
R(ADD(16,READDW(eax),16,(dw)14));
R(MOV(16,READDW(ebx),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebp)), ds))));
R(AND(16,READDW(eax),16,(dw)65520));
R(ADD(16,READDW(ebx),16,(dw)3));
R(SHL(16,READDW(eax),16,(dw)1));
R(SHR(16,READDW(ebx),16,(dw)4));
R(ADD(16,READDW(eax),16,(dw)READDW(ebx)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(eax)), ds)),8,(db)1));
R(POP(32,(READDD(ebp))));
fdggrtetyrklmjyurtmkljrtymjklyut:
R(ADD(32,READDD(ebp),32,(dd)4));
R(CMP(32,READDD(ebp),32,(dd)(8*4)));
R(JNE(zecompetion));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC pose_une_bombe_bonus
pose_une_bombe_bonus:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente_avant_med), ds)),32,(dd)attente_avant_med2));
R(JNE(errteerterttyjtyutyuutytyuyutyutyututy));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente_entre_chake_bombe), ds)),32,(dd)0));
R(JE(okokokokok_cette_fois));
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,attente_entre_chake_bombe), ds))));
R(JMP(errteerterttyjtyutyuutytyuyutyutyututy));
okokokokok_cette_fois:
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,viseur__nouvelle_attente_entre_chake_bombe), ds)),32,(dd)4));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nouvelle_attente_entre_chake_bombe2)))));
R(ADD(32,READDD(esi),32,read_dd(realAddress(offsetof(struct Mem,viseur__nouvelle_attente_entre_chake_bombe), ds))));
R(CMP(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,viseur__nouvelle_attente_entre_chake_bombe)))));
R(JNE(ertteertertertertter));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,acceleration), ds)),32,(dd)20));
R(JE(retteterrterteterertert));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,acceleration), ds))));
retteterrterteterertert:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,viseur__nouvelle_attente_entre_chake_bombe), ds)),32,(dd)0));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,nouvelle_attente_entre_chake_bombe2)))));
ertteertertertertter:
R(MOV(32,READDD(ebx),32,read_dd(realAddress(READDD(esi), ds))));
R(SUB(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,acceleration), ds))));
R(JS(loupe));
R(JZ(loupe));
R(JMP(rrtyertyrtyrtyrtyertyrtytyr));
loupe:
R(MOV(32,READDD(ebx),32,(dd)1));
rrtyertyrtyrtyrtyertyrtytyr:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente_entre_chake_bombe), ds)),32,(dd)READDD(ebx)));
R(MOV(32,READDD(ebx),32,read_dd(realAddress(offsetof(struct Mem,liste_bombbbb2), ds))));
R(ADD(32,READDD(ebx),32,(dd)12));
R(LEA(32,m.esi.dd.val,32,(((dd)(offsetof(struct Mem,liste_bombbbb)+READDD(ebx))))));
R(CMP(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,liste_bombbbb2)))));
R(JNE(e6rtertertert));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
e6rtertertert:
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,liste_bombbbb2), ds)),32,(dd)READDD(ebx)));
R(MOV(32,READDD(eax),32,read_dd(realAddress((offsetof(struct Mem,liste_bombbbb)+READDD(ebx)), ds))));
R(MOV(16,READDW(edx),16,read_dw(realAddress(((offsetof(struct Mem,liste_bombbbb)+4)+READDD(ebx)), ds))));
R(MOV(16,READDW(ecx),16,read_dw(realAddress(((offsetof(struct Mem,liste_bombbbb)+6)+READDD(ebx)), ds))));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(((offsetof(struct Mem,liste_bombbbb)+8)+READDD(ebx)), ds))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(ADD(32,READDD(esi),32,(dd)READDD(eax)));
R(CMP(8,*((db *) realAddress((READDD(esi)-(32*13)), ds)),8,(db)0));
R(JNE(ytreteterrterteterertter));
R(CMP(8,*((db *) realAddress(READDD(esi), ds)),8,(db)0));
R(JNE(ytreteterrterteterertter));
R(MOV(8,*((db *) realAddress(READDD(esi), ds)),8,(db)1));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
yttyrrtyrtyrtyrtytyrrtyrtyyrtrty:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)0));
R(JE(yerertertrteterert));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(yttyrrtyrtyrtyrtytyrrtyrtyyrtrty));
yerertertrteterert:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(eax)), ds)),8,(db)READDBl(edx)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(eax)), ds)),8,(db)0));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(0*4)), ds)),32,(((dd)offsetof(struct Mem,infojoueur2)))));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)READDD(ebp)));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(eax)));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(3*4)), ds)),16,(dw)3));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(3*4))+2), ds)),16,(dw)0));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)READDW(ecx)));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(MOV(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(5*4)), ds)),16,(dw)READDW(edx)));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(5*4))+2), ds)),16,(dw)0));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,liste_bombe), ds))));
ytreteterrterteterertter:
errteerterttyjtyutyuutytyuyutyutyututy:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC calc_ombres
calc_ombres:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,kel_ombre), ds)),32,(dd)0));
mertrymrtejklertyjklmtyer:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino)+READDD(ebp)), ds)),32,(dd)0));
R(JE(pas_un_lapin_345));
R(MOV(32,READDD(eax),32,(dd)1));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)6));
R(JB(aaaaaa));
R(MOV(32,READDD(eax),32,(dd)2));
aaaaaa:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino5)+READDD(ebp)), ds)),32,(dd)12));
R(JB(aaaaaa2));
R(MOV(32,READDD(eax),32,(dd)3));
aaaaaa2:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino2)+READDD(ebp)), ds)),32,(dd)3));
R(JNE(veuoooooooooooi));
R(MOV(32,READDD(eax),32,(dd)4));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)27));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)24));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)21));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)18));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)16));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)14));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)11));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)8));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,lapipipino3)+READDD(ebp)), ds)),32,(dd)5));
R(JA(reeeeeeeeeet));
R(INC(32,(READDD(eax))));
reeeeeeeeeet:
R(MOV(32,READDD(ecx),32,(dd)READDD(ebp)));
R(SHL(32,READDD(eax),8,(db)READDBl(ecx)));
R(OR(32,read_dd(realAddress(offsetof(struct Mem,kel_ombre), ds)),32,(dd)READDD(eax)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(8*2))+READDD(ebx)), ds))));
R(ADD(16,READDW(eax),16,(dw)15));
R(EAX_X_320);
R(ADD(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebx)), ds))));
R(ADD(16,READDW(eax),16,(dw)4));
R(SUB(16,READDW(eax),16,(dw)8));
R(SUB(16,READDW(eax),16,(dw)(320*19)));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,ombres)+READDD(ebx)), ds)),16,(dw)READDW(eax)));
R(JMP(pas_un_lapin_345));
veuoooooooooooi:
R(MOV(32,READDD(ecx),32,(dd)READDD(ebp)));
R(SHL(32,READDD(eax),8,(db)READDBl(ecx)));
R(OR(32,read_dd(realAddress(offsetof(struct Mem,kel_ombre), ds)),32,(dd)READDD(eax)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress(((offsetof(struct Mem,donnee)+(8*2))+READDD(ebx)), ds))));
R(ADD(16,READDW(eax),16,(dw)15));
R(EAX_X_320);
R(ADD(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,donnee)+READDD(ebx)), ds))));
R(ADD(16,READDW(eax),16,(dw)4));
R(MOV(16,read_dw(realAddress((offsetof(struct Mem,ombres)+READDD(ebx)), ds)),16,(dw)READDW(eax)));
pas_un_lapin_345:
R(ADD(32,READDD(ebp),32,(dd)4));
R(ADD(32,READDD(ebx),32,(dd)2));
R(CMP(32,READDD(ebx),32,(dd)(2*8)));
R(JNE(mertrymrtejklertyjklmtyer));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC aff_ombres
aff_ombres:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(esi),32,(dd)(1582080+(64000*5))));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,buffer)))));
R(PUSH(16,(READDW(fs))));
R(POP(16,(READDW(ds))));
R(XOR(32,READDD(ebp),32,(dd)READDD(ebp)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
rtmklmrtyjklrtymjkrtyjklmrtyrty:
R(MOV(32,READDD(eax),32,(dd)15));
R(MOV(32,READDD(ecx),32,(dd)READDD(ebp)));
R(SHL(32,READDD(eax),8,(db)READDBl(ecx)));
R(MOV(32,READDD(edx),32,read_dd(realAddress(offsetof(struct Mem,kel_ombre), es))));
R(AND(32,READDD(edx),32,(dd)READDD(eax)));
R(SHR(32,READDD(edx),8,(db)READDBl(ecx)));
R(OR(32,READDD(edx),32,(dd)READDD(edx)));
R(JZ(nananan_pas_dombre_ici));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(edi))));
R(PUSH(32,(READDD(ebx))));
R(CMP(32,READDD(edx),32,(dd)2));
R(JNE(oooooorytrtyrtyp));
R(ADD(32,READDD(esi),32,(dd)24));
oooooorytrtyrtyp:
R(CMP(32,READDD(edx),32,(dd)3));
R(JNE(oooooorytrtyrtyp4));
R(ADD(32,READDD(esi),32,(dd)(24*2)));
oooooorytrtyrtyp4:
R(CMP(32,READDD(edx),32,(dd)4));
R(JB(caaaaaaaaaaaaaaaaa));
R(MOV(32,READDD(esi),32,(dd)(((1966080+(64000*8))+1)+(1*320))));
R(TEST(32,READDD(ebp),32,(dd)4));
R(JNZ(c_une_fille));
R(ADD(32,READDD(esi),32,(dd)(320*((33+33)))));
c_une_fille:
R(SUB(32,READDD(edx),32,(dd)4));
R(SHL(32,READDD(edx),32,(dd)5));
R(ADD(32,READDD(esi),32,(dd)READDD(edx)));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,ombres)+READDD(ebx)), es))));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(SPRITE_32_32);
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(edi))));
R(POP(32,(READDD(esi))));
R(JMP(nananan_pas_dombre_ici));
caaaaaaaaaaaaaaaaa:
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,read_dw(realAddress((offsetof(struct Mem,ombres)+READDD(ebx)), es))));
R(ADD(32,READDD(edi),32,(dd)READDD(eax)));
R(ADD(32,READDD(esi),32,(dd)(71+(150*320))));
R(SPRITE_8_16);
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(edi))));
R(POP(32,(READDD(esi))));
nananan_pas_dombre_ici:
R(ADD(32,READDD(ebx),32,(dd)2));
R(ADD(32,READDD(ebp),32,(dd)4));
R(CMP(32,READDD(ebx),32,(dd)(8*2)));
R(JNE(rtmklmrtyjklrtymjkrtyjklmrtyrty));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC deplacement_bombes
deplacement_bombes:
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds)),16,(dw)1));
R(JNE(poussagepas_ca___x335));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)0));
R(JNE(poussagecontinue_lme_train_trainx335));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((1*1))), ds)),8,(db)66));
R(JNE(poussagereterertertertertterertertx335));
R(NEG(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(AND(32,READDD(ebx),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(ebx)), ds))));
R(MOV(8,READDBl(eax),8,(db)3));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2bopx547));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2bopx547:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)45));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagecontinue_lme_train_trainx335));
poussagereterertertertertterertertx335:
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((1*1))), ds)),8,(db)0));
R(JNE(poussagestooppppesx335));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(ebx)), ds)),8,(db)1));
R(JE(poussagestooppppesx335));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*1))), ds)),8,(db)0));
R(JE(poussagestooppppes_pasx335));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*1))), ds)),8,(db)5));
R(JB(poussageya_une_bombe_stope_ou_explosex335));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*1))), ds)),8,(db)54));
R(JA(poussagestooppppesx335));
poussagestooppppes_pasx335:
R(JMP(poussagecontinue_lme_train_trainx335));
poussageya_une_bombe_stope_ou_explosex335:
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)(1*1)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx335:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(poussageerertertrteterertx335));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx335));
poussageerertertrteterertx335:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(JNE(poussagetantpisx335));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(JE(poussagefait_pas_peterx335));
poussagetantpisx335:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(JMP(poussagestooppppesx335));
poussagefait_pas_peterx335:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
poussagestooppppesx335:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)0));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx548));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx548:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagepas_ca___x335));
poussagecontinue_lme_train_trainx335:
R(ADD(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)1));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)(8*(1))));
R(JNE(poussagepas_changement_casex335));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*1))), ds)),8,(db)0));
R(JE(poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx335));
R(SUB(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)1));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*3))+2), ds)),16,(dw)0));
R(JMP(poussagestooppppesx335));
poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx335:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)(((0-8))*(1))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)0));
R(ADD(32,READDD(ebx),32,(dd)(1*1)));
R(ADD(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)(1*1)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)READDBl(eax)));
poussagepas_changement_casex335:
R(MOV(16,READDW(eax),16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,truc_x)+READDD(ebx))+(((13*16)*0))), ds)),8,(db)READDBl(eax)));
poussagepas_ca___x335:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds)),16,(dw)((0-1))));
R(JNE(poussagepas_ca___x336));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)0));
R(JNE(poussagecontinue_lme_train_trainx336));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)66));
R(JNE(poussagereterertertertertterertertx336));
R(NEG(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(AND(32,READDD(ebx),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(ebx)), ds))));
R(MOV(8,READDBl(eax),8,(db)3));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2bopx549));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2bopx549:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)45));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagecontinue_lme_train_trainx336));
poussagereterertertertertterertertx336:
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)0));
R(JNE(poussagestooppppesx336));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(ebx)), ds)),8,(db)1));
R(JE(poussagestooppppesx336));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)0));
R(JE(poussagestooppppes_pasx336));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)5));
R(JB(poussageya_une_bombe_stope_ou_explosex336));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)54));
R(JA(poussagestooppppesx336));
poussagestooppppes_pasx336:
R(JMP(poussagecontinue_lme_train_trainx336));
poussageya_une_bombe_stope_ou_explosex336:
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)(((0-1))*1)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx336:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(poussageerertertrteterertx336));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx336));
poussageerertertrteterertx336:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(JNE(poussagetantpisx336));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(JE(poussagefait_pas_peterx336));
poussagetantpisx336:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(JMP(poussagestooppppesx336));
poussagefait_pas_peterx336:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
poussagestooppppesx336:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)0));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx550));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx550:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagepas_ca___x336));
poussagecontinue_lme_train_trainx336:
R(ADD(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)((0-1))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)(8*(((0-1))))));
R(JNE(poussagepas_changement_casex336));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)0));
R(JE(poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx336));
R(SUB(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)((0-1))));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*3))+2), ds)),16,(dw)0));
R(JMP(poussagestooppppesx336));
poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx336:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)(((0-8))*(((0-1))))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)0));
R(ADD(32,READDD(ebx),32,(dd)(((0-1))*1)));
R(ADD(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)(((0-1))*1)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)READDBl(eax)));
poussagepas_changement_casex336:
R(MOV(16,READDW(eax),16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,truc_x)+READDD(ebx))+(((13*16)*0))), ds)),8,(db)READDBl(eax)));
poussagepas_ca___x336:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds)),16,(dw)1));
R(JNE(poussagepas_ca___x337));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)0));
R(JNE(poussagecontinue_lme_train_trainx337));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((1*32))), ds)),8,(db)66));
R(JNE(poussagereterertertertertterertertx337));
R(NEG(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(AND(32,READDD(ebx),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(ebx)), ds))));
R(MOV(8,READDBl(eax),8,(db)3));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2bopx551));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2bopx551:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)45));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagecontinue_lme_train_trainx337));
poussagereterertertertertterertertx337:
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((1*32))), ds)),8,(db)0));
R(JNE(poussagestooppppesx337));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(ebx)), ds)),8,(db)1));
R(JE(poussagestooppppesx337));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*32))), ds)),8,(db)0));
R(JE(poussagestooppppes_pasx337));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*32))), ds)),8,(db)5));
R(JB(poussageya_une_bombe_stope_ou_explosex337));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*32))), ds)),8,(db)54));
R(JA(poussagestooppppesx337));
poussagestooppppes_pasx337:
R(JMP(poussagecontinue_lme_train_trainx337));
poussageya_une_bombe_stope_ou_explosex337:
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)(1*32)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx337:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(poussageerertertrteterertx337));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx337));
poussageerertertrteterertx337:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(JNE(poussagetantpisx337));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(JE(poussagefait_pas_peterx337));
poussagetantpisx337:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(JMP(poussagestooppppesx337));
poussagefait_pas_peterx337:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
poussagestooppppesx337:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)0));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx552));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx552:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagepas_ca___x337));
poussagecontinue_lme_train_trainx337:
R(ADD(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)1));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)(8*(1))));
R(JNE(poussagepas_changement_casex337));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*32))), ds)),8,(db)0));
R(JE(poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx337));
R(SUB(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)1));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*3))+2), ds)),16,(dw)0));
R(JMP(poussagestooppppesx337));
poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx337:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)(((0-8))*(1))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)0));
R(ADD(32,READDD(ebx),32,(dd)(1*32)));
R(ADD(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)(1*32)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)READDBl(eax)));
poussagepas_changement_casex337:
R(MOV(16,READDW(eax),16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,truc_x)+READDD(ebx))+(((13*16)*2))), ds)),8,(db)READDBl(eax)));
poussagepas_ca___x337:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds)),16,(dw)((0-1))));
R(JNE(poussagepas_ca___x338));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)0));
R(JNE(poussagecontinue_lme_train_trainx338));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)66));
R(JNE(poussagereterertertertertterertertx338));
R(NEG(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(AND(32,READDD(ebx),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(ebx)), ds))));
R(MOV(8,READDBl(eax),8,(db)3));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2bopx553));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2bopx553:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)45));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagecontinue_lme_train_trainx338));
poussagereterertertertertterertertx338:
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)0));
R(JNE(poussagestooppppesx338));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(ebx)), ds)),8,(db)1));
R(JE(poussagestooppppesx338));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)0));
R(JE(poussagestooppppes_pasx338));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)5));
R(JB(poussageya_une_bombe_stope_ou_explosex338));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)54));
R(JA(poussagestooppppesx338));
poussagestooppppes_pasx338:
R(JMP(poussagecontinue_lme_train_trainx338));
poussageya_une_bombe_stope_ou_explosex338:
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)(((0-1))*32)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx338:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(poussageerertertrteterertx338));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx338));
poussageerertertrteterertx338:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(JNE(poussagetantpisx338));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(JE(poussagefait_pas_peterx338));
poussagetantpisx338:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(JMP(poussagestooppppesx338));
poussagefait_pas_peterx338:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
poussagestooppppesx338:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)0));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx554));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx554:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagepas_ca___x338));
poussagecontinue_lme_train_trainx338:
R(ADD(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)((0-1))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)(8*(((0-1))))));
R(JNE(poussagepas_changement_casex338));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)0));
R(JE(poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx338));
R(SUB(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)((0-1))));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*3))+2), ds)),16,(dw)0));
R(JMP(poussagestooppppesx338));
poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx338:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)(((0-8))*(((0-1))))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)0));
R(ADD(32,READDD(ebx),32,(dd)(((0-1))*32)));
R(ADD(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)(((0-1))*32)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)READDBl(eax)));
poussagepas_changement_casex338:
R(MOV(16,READDW(eax),16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,truc_x)+READDD(ebx))+(((13*16)*2))), ds)),8,(db)READDBl(eax)));
poussagepas_ca___x338:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds)),16,(dw)1));
R(JNE(poussagepas_ca___x339));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)0));
R(JNE(poussagecontinue_lme_train_trainx339));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((1*1))), ds)),8,(db)66));
R(JNE(poussagereterertertertertterertertx339));
R(NEG(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(AND(32,READDD(ebx),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(ebx)), ds))));
R(MOV(8,READDBl(eax),8,(db)3));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2bopx555));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2bopx555:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)45));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagecontinue_lme_train_trainx339));
poussagereterertertertertterertertx339:
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((1*1))), ds)),8,(db)0));
R(JNE(poussagestooppppesx339));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(ebx)), ds)),8,(db)1));
R(JE(poussagestooppppesx339));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*1))), ds)),8,(db)0));
R(JE(poussagestooppppes_pasx339));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*1))), ds)),8,(db)5));
R(JB(poussageya_une_bombe_stope_ou_explosex339));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*1))), ds)),8,(db)54));
R(JA(poussagestooppppesx339));
poussagestooppppes_pasx339:
R(JMP(poussagecontinue_lme_train_trainx339));
poussageya_une_bombe_stope_ou_explosex339:
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)(1*1)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx339:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(poussageerertertrteterertx339));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx339));
poussageerertertrteterertx339:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(JNE(poussagetantpisx339));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(JE(poussagefait_pas_peterx339));
poussagetantpisx339:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(JMP(poussagestooppppesx339));
poussagefait_pas_peterx339:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
poussagestooppppesx339:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)0));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx556));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx556:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagepas_ca___x339));
poussagecontinue_lme_train_trainx339:
R(ADD(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)1));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)(8*(1))));
R(JNE(poussagepas_changement_casex339));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*1))), ds)),8,(db)0));
R(JE(poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx339));
R(SUB(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)1));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*3))+2), ds)),16,(dw)0));
R(JMP(poussagestooppppesx339));
poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx339:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)(((0-8))*(1))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)0));
R(ADD(32,READDD(ebx),32,(dd)(1*1)));
R(ADD(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)(1*1)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)READDBl(eax)));
poussagepas_changement_casex339:
R(MOV(16,READDW(eax),16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,truc_x)+READDD(ebx))+(((13*16)*0))), ds)),8,(db)READDBl(eax)));
poussagepas_ca___x339:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds)),16,(dw)((0-1))));
R(JNE(poussagepas_ca___x340));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)0));
R(JNE(poussagecontinue_lme_train_trainx340));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)66));
R(JNE(poussagereterertertertertterertertx340));
R(NEG(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(AND(32,READDD(ebx),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(ebx)), ds))));
R(MOV(8,READDBl(eax),8,(db)3));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2bopx557));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2bopx557:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)45));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagecontinue_lme_train_trainx340));
poussagereterertertertertterertertx340:
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)0));
R(JNE(poussagestooppppesx340));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(ebx)), ds)),8,(db)1));
R(JE(poussagestooppppesx340));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)0));
R(JE(poussagestooppppes_pasx340));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)5));
R(JB(poussageya_une_bombe_stope_ou_explosex340));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)54));
R(JA(poussagestooppppesx340));
poussagestooppppes_pasx340:
R(JMP(poussagecontinue_lme_train_trainx340));
poussageya_une_bombe_stope_ou_explosex340:
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)(((0-1))*1)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx340:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(poussageerertertrteterertx340));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx340));
poussageerertertrteterertx340:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(JNE(poussagetantpisx340));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(JE(poussagefait_pas_peterx340));
poussagetantpisx340:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(JMP(poussagestooppppesx340));
poussagefait_pas_peterx340:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
poussagestooppppesx340:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+0), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)0));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx558));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx558:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagepas_ca___x340));
poussagecontinue_lme_train_trainx340:
R(ADD(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)((0-1))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)(8*(((0-1))))));
R(JNE(poussagepas_changement_casex340));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*1))), ds)),8,(db)0));
R(JE(poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx340));
R(SUB(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)((0-1))));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*3))+2), ds)),16,(dw)0));
R(JMP(poussagestooppppesx340));
poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx340:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds)),16,(dw)(((0-8))*(((0-1))))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)0));
R(ADD(32,READDD(ebx),32,(dd)(((0-1))*1)));
R(ADD(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)(((0-1))*1)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)READDBl(eax)));
poussagepas_changement_casex340:
R(MOV(16,READDW(eax),16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+0), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,truc_x)+READDD(ebx))+(((13*16)*0))), ds)),8,(db)READDBl(eax)));
poussagepas_ca___x340:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds)),16,(dw)1));
R(JNE(poussagepas_ca___x341));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)0));
R(JNE(poussagecontinue_lme_train_trainx341));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((1*32))), ds)),8,(db)66));
R(JNE(poussagereterertertertertterertertx341));
R(NEG(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(AND(32,READDD(ebx),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(ebx)), ds))));
R(MOV(8,READDBl(eax),8,(db)3));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2bopx559));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2bopx559:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)45));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagecontinue_lme_train_trainx341));
poussagereterertertertertterertertx341:
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((1*32))), ds)),8,(db)0));
R(JNE(poussagestooppppesx341));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(ebx)), ds)),8,(db)1));
R(JE(poussagestooppppesx341));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*32))), ds)),8,(db)0));
R(JE(poussagestooppppes_pasx341));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*32))), ds)),8,(db)5));
R(JB(poussageya_une_bombe_stope_ou_explosex341));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*32))), ds)),8,(db)54));
R(JA(poussagestooppppesx341));
poussagestooppppes_pasx341:
R(JMP(poussagecontinue_lme_train_trainx341));
poussageya_une_bombe_stope_ou_explosex341:
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)(1*32)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx341:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(poussageerertertrteterertx341));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx341));
poussageerertertrteterertx341:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(JNE(poussagetantpisx341));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(JE(poussagefait_pas_peterx341));
poussagetantpisx341:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(JMP(poussagestooppppesx341));
poussagefait_pas_peterx341:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
poussagestooppppesx341:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)0));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx560));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx560:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagepas_ca___x341));
poussagecontinue_lme_train_trainx341:
R(ADD(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)1));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)(8*(1))));
R(JNE(poussagepas_changement_casex341));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((1*32))), ds)),8,(db)0));
R(JE(poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx341));
R(SUB(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)1));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*3))+2), ds)),16,(dw)0));
R(JMP(poussagestooppppesx341));
poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx341:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)(((0-8))*(1))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)0));
R(ADD(32,READDD(ebx),32,(dd)(1*32)));
R(ADD(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)(1*32)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)READDBl(eax)));
poussagepas_changement_casex341:
R(MOV(16,READDW(eax),16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,truc_x)+READDD(ebx))+(((13*16)*2))), ds)),8,(db)READDBl(eax)));
poussagepas_ca___x341:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
R(PUSH(32,(READDD(ebx))));
R(PUSH(32,(READDD(eax))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds)),16,(dw)((0-1))));
R(JNE(poussagepas_ca___x342));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)0));
R(JNE(poussagecontinue_lme_train_trainx342));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)66));
R(JNE(poussagereterertertertertterertertx342));
R(NEG(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds))));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(AND(32,READDD(ebx),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(ebx)), ds))));
R(MOV(8,READDBl(eax),8,(db)3));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2bopx561));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2bopx561:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)45));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagecontinue_lme_train_trainx342));
poussagereterertertertertterertertx342:
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)0));
R(JNE(poussagestooppppesx342));
R(CMP(8,*((db *) realAddress((offsetof(struct Mem,truc_monstre)+READDD(ebx)), ds)),8,(db)1));
R(JE(poussagestooppppesx342));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)0));
R(JE(poussagestooppppes_pasx342));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)5));
R(JB(poussageya_une_bombe_stope_ou_explosex342));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)54));
R(JA(poussagestooppppesx342));
poussagestooppppes_pasx342:
R(JMP(poussagecontinue_lme_train_trainx342));
poussageya_une_bombe_stope_ou_explosex342:
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(32,READDD(esi),32,(dd)READDD(ebx)));
R(ADD(32,READDD(esi),32,(dd)(((0-1))*32)));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx342:
R(CMP(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(2*4)), ds)),32,(dd)READDD(esi)));
R(JE(poussageerertertrteterertx342));
R(ADD(32,READDD(ebx),32,(dd)taille_dune_info_bombe));
R(JMP(poussagettyrrtyrtyrtyrtytyrrtyrtyyrtrtyx342));
poussageerertertrteterertx342:
R(CMP(16,read_dw(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4)), ds)),16,(dw)0));
R(JNE(poussagetantpisx342));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*4))+2), ds)),16,(dw)0));
R(JE(poussagefait_pas_peterx342));
poussagetantpisx342:
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebx))+4)+(4*3))+2), ds)),16,(dw)0));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(JMP(poussagestooppppesx342));
poussagefait_pas_peterx342:
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
poussagestooppppesx342:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*4))+2), ds)),16,(dw)0));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(PUSH(32,(READDD(esi))));
R(PUSH(32,(READDD(ebx))));
R(MOV(8,READDBl(eax),8,(db)0));
R(SUB(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,truc2)))));
R(AND(32,READDD(esi),32,(dd)31));
R(MOV(8,READDBl(ebx),8,*((db *) realAddress((offsetof(struct Mem,panning2)+READDD(esi)), ds))));
R(SHL(32,READDD(ebx),32,(dd)4));
R(OR(8,READDBl(eax),8,(db)READDBl(ebx)));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit2opx562));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit2opx562:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)2));
R(ADD(32,READDD(eax),32,(dd)40));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(ebx))));
R(POP(32,(READDD(esi))));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(JMP(poussagepas_ca___x342));
poussagecontinue_lme_train_trainx342:
R(ADD(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)((0-1))));
R(CMP(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)(8*(((0-1))))));
R(JNE(poussagepas_changement_casex342));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,truc2)+READDD(ebx))+((((0-1))*32))), ds)),8,(db)0));
R(JE(poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx342));
R(SUB(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)((0-1))));
R(MOV(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(1*4)), ds)),32,(dd)1));
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(4*3))+2), ds)),16,(dw)0));
R(JMP(poussagestooppppesx342));
poussagenan_pas_de_bombe_ayant_squatte_entre_tempsx342:
R(MOV(16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds)),16,(dw)(((0-8))*(((0-1))))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)0));
R(ADD(32,READDD(ebx),32,(dd)(((0-1))*32)));
R(ADD(32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds)),32,(dd)(((0-1))*32)));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc2)+READDD(ebx)), ds)),8,(db)READDBl(eax)));
poussagepas_changement_casex342:
R(MOV(16,READDW(eax),16,read_dw(realAddress(((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(5*4))+2), ds))));
R(MOV(32,READDD(ebx),32,read_dd(realAddress((((offsetof(struct Mem,liste_bombe)+READDD(ebp))+4)+(2*4)), ds))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_x)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,truc_y)+READDD(ebx)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,truc_x)+READDD(ebx))+(((13*16)*2))), ds)),8,(db)READDBl(eax)));
poussagepas_ca___x342:
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebx))));
RET;

//PROC compact
compact:
RET;

//PROC decompact
decompact:
RET;

//PROC zget_information
zget_information:
RET;

//PROC ferme_socket
ferme_socket:
RET;

_main:
R(STI);
R(CLD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,ttp), ds))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,temps_avant_demo), ds)),32,(dd)READDD(eax)));
CALL(doStuffClavierExtended);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
CALL(inst_clavier);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(MOV(32,READDD(eax),32,(dd)taille_moire));
R(MOV(16,READDW(ecx),16,(dw)READDW(eax)));
R(SHR(32,READDD(eax),32,(dd)16));
R(MOV(16,READDW(ebx),16,(dw)READDW(eax)));
R(MOV(16,READDW(eax),16,(dw)1281));
R(INT(49));
R(JNC(ca_roule_roll));
nanananaaaaaaaaaaaa_demande_ligne_c:
R(JMP(okokokokokoioioiioio));
ca_roule_roll:
R(PUSH(16,(READDW(ebx))));
R(PUSH(16,(READDW(ecx))));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(MOV(16,READDW(ecx),16,(dw)1));
R(INT(49));
R(JNC(ca_roule_roll2));
R(MOV(16,READDW(eax),16,(dw)19456));
R(INT(33));
ca_roule_roll2:
R(POP(16,(READDW(edx))));
R(POP(16,(READDW(ecx))));
R(MOV(16,READDW(ebx),16,(dw)READDW(eax)));
R(MOV(16,READDW(fs),16,(dw)READDW(eax)));
R(MOV(16,READDW(eax),16,(dw)7));
R(INT(49));
R(MOV(32,READDD(eax),32,(dd)taille_moire));
R(MOV(16,READDW(edx),16,(dw)READDW(eax)));
R(SHR(32,READDD(eax),32,(dd)16));
R(MOV(16,READDW(ecx),16,(dw)READDW(eax)));
R(MOV(16,READDW(ebx),16,(dw)READDW(fs)));
R(MOV(16,READDW(eax),16,(dw)8));
R(INT(49));
R(JNC(tca_roule_roll));
R(MOV(16,READDW(eax),16,(dw)19456));
R(INT(33));
tca_roule_roll:
R(MOV(16,READDW(eax),16,(dw)2));
R(MOV(16,READDW(ebx),16,(dw)40960));
R(INT(49));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
okokokokokoioioiioio:
CALL(init_packed_liste);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,modeinfo), ds)),8,(db)1));
R(JNE(ertrerertrtrer));
CALL(zget_information);
ertrerertrtrer:
CALL(load_data);
R(MOV(16,READDW(eax),16,(dw)19));
R(INT(16));
CALL(affpal);
hoooooooop:
CALL(init_menu);
reterrterterte:
CALL(menu);
CALL(pal_visage);
CALL(controle);
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nb_ordy_connected), ds))));
R(INC(32,(READDD(ecx))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
touche_presseddretertertertertetrtrertertertx349:
R(CMP(8,*((db *) realAddress(((READDD(esi)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_pressedderterertertertx349));
R(PUSH(32,(READDD(eax))));
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,ttp), ds))));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,temps_avant_demo), ds)),32,(dd)READDD(eax)));
R(POP(32,(READDD(eax))));
touche_pressedderterertertertx349:
R(ADD(32,READDD(esi),32,(dd)64));
R(DEC(32,(READDD(ecx))));
R(JNZ(touche_presseddretertertertertetrtrertertertx349));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)2));
R(JNE(erterertrtertetertyutyuyuttyuuty));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_vbl_avant_le_droit_de_poser_bombe), ds)),32,(dd)0));
R(JNE(erterertrtertetertyutyuyuttyuuty));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nb_ordy_connected), ds))));
R(INC(32,(READDD(ecx))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
touche_presseretertertertertetrtrertertertx350:
R(CMP(8,*((db *) realAddress(((READDD(esi)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseerterertertertx350));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
touche_presseerterertertertx350:
R(ADD(32,READDD(esi),32,(dd)64));
R(DEC(32,(READDD(ecx))));
R(JNZ(touche_presseretertertertertetrtrertertertx350));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
erterertrtertetertyutyuyuttyuuty:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,nosetjmp), ds)),8,(db)1));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(MOV(16,READDW(edx),16,(dw)986));
vblavbl1x351:
IN(READDBl(eax),READDW(edx));
R(TEST(8,READDBl(eax),8,(db)8));
R(JNE(vblavbl1x351));
vblavbl2x351:
IN(READDBl(eax),READDW(edx));
R(TEST(8,READDBl(eax),8,(db)8));
R(JE(vblavbl2x351));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(XOR(32,READDD(edx),32,(dd)READDD(edx)));
directmenu:
CALL(menu_intelligence);
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,taille_exe_gonfle), ds)),32,(dd)0));
R(JE(get_all_infos3donoterasekeyslabelx352));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,donnee2)))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(MOV(32,READDD(ecx),32,(dd)touches_size));
R(REP_MOVSB);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
get_all_infos3donoterasekeyslabelx352:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,on_les_dans_le_menu), ds)),8,(db)1));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,sors_du_menu_aussitot), ds)),8,(db)1));
R(JNE(y_special_mrb));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
y_special_mrb:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
R(JNE(erertrterteertrteertertrtertertyeertrteertterertertterertterert));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)0));
erertrterteertrteertertrtertertyeertrteertterertertterertterert:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,attente_nouveau_esc), ds)),32,(dd)0));
R(JNE(ook));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JNE(oook2));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
R(JE(rtertertyeertrteertterertertterertterert));
R(JMP(oook2));
ook:
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,attente_nouveau_esc), ds))));
oook2:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre), ds)),8,(db)1));
R(JNE(reterrterterte));
nouvelle_partie345:
CALL(nouvelle_partie);
nouvelle_manche3:
CALL(nouvelle_manche);
retertdgrfgd:
CALL(pal_visage);
CALL(controle);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JNE(trtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,on_les_dans_le_menu), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,nosetjmp), ds)),8,(db)2));
R(INC(32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(MOV(16,READDW(edx),16,(dw)986));
vblavbl1x353:
IN(READDBl(eax),READDW(edx));
R(TEST(8,READDBl(eax),8,(db)8));
R(JNE(vblavbl1x353));
vblavbl2x353:
IN(READDBl(eax),READDW(edx));
R(TEST(8,READDBl(eax),8,(db)8));
R(JE(vblavbl2x353));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(XOR(32,READDD(edx),32,(dd)READDD(edx)));
directjeu:
CALL(master1);
trtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)77));
R(JE(hoooooooop));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
R(JE(nouvelle_manche3));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JE(ertyerttyrrtyyrtretertdgrfgd));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,sortie_slave), ds)),8,(db)0));
R(JNE(rtertertyeertrteertterertertterertterert));
R(JMP(retertdgrfgd));
ertyerttyrrtyyrtretertdgrfgd:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JNE(retertdgrfgd));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)1));
R(JNE(retertdgrfgd));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)77));
R(JMP(retertdgrfgd));
rtertertyeertrteertterertertterertterert:
R(MOV(16,READDW(eax),16,(dw)3));
R(INT(16));
rtertertyeertrteertterertertterertterert2:
busy_ou:
mechant5:
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,lost_conney), ds))));
R(MOV(8,READDBh(eax),8,(db)76));
R(INT(33));
quitte_baby:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(MOV(16,READDW(eax),16,(dw)3));
R(INT(16));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,lost_conney), ds)),8,(db)1));
R(MOV(8,READDBl(edx),8,(db)7));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(JMP(rtertertyeertrteertterertertterertterert2));
//PROC master1
master1:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)68));
R(JNE(trtyrtyrtyrtyrtyterterertrteert));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_draw), ds)),32,(dd)duree_draw2));
R(JNE(erertertteryr));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)1));
erertertteryr:
CALL(copie_le_fond_draw);
CALL(aff_page2);
R(JMP(rtyrtyrtytyrrtyyttyutyutyutyutyutyutyutyutyuyuttyu));
retetrterterertertrteertertertertre:
rtyrtyrtytyrrtyyttyutyutyutyutyutyutyutyutyuyuttyu:
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,duree_draw), ds))));
R(JNZ(reterertertert));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
reterertertert:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_draw), ds)),32,(dd)450));
R(JA(ertetrtrkjjklmkjlmetkjlmdikgrhrfhgrrethghkgh));
R(CMP(16,read_dw(realAddress(offsetof(struct Mem,bdraw666), ds)),16,(dw)14649));
R(JE(ertetrtrkjjklmkjlmetkjlmdikgrhrfhgrrethghkgh));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JNE(touche_presseque_mastererterertertertx355));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,total_t)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseque_mastererterertertertx355));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
touche_presseque_mastererterertertertx355:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
ertetrtrkjjklmkjlmetkjlmdikgrhrfhgrrethghkgh:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_draw), ds)),32,(dd)400));
R(JA(rerteertertertert3r0));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nb_ordy_connected), ds))));
R(INC(32,(READDD(ecx))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
touche_presseretertertertertetrtrertertertx356:
R(CMP(8,*((db *) realAddress(((READDD(esi)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseerterertertertx356));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
touche_presseerterertertertx356:
R(ADD(32,READDD(esi),32,(dd)64));
R(DEC(32,(READDD(ecx))));
R(JNZ(touche_presseretertertertertetrtrertertertx356));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
rerteertertertert3r0:
CALL(compact);
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,taille_exe_gonfle), ds)),32,(dd)0));
R(JE(get_all_infos2donoterasekeyslabelx357));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,donnee2)))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(MOV(32,READDD(ecx),32,(dd)touches_size));
R(REP_MOVSB);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
get_all_infos2donoterasekeyslabelx357:
R(JMP(exitfunction));
trtyrtyrtyrtyrtyterterertrteert:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)3));
R(JNE(terterertrteert));
CALL(gestion_jeu);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)90));
R(JE(meeeeeed));
CALL(compact);
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,taille_exe_gonfle), ds)),32,(dd)0));
R(JE(get_all_infos2donoterasekeyslabelx358));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,donnee2)))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(MOV(32,READDD(ecx),32,(dd)touches_size));
R(REP_MOVSB);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
get_all_infos2donoterasekeyslabelx358:
CALL(rec_play_touches);
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)3));
R(JNE(yttyutyutyutyutyutyutyutyutyuyuttyu));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,last_sucker), ds)),8,(db)0));
R(JNE(yttyutyutyutyutyutyutyutyutyuyuttyu));
CALL(copie_le_fond);
CALL(affiche_sprites);
CALL(aff_page2);
yttyutyutyutyutyutyutyutyutyuyuttyu:
R(JMP(exitfunction));
terterertrteert:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)86));
R(JNE(etrertertrtertertyerterttrtyrtyrtyrtyrtyterterertrteert));
victoire_sup:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_vic), ds)),32,(dd)duree_vic2));
R(JNE(kierertertteryr));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)1));
kierertertteryr:
CALL(copie_le_fond_vic);
CALL(aff_page2);
kiretetrterterertertrteertertertertre:
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,duree_vic), ds))));
R(JNZ(ireterertertertu));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)77));
ireterertertertu:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_vic), ds)),32,(dd)(duree_vic2-60)));
R(JA(rerteertertertert));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_vic), ds)),32,(dd)(duree_vic2-60)));
R(JA(rerteertertertertpopop));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JNE(touche_presseque_mastererterertertertx359));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,total_t)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseque_mastererterertertertx359));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)77));
touche_presseque_mastererterertertertx359:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
rerteertertertertpopop:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_vic), ds)),32,(dd)(duree_vic2/2)));
R(JA(rerteertertertert));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nb_ordy_connected), ds))));
R(INC(32,(READDD(ecx))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
touche_presseretertertertertetrtrertertertx360:
R(CMP(8,*((db *) realAddress(((READDD(esi)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseerterertertertx360));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)77));
touche_presseerterertertertx360:
R(ADD(32,READDD(esi),32,(dd)64));
R(DEC(32,(READDD(ecx))));
R(JNZ(touche_presseretertertertertetrtrertertertx360));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
rerteertertertert:
CALL(compact);
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,taille_exe_gonfle), ds)),32,(dd)0));
R(JE(get_all_infos2donoterasekeyslabelx361));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,donnee2)))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(MOV(32,READDD(ecx),32,(dd)touches_size));
R(REP_MOVSB);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
get_all_infos2donoterasekeyslabelx361:
R(JMP(exitfunction));
etrertertrtertertyerterttrtyrtyrtyrtyrtyterterertrteert:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)90));
R(JNE(rtyerterttrtyrtyrtyrtyrtyterterertrteertrtrtrtrtyyrtyooooooooooooo));
meeeeeed:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_med), ds)),32,(dd)duree_med2));
R(JNE(ierertertteryr));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)1));
ierertertteryr:
CALL(copie_le_fond_med);
CALL(aff_page2);
R(JMP(irtyrtyrtytyrrtyyttyutyutyutyutyutyutyutyutyuyuttyu));
iretetrterterertertrteertertertertre:
irtyrtyrtytyrrtyyttyutyutyutyutyutyutyutyutyuyuttyu:
pas_med:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(16,READDW(eax),16,(dw)READDW(ds)));
R(MOV(16,READDW(es),16,(dw)READDW(eax)));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,blow_what2)))));
R(MOV(32,READDD(ecx),32,(dd)14));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(REP_STOSW);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
R(MOV(32,READDD(eax),32,read_dd(realAddress(offsetof(struct Mem,changement), ds))));
R(AND(32,READDD(eax),32,(dd)63));
R(CMP(32,READDD(eax),32,(dd)32));
R(JNE(tout_));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,briques)+(8*4)), ds)),8,(db)1));
tout_:
R(OR(32,READDD(eax),32,(dd)READDD(eax)));
R(JNZ(tout_3));
R(PUSH(32,(READDD(ebp))));
R(PUSH(32,(READDD(eax))));
R(MOV(8,READDBl(eax),8,(db)3));
R(OR(8,READDBl(eax),8,(db)112));
R(MOV(32,READDD(ebp),32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds))));
R(ADD(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)2));
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)(14*2)));
R(JNE(bruit3opx363));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,last_voice), ds)),32,(dd)0));
bruit3opx363:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,blow_what2)+READDD(ebp)), ds)),8,(db)READDBl(eax)));
R(MOV(32,READDD(eax),32,(dd)35));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,blow_what2)+READDD(ebp))+1), ds)),8,(db)READDBl(eax)));
R(POP(32,(READDD(eax))));
R(POP(32,(READDD(ebp))));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,briques)+(8*4)), ds)),8,(db)0));
tout_3:
R(DEC(32,read_dd(realAddress(offsetof(struct Mem,duree_med), ds))));
R(JNZ(ireterertertert));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
ireterertertert:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_med), ds)),32,(dd)(duree_med2-(1*90))));
R(JA(rerteertertertertpopop2));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JNE(touche_presseque_mastererterertertertx364));
R(CMP(8,*((db *) realAddress(((offsetof(struct Mem,total_t)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseque_mastererterertertertx364));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
touche_presseque_mastererterertertertx364:
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
rerteertertertertpopop2:
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,duree_med), ds)),32,(dd)(duree_med2-(3*60))));
R(JA(okokokokokokok345345345));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(32,READDD(ecx),32,read_dd(realAddress(offsetof(struct Mem,nb_ordy_connected), ds))));
R(INC(32,(READDD(ecx))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
touche_presseretertertertertetrtrertertertx365:
R(CMP(8,*((db *) realAddress(((READDD(esi)+(7*8))+2), ds)),8,(db)1));
R(JNE(touche_presseerterertertertx365));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
touche_presseerterertertertx365:
R(ADD(32,READDD(esi),32,(dd)64));
R(DEC(32,(READDD(ecx))));
R(JNZ(touche_presseretertertertertetrtrertertertx365));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
okokokokokokok345345345:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)37));
R(JNE(iertrtrterteert));
R(XOR(32,READDD(ebx),32,(dd)READDD(ebx)));
ierterertertrtyetyutyutyuutytyutyu:
R(CMP(32,read_dd(realAddress((offsetof(struct Mem,victoires)+READDD(ebx)), ds)),32,(dd)5));
R(JNE(iertterteteertzerzerzerzerrteretr));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)86));
R(JMP(victoire_sup));
iertterteteertzerzerzerzerrteretr:
R(ADD(32,READDD(ebx),32,(dd)4));
R(CMP(32,READDD(ebx),32,(dd)(4*8)));
R(JNE(ierterertertrtyetyutyutyuutytyutyu));
iertrtrterteert:
rtyerterttrtyrtyrtyrtyrtyterterertrteertrtrtrtrtyyrtyooooooooooooo:
CALL(compact);
R(CMP(32,read_dd(realAddress(offsetof(struct Mem,taille_exe_gonfle), ds)),32,(dd)0));
R(JE(get_all_infos2donoterasekeyslabelx366));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,donnee2)))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(MOV(32,READDD(ecx),32,(dd)touches_size));
R(REP_MOVSB);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
get_all_infos2donoterasekeyslabelx366:
exitfunction:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,donnee2)+(8*7)), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee2)+(8*7))+1), ds)),8,(db)0));
R(MOV(8,*((db *) realAddress(((offsetof(struct Mem,donnee2)+(8*7))+2), ds)),8,(db)0));
RET;

//PROC doStuffClavierExtended
doStuffClavierExtended:
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+72), ds)),8,(db)72));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+80), ds)),8,(db)80));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+91), ds)),8,(db)91));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+92), ds)),8,(db)92));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+93), ds)),8,(db)93));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+29), ds)),8,(db)123));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+53), ds)),8,(db)116));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+79), ds)),8,(db)117));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+83), ds)),8,(db)118));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+82), ds)),8,(db)122));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+73), ds)),8,(db)120));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+71), ds)),8,(db)121));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+81), ds)),8,(db)119));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+56), ds)),8,(db)124));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+28), ds)),8,(db)125));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+55), ds)),8,(db)84));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+72), ds)),8,(db)112));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+80), ds)),8,(db)113));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+75), ds)),8,(db)114));
R(MOV(8,*((db *) realAddress((offsetof(struct Mem,clavier_extanded)+77), ds)),8,(db)115));
RET;

//PROC a_la_ligne
a_la_ligne:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(MOV(8,READDBl(edx),8,(db)10));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(MOV(8,READDBl(edx),8,(db)13));
R(MOV(8,READDBh(eax),8,(db)2));
R(INT(33));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;

//PROC init_menu
init_menu:
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,special_on_a_loadee_nivo), ds)),8,(db)1));
R(JNE(ertterterrtertytyrrtyrtyrtyrtyrtyrtyrty));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,temps_avant_demo), ds)),32,(dd)1));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,special_on_a_loadee_nivo), ds)),8,(db)2));
ertterterrtertytyrrtyrtyrtyrtyrtyrtyrty:
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,affiche_pal), ds)),8,(db)1));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,sortie), ds)),8,(db)0));
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,attente_nouveau_esc), ds)),32,(dd)20));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre), ds)),8,(db)83));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,ordre2), ds)),8,(db)3));
R(PUSH(16,(READDW(ds))));
R(POP(16,(READDW(es))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,master), ds)),8,(db)0));
R(JE(trtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2erte34fgh));
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;
trtyrtrtyrtyrtyrtyrtytyrrtyrtytyryrtrty2erte34fgh:
R(PUSH(16,(READDW(eax))));
R(MOV(8,READDBl(eax),8,*((db *) realAddress(offsetof(struct Mem,team3_sauve), ds))));
R(AND(8,READDBl(eax),8,(db)3));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)READDBl(eax)));
R(POP(16,(READDW(eax))));
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)0));
R(JNE(tetrtyertyrdfgdfggdffgdgdf0));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,n_team)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,team)))));
R(MOV(32,READDD(ecx),32,(dd)9));
R(REP_MOVSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
tetrtyertyrdfgdfggdffgdgdf0:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)2));
R(JNE(tetrtyertyrdfgdfggdffgdgdf));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,s_team)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,team)))));
R(MOV(32,READDD(ecx),32,(dd)9));
R(REP_MOVSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
tetrtyertyrdfgdfggdffgdgdf:
R(CMP(8,*((db *) realAddress(offsetof(struct Mem,team3), ds)),8,(db)1));
R(JNE(tetrtyertyrdfgdfggdffgdgdfe));
R(PUSHAD);
R(PUSH(16,(READDW(ds))));
R(PUSH(16,(READDW(es))));
R(LEA(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,c_team)))));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,team)))));
R(MOV(32,READDD(ecx),32,(dd)9));
R(REP_MOVSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
tetrtyertyrdfgdfggdffgdgdfe:
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_play)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)(64/4)));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,fx)))));
R(XOR(16,READDW(eax),16,(dw)READDW(eax)));
R(MOV(32,READDD(ecx),32,(dd)14));
R(REP_STOSW);
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,total_t)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)(((64/4))*8)));
R(REP_STOSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,name_joueur)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message1)))));
R(MOV(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,texte1)))));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message1)))));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message1)))));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message1)))));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message1)))));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message1)))));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message1)))));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(MOV(32,m.esi.dd.val,32,(((dd)offsetof(struct Mem,message1)))));
R(MOV(32,READDD(ecx),32,(dd)32));
R(REP_MOVSD);
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,control_joueur)))));
R(MOV(32,READDD(eax),32,(dd)(64*8)));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(MOV(32,read_dd(realAddress(offsetof(struct Mem,nombre_de_dyna), ds)),32,(dd)0));
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,nb_ai_bombermen), ds)),8,(db)0));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,temps_joueur)))));
R(MOV(32,READDD(eax),32,(dd)temps_re_menu));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(MOV(8,*((db *) realAddress(offsetof(struct Mem,action_replay), ds)),8,(db)0));
R(LEA(32,m.edi.dd.val,32,(((dd)offsetof(struct Mem,lapipipino)))));
R(XOR(32,READDD(eax),32,(dd)READDD(eax)));
R(MOV(32,READDD(ecx),32,(dd)8));
R(REP_STOSD);
R(POP(16,(READDW(es))));
R(POP(16,(READDW(ds))));
R(POPAD);
RET;


m.executionFinished = 1;
moveToBackGround:
return;
}
